@@ -22,4 +22,7 @@ void app_init(int argc, char **argv);
 void app_exit(void);
 int app_is_embedded(void);
 
+void app_splash_init(const gchar* name);
+void app_splash_done(void);
+
 #endif /* APP_PROCS_H */
