@@ -17,22 +17,23 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#  include "config.h"
+#  include <config.h>
 #endif
 
 #include <gnome.h>
 #include <bonobo.h>
-#ifdef USE_OAF
-#  include <liboaf/liboaf.h>
-#else
-#  include <libgnorba/gnorba.h>
-#endif
-
+#include <liboaf/liboaf.h>
 
 #include "display.h"
 #include "menus.h"
 #include "disp_callbacks.h"
 #include "app_procs.h"
+#include "interface.h"
+#include "load_save.h"
+
+#ifdef GNOME_PRINT
+#  include "render_gnomeprint.h"
+#endif
 
 typedef struct _EmbeddedDia EmbeddedDia;
 typedef struct _EmbeddedView EmbeddedView;
@@ -50,16 +51,13 @@ struct _EmbeddedView {
 };
 
 static BonoboGenericFactory *factory = NULL;
-static int running_objects = 0;
-
 
 static void
-dia_view_activate(BonoboView *view, gboolean activate,
-		  EmbeddedView *view_data)
+view_show_hide (EmbeddedView *view_data,
+		gboolean      activate)
 {
   DDisplay *ddisp = view_data->display;
 
-  bonobo_view_activate_notify(view, activate);
   if (activate) {
     toolbox_show();
 
@@ -69,6 +67,7 @@ dia_view_activate(BonoboView *view, gboolean activate,
     gtk_widget_show(ddisp->hsb);
     gtk_widget_show(ddisp->vsb);
     gtk_widget_show(ddisp->zoom_status->parent);
+    gtk_widget_show(ddisp->modified_status);
   } else {
     toolbox_hide();
 
@@ -78,9 +77,18 @@ dia_view_activate(BonoboView *view, gboolean activate,
     gtk_widget_hide(ddisp->hsb);
     gtk_widget_hide(ddisp->vsb);
     gtk_widget_hide(ddisp->zoom_status->parent);
+    gtk_widget_hide(ddisp->modified_status);
   }
 }
 
+static void
+dia_view_activate(BonoboView *view, gboolean activate,
+		  EmbeddedView *view_data)
+{
+  bonobo_view_activate_notify(view, activate);
+  view_show_hide(view_data,activate);
+}
+
 static void
 dia_view_display_destroy (GtkWidget *ddisp_shell, EmbeddedView *view_data)
 {
@@ -111,14 +119,6 @@ dia_embeddable_destroy(BonoboEmbeddable *embeddable,
 
   diagram_destroy(embedded_dia->diagram);
   g_free(embedded_dia); 
-
-  running_objects--;
-  if (running_objects > 0)
-    return;
-  
-  /* When last object has gone unref the factory & quit. */
-  bonobo_object_unref(BONOBO_OBJECT(factory));
-  gtk_main_quit();
 }
 
 static void
@@ -142,9 +142,6 @@ view_factory (BonoboEmbeddable *embeddable,
 {
   EmbeddedView *view_data;
   BonoboView  *view;
-  GtkWidget *menuitem;
-  GString *path;
-
   
   /*
    * Create the private view data.
@@ -154,22 +151,16 @@ view_factory (BonoboEmbeddable *embeddable,
   
   view_data->display = new_display(embedded_dia->diagram);
 
+  view_show_hide (view_data, FALSE);
+  
   gtk_signal_connect (GTK_OBJECT (view_data->display->shell), "destroy",
                       GTK_SIGNAL_FUNC (dia_view_display_destroy),
                       view_data);
-  
-  gtk_widget_hide(view_data->display->origin);
-  gtk_widget_hide(view_data->display->hrule);
-  gtk_widget_hide(view_data->display->vrule);
-  gtk_widget_hide(view_data->display->hsb);
-  gtk_widget_hide(view_data->display->vsb);
-  gtk_widget_hide(view_data->display->zoom_status->parent);
 
   view = bonobo_view_new (view_data->display->shell);
   view_data->view = view;
   gtk_object_set_data (GTK_OBJECT (view), "view_data", view_data);
   
-  
   gtk_signal_connect(GTK_OBJECT (view), "activate",
 		     GTK_SIGNAL_FUNC (dia_view_activate), view_data);
   
@@ -180,42 +171,120 @@ view_factory (BonoboEmbeddable *embeddable,
 		      GTK_SIGNAL_FUNC (dia_view_destroy), view_data);
 
 
-  /* Gross stuff: */
-  path = g_string_new("<Display>/");
-  g_string_append(path, _("File/New diagram"));
-  menuitem = menus_get_item_from_path(path->str);
+#if 0
+  menuitem = menus_get_item_from_path("<Display>/File/New diagram");
   gtk_widget_hide(menuitem);
-  g_string_truncate(path, 10);
-  g_string_append(path, _("File/Open..."));
-  menuitem = menus_get_item_from_path(path->str);
+  menuitem = menus_get_item_from_path("<Display>/File/Open...");
   gtk_widget_hide(menuitem);
-  g_string_truncate(path, 10);
-  g_string_append(path, _("File/Save As..."));
-  menuitem = menus_get_item_from_path(path->str);
+  menuitem = menus_get_item_from_path("<Display>/File/Save As...");
   gtk_widget_hide(menuitem);
-  g_string_truncate(path, 10);
-  g_string_append(path, _("File/Close"));
-  menuitem = menus_get_item_from_path(path->str);
+  menuitem = menus_get_item_from_path("<Display>/File/Close");
   gtk_widget_hide(menuitem);
-  g_string_truncate(path, 10);
-  g_string_append(path, _("File/Exit"));
-  menuitem = menus_get_item_from_path(path->str);
+  menuitem = menus_get_item_from_path("<Display>/File/Exit");
   gtk_widget_hide(menuitem);
-  g_string_truncate(path, 10);
-  g_string_append(path, _("View/New View"));
-  menuitem = menus_get_item_from_path(path->str);
+  menuitem = menus_get_item_from_path("<Display>/View/New View");
   gtk_widget_hide(menuitem);
-  g_string_free(path, TRUE);
+#endif
   
   return view;
 }
 
+static int
+save_fn (BonoboPersistFile *pf,
+	 const CORBA_char  *filename,
+	 CORBA_Environment *ev,
+	 void              *closure)
+{
+  EmbeddedDia *dia = closure;
+
+  if (!diagram_save (dia->diagram, filename))
+    CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
+			 ex_Bonobo_Storage_IOError, NULL);
+  return 0;
+}
+
+static void 
+refresh_view (DDisplay *ddisp)
+{
+  Point        middle;
+  Rectangle   *visible;
+
+  visible = &ddisp->visible;
+  middle.x = visible->left*0.5 + visible->right*0.5;
+  middle.y = visible->top*0.5 + visible->bottom*0.5;
+  
+  ddisplay_zoom(ddisp, &middle, 1.0);
+  gtk_widget_queue_draw(ddisp->shell);
+}
+
+static int
+load_fn (BonoboPersistFile *pf,
+	 const CORBA_char  *filename,
+	 CORBA_Environment *ev,
+	 void              *closure)
+{
+  EmbeddedDia *dia = closure;
+
+  if (!diagram_load_into (dia->diagram, filename, NULL)) {
+	  g_warning ("Failed to load '%s'", filename);
+	  CORBA_exception_set (ev, CORBA_USER_EXCEPTION,
+			       ex_Bonobo_Persist_WrongDataType,
+			       NULL);
+  } else {
+	  GSList *l;
+	  diagram_update_extents(dia->diagram);
+	  for (l=dia->diagram->displays;l;l=l->next)
+		  refresh_view (l->data);
+  }
+  return 0;
+}
+
+#ifdef GNOME_PRINT
+static void
+object_print (GnomePrintContext         *ctx,
+	      double                     width,
+	      double                     height,
+	      const Bonobo_PrintScissor *scissor,
+	      gpointer                   user_data)
+{
+  RendererGPrint *rend;
+  EmbeddedDia    *edia = user_data;
+  Diagram        *dia = edia->diagram;
+  Rectangle      *extents;
+  double          scalex, scaley;
+
+  rend = new_gnomeprint_renderer(dia, ctx);
+
+  /*
+   * FIXME: we need to de-complicate all this - this should
+   * probably be done by scaling the view to the extent or
+   * alternatively keeping the visible bounds on the embeddable
+   * not the view; or possibly none of these.
+   */
+  extents = &dia->data->extents;
+
+  scalex = width / (extents->right - extents->left);
+  scaley = -height / (extents->bottom - extents->top);
+
+  gnome_print_translate (ctx, - scalex* extents->left,
+			 height - scaley * extents->top);
+  gnome_print_scale     (ctx, scalex, scaley);
+
+
+  data_render(dia->data, (Renderer *)rend, extents, NULL, NULL);
+
+  g_free (rend);
+}
+#endif
+
 static BonoboObject *
 embeddable_factory (BonoboGenericFactory *this,
 		    void *data)
 {
-  BonoboEmbeddable *embeddable;
-  EmbeddedDia *embedded_dia;
+  BonoboPersistFile *pfile;
+  BonoboEmbeddable  *embeddable;
+  BonoboPrint       *print;
+  EmbeddedDia       *embedded_dia;
 
   embedded_dia = g_new0 (EmbeddedDia, 1);
   if (embedded_dia == NULL)
@@ -231,7 +300,6 @@ embeddable_factory (BonoboGenericFactory *this,
     return NULL;
   }
 	
-  running_objects++;
   embedded_dia->embeddable = embeddable;
 
   gtk_signal_connect(GTK_OBJECT(embeddable), "system_exception",
@@ -242,44 +310,47 @@ embeddable_factory (BonoboGenericFactory *this,
 		     GTK_SIGNAL_FUNC (dia_embeddable_destroy),
 		     embedded_dia);
 
+  /* Register the Bonobo::PersistFile interface. */
+  pfile = bonobo_persist_file_new (load_fn, save_fn, embedded_dia);
+  bonobo_object_add_interface (
+	  BONOBO_OBJECT (embeddable),
+	  BONOBO_OBJECT (pfile));
+
+#ifdef GNOME_PRINT
+  /* Register the Bonobo::Print interface */
+  print = bonobo_print_new (object_print, embedded_dia);
+  if (!print) {
+	  bonobo_object_unref (BONOBO_OBJECT (embeddable));
+	  return NULL;
+  }
+  
+  bonobo_object_add_interface (BONOBO_OBJECT (embeddable),
+			       BONOBO_OBJECT (print));
+#endif
+
   return BONOBO_OBJECT (embeddable);
 }
 
 static BonoboGenericFactory *
 init_dia_factory (void)
 {
-#ifdef USE_OAF
-  return bonobo_generic_factory_new ("OAFIID:dia-diagram-factory:31cb6c0f-e1f5-4c54-8824-8f49fd7e50e7",
+  return bonobo_generic_factory_new ("OAFIID:GNOME_Dia_DiagramFactory",
 				     embeddable_factory, NULL);
-#else
-  return bonobo_generic_factory_new ("embeddable-factory:dia-diagram",
-				     embeddable_factory, NULL);
-#endif
 }
 
 static void
 init_server_factory (int argc, char **argv)
 {
-  CORBA_Environment ev;
   CORBA_ORB orb;
   
-#ifdef USE_OAF  
-  gnome_init_with_popt_table("bonobo-dia-diagram", VERSION,
-			     argc, argv, oaf_popt_options, 0, NULL);
-  orb = oaf_init(argc, argv);
-#else
-  CORBA_exception_init (&ev);
+  gnome_init_with_popt_table ("dia-embedd", VERSION,
+			      argc, argv, oaf_popt_options, 0, NULL);
 
-  gnome_CORBA_init_with_popt_table ("bonobo-dia-diagram", VERSION,
-				    &argc, argv, NULL, 0, NULL, GNORBA_INIT_SERVER_FUNC, &ev);
-  
-  CORBA_exception_free (&ev);
-  
-  orb = gnome_CORBA_ORB ();
-#endif
+  orb = oaf_init (argc, argv);
 
   if (bonobo_init (orb, NULL, NULL) == FALSE)
     g_error (_("Could not initialize Bonobo!"));
+  bonobo_activate ();
 }
 
 int
@@ -288,6 +359,14 @@ app_is_embedded(void)
   return 1;
 }
 
+static void
+last_unref_cb (BonoboObject *bonobo_object,
+	       gpointer      dummy)
+{
+  bonobo_object_unref (BONOBO_OBJECT (factory));
+  gtk_main_quit ();
+}
+
 int
 main (int argc, char **argv)
 {
@@ -297,6 +376,7 @@ main (int argc, char **argv)
   factory = init_dia_factory ();
 
   app_init(0, NULL);
+  app_splash_done();
   
 #ifdef GNOME
   menuitem = menus_get_item_from_path("<Toolbox>/File/New diagram");
@@ -316,6 +396,11 @@ main (int argc, char **argv)
   gtk_widget_hide(menuitem);
 #endif
   
+
+  gtk_signal_connect (GTK_OBJECT (bonobo_context_running_get ()),
+		      "last_unref",
+		      GTK_SIGNAL_FUNC (last_unref_cb),
+		      NULL);
   /*
    * Start processing.
    */
