@@ -15,10 +15,12 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+#include <config.h>
+
 #include <assert.h>
 #include <string.h>
 
-#include "config.h"
 #include "intl.h"
 #include "diagram.h"
 #include "group.h"
@@ -34,50 +36,82 @@
 #include "app_procs.h"
 #include "dia_dirs.h"
 #include "load_save.h"
+#include "recent_files.h"
 
 GList *open_diagrams = NULL;
 
+static void
+diagram_init(Diagram *dia, const char *filename)
+{
+  if(dia->data)
+    diagram_data_destroy(dia->data);
+  
+  dia->data = new_diagram_data();
+  dia->data->grid.width_x = prefs.grid.x;
+  dia->data->grid.width_y = prefs.grid.y;
+
+  g_free(dia->filename);
+  dia->filename = g_strdup(filename);
+  
+  dia->unsaved = TRUE;
+  dia->modified = FALSE;
+
+  if (dia->undo)
+    undo_destroy(dia->undo);
+  dia->undo = new_undo_stack(dia);
+
+  if (!g_list_find(open_diagrams, dia))
+    open_diagrams = g_list_prepend(open_diagrams, dia);
+
+  layer_dialog_update_diagram_list();
+}
+
+int
+diagram_load_into(Diagram         *diagram,
+		  const char      *filename,
+		  DiaImportFilter *ifilter)
+{
+  if (!ifilter)
+    ifilter = filter_guess_import_filter(filename);
+  if (!ifilter)  /* default to native format */
+    ifilter = &dia_import_filter;
+
+  diagram_init(diagram, filename);
+
+  if (ifilter->import(filename, diagram->data, ifilter->user_data)) {
+    diagram->unsaved = FALSE;
+    diagram_set_modified(diagram, FALSE);
+    recent_file_history_add(filename, ifilter);
+    return TRUE;
+  } else
+    return FALSE;
+}
+
 Diagram *
 diagram_load(const char *filename, DiaImportFilter *ifilter)
 {
   Diagram *diagram;
 
-  if (!ifilter)
-    ifilter = filter_guess_import_filter(filename);
-  if (!ifilter)  /* default to native format */
-    ifilter = &dia_import_filter;
   diagram = new_diagram(filename);
-  if (ifilter->import(filename, diagram->data)) {
-    diagram->unsaved = FALSE;
-    diagram_set_modified(diagram, FALSE);
-  } else {
+
+  if (!diagram_load_into (diagram, filename, ifilter)) {
     diagram_destroy(diagram);
     diagram = NULL;
   }
+
   return diagram;
 }
 
 Diagram *
 new_diagram(const char *filename)  /* Note: filename is copied */
 {
-  Diagram *dia = g_new(Diagram, 1);
-  
-  dia->data = new_diagram_data();
-  dia->data->grid.width_x = prefs.grid.x;
-  dia->data->grid.width_y = prefs.grid.y;
-
-  dia->filename = g_strdup(filename);
-  
-  dia->unsaved = TRUE;
-  dia->modified = FALSE;
+  Diagram *dia = g_new0(Diagram, 1);
 
   dia->display_count = 0;
   dia->displays = NULL;
 
-  dia->undo = new_undo_stack(dia);
-  
-  open_diagrams = g_list_prepend(open_diagrams, dia);
-  layer_dialog_update_diagram_list();
+  diagram_init(dia, filename);
+
   return dia;
 }
 
@@ -168,64 +202,50 @@ void diagram_update_menu_sensitivity(Diagram *dia)
   static GtkWidget *align_v_b;
   static GtkWidget *align_v_e;
   static GtkWidget *align_v_a;
-  static GString *path;
-  char *display = "<Display>";
   
   if (initialized==0) {
 #   ifdef GNOME
     if (ddisplay_active () == NULL) return;
 #   endif
-    path = g_string_new ("<Display>");
-    g_string_append (path,_("/Edit/Copy"));
-    copy = menus_get_item_from_path(path->str);
-    g_string_append (g_string_assign(path, display),_("/Edit/Cut"));
-    cut = menus_get_item_from_path(path->str);
-    g_string_append (g_string_assign(path, display),_("/Edit/Paste"));
-    paste = menus_get_item_from_path(path->str);
+    copy = menus_get_item_from_path("<Display>/Edit/Copy");
+    cut = menus_get_item_from_path("<Display>/Edit/Cut");
+    paste = menus_get_item_from_path("<Display>/Edit/Paste");
 #   ifndef GNOME
-    g_string_append (g_string_assign(path, display),_("/Edit/Delete"));
-    delete = menus_get_item_from_path(path->str);
+    delete = menus_get_item_from_path("<Display>/Edit/Delete");
 #   endif
 
-    g_string_append (g_string_assign(path, display),_("/Edit/Copy Text"));
-    copy_text = menus_get_item_from_path(path->str);
-    g_string_append (g_string_assign(path, display),_("/Edit/Cut Text"));
-    cut_text = menus_get_item_from_path(path->str);
-    g_string_append (g_string_assign(path, display),_("/Edit/Paste Text"));
-    paste_text = menus_get_item_from_path(path->str);
+    copy_text = menus_get_item_from_path("<Display>/Edit/Copy Text");
+    cut_text = menus_get_item_from_path("<Display>/Edit/Cut Text");
+    paste_text = menus_get_item_from_path("<Display>/Edit/Paste Text");
 
-    g_string_append (g_string_assign(path, display),_("/Objects/Send to Back"));
-    send_to_back = menus_get_item_from_path(path->str);
-    g_string_append (g_string_assign(path, display),_("/Objects/Bring to Front"));
-    bring_to_front = menus_get_item_from_path(path->str);
-  
-    g_string_append (g_string_assign(path, display),_("/Objects/Group"));
-    group = menus_get_item_from_path(path->str);
-    g_string_append (g_string_assign(path, display),_("/Objects/Ungroup"));
-    ungroup = menus_get_item_from_path(path->str);
-
-    g_string_append (g_string_assign(path, display),_("/Objects/Align Horizontal/Left"));
-    align_h_l = menus_get_item_from_path(path->str);
-    g_string_append (g_string_assign(path, display),_("/Objects/Align Horizontal/Center"));
-    align_h_c = menus_get_item_from_path(path->str);
-    g_string_append (g_string_assign(path, display),_("/Objects/Align Horizontal/Right"));
-    align_h_r = menus_get_item_from_path(path->str);
-    g_string_append (g_string_assign(path, display),_("/Objects/Align Horizontal/Equal Distance"));
-    align_h_e = menus_get_item_from_path(path->str);
-    g_string_append (g_string_assign(path, display),_("/Objects/Align Horizontal/Adjacent"));
-    align_h_a = menus_get_item_from_path(path->str);
-    g_string_append (g_string_assign(path, display),_("/Objects/Align Vertical/Top"));
-    align_v_t = menus_get_item_from_path(path->str);
-    g_string_append (g_string_assign(path, display),_("/Objects/Align Vertical/Center"));
-    align_v_c = menus_get_item_from_path(path->str);
-    g_string_append (g_string_assign(path, display),_("/Objects/Align Vertical/Bottom"));
-    align_v_b = menus_get_item_from_path(path->str);
-    g_string_append (g_string_assign(path, display),_("/Objects/Align Vertical/Equal Distance"));
-    align_v_e = menus_get_item_from_path(path->str);
-    g_string_append (g_string_assign(path, display),_("/Objects/Align Vertical/Adjacent"));
-    align_v_a = menus_get_item_from_path(path->str);
-
-    g_string_free (path,FALSE);
+    send_to_back = menus_get_item_from_path("<Display>/Objects/Send to Back");
+    bring_to_front =
+      menus_get_item_from_path("<Display>/Objects/Bring to Front");
+    group = menus_get_item_from_path("<Display>/Objects/Group");
+    ungroup = menus_get_item_from_path("<Display>/Objects/Ungroup");
+    align_h_l =
+      menus_get_item_from_path("<Display>/Objects/Align Horizontal/Left");
+    align_h_c =
+      menus_get_item_from_path("<Display>/Objects/Align Horizontal/Center");
+    align_h_r =
+      menus_get_item_from_path("<Display>/Objects/Align Horizontal/Right");
+    align_h_e =
+      menus_get_item_from_path(
+	  "<Display>/Objects/Align Horizontal/Equal Distance");
+    align_h_a =
+      menus_get_item_from_path("<Display>/Objects/Align Horizontal/Adjacent");
+    align_v_t =
+      menus_get_item_from_path("<Display>/Objects/Align Vertical/Top");
+    align_v_c =
+      menus_get_item_from_path("<Display>/Objects/Align Vertical/Center");
+    align_v_b =
+      menus_get_item_from_path("<Display>/Objects/Align Vertical/Bottom");
+    align_v_e =
+      menus_get_item_from_path(
+	  "<Display>/Objects/Align Vertical/Equal Distance");
+    align_v_a =
+      menus_get_item_from_path("<Display>/Objects/Align Vertical/Adjacent");
+    
     initialized = 1;
   }
   
@@ -237,7 +257,7 @@ void diagram_update_menu_sensitivity(Diagram *dia)
 #endif
 
   gtk_widget_set_sensitive(copy_text, active_focus() != NULL);
-  gtk_widget_set_sensitive(cut_text, 0); /* Not implemented */
+  gtk_widget_set_sensitive(cut_text, 0); 
   gtk_widget_set_sensitive(paste_text, active_focus() != NULL);
 
   gtk_widget_set_sensitive(send_to_back, dia->data->selected_count > 0);
@@ -771,10 +791,10 @@ diagram_set_filename(Diagram *dia, char *filename)
   char *title;
   
   g_free(dia->filename);
-  dia->filename = strdup(filename);
+  dia->filename = g_strdup(filename);
 
 
-  title = strrchr(filename, '/');
+  title = strrchr(filename, G_DIR_SEPARATOR);
   if (title==NULL) {
     title = filename;
   } else {
@@ -791,6 +811,7 @@ diagram_set_filename(Diagram *dia, char *filename)
   }
 
   layer_dialog_update_diagram_list();
+  recent_file_history_add((const char *)filename, NULL);
 }
 
 
