@@ -41,6 +41,7 @@ struct _DDisplay {
   GtkWidget *hrule, *vrule;       /* widgets for rulers                */
   GtkWidget *origin;              /* widgets for rulers                */
 
+  GtkWidget *snap_status;
   GtkWidget *zoom_status;         
   GtkWidget *modified_status;
 
@@ -63,6 +64,7 @@ struct _DDisplay {
   
   GSList *update_areas;           /* Update areas list                 */
   GSList *display_areas;          /* Display areas list                */
+  guint update_id;                /* idle handler ID for redraws       */
 
   /* input contexts */
   GdkIC *ic;
@@ -71,8 +73,6 @@ struct _DDisplay {
 
 extern GdkCursor *default_cursor;
 
-extern DDisplay *active_display;
-
 DDisplay *new_display(Diagram *dia);
 /* Normal destroy is done through shell widget destroy event. */
 void ddisplay_really_destroy(DDisplay *ddisp); 
@@ -109,6 +109,7 @@ void ddisplay_resize_canvas(DDisplay *ddisp,
 void ddisplay_render_pixmap(DDisplay *ddisp, Rectangle *update);
 
 DDisplay *ddisplay_active(void);
+Diagram *ddisplay_active_diagram(void);
 
 void ddisplay_close(DDisplay *ddisp);
 
@@ -125,6 +126,8 @@ void ddisplay_scroll_right(DDisplay *ddisp);
 
 void display_update_menu_state(DDisplay *ddisp);
 void ddisplay_update_statusbar(DDisplay *ddisp);
+/* Have to be called from interface.c */
+GtkPixmap *snap_status_load_images(GdkWindow *window);
 
 void display_set_active(DDisplay *ddisp);
 
