@@ -19,7 +19,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-#include "config.h"
+#include <config.h>
 
 #if defined(HAVE_LIBPNG) && defined(HAVE_LIBART)
 
@@ -42,7 +42,8 @@
 #define BAND_HEIGHT 50
 
 static void
-export_png(DiagramData *data, const gchar *filename, const gchar *diafilename)
+export_png(DiagramData *data, const gchar *filename, 
+           const gchar *diafilename, void* user_data)
 {
   Rectangle *ext = &data->extents;
   DDisplay *ddisp;
