@@ -33,4 +33,4 @@ struct _Grid {
 void grid_draw(DDisplay *ddisp, Rectangle *update);
 void snap_to_grid(DDisplay *ddisp, coord *x, coord *y);
 
-#endif GRID_H
+#endif /* GRID_H */
