@@ -15,19 +15,26 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
-/* $Header: /cvs/gnome/dia/app/interface.c,v 1.43 2000/07/31 06:12:47 jamesh Exp $ */
+/* $Header: /cvs/gnome/dia/app/interface.c,v 1.53 2001/03/23 14:29:48 lclausen Exp $ */
 
-#include "config.h"
+#include <config.h>
 
+#include <stdio.h>
+#include <string.h>
+
+#include "diagram.h"
+#include "layer_dialog.h"
 #include "interface.h"
-
+#include "display.h"
 #include "pixmaps.h"
-
 #include "preferences.h"
-
 #include "commands.h"
-
 #include "dia_dirs.h"
+#include <gdk-pixbuf/gdk-pixbuf.h>
+
+static const GtkTargetEntry create_object_targets[] = {
+  { "application/x-dia-object", 0, 0 },
+};
 
 ToolButton tool_data[] =
 {
@@ -137,6 +144,90 @@ origin_button_press(GtkWidget *widget, GdkEventButton *event, gpointer data)
   return FALSE;
 }
 
+#ifndef WITHOUT_ZOOM_COMBO
+static void
+zoom_activate_callback(GtkWidget *dummy, gpointer user_data) {
+  DDisplay *ddisp = (DDisplay *)user_data;
+  gchar *zoom_text = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(ddisp->zoom_status)->entry));
+  float zoom_amount, magnify;
+
+  if (sscanf(zoom_text, "%f", &zoom_amount)) {
+    Point middle;
+    Rectangle *visible;
+
+    magnify = (zoom_amount*DDISPLAY_NORMAL_ZOOM/100.0)/ddisp->zoom_factor;
+    if (fabs(magnify - 1.0) > 0.000001) {
+      visible = &ddisp->visible;
+      middle.x = visible->left*0.5 + visible->right*0.5;
+      middle.y = visible->top*0.5 + visible->bottom*0.5;
+      ddisplay_zoom(ddisp, &middle, magnify);
+    }
+  }
+}
+
+static GtkWidget*
+create_zoom_widget(DDisplay *ddisp) { 
+  GtkWidget *combo = NULL;
+  GList *items = NULL;
+
+  combo = gtk_combo_new();
+  items = g_list_append(items, "400%");
+  items = g_list_append(items, "283%");
+  items = g_list_append(items, "200%");
+  items = g_list_append(items, "141%");
+  items = g_list_append(items, "100%");
+  items = g_list_append(items, "85%");
+  items = g_list_append(items, "70.7%");
+  items = g_list_append(items, "50%");
+  items = g_list_append(items, "35.4%");
+  items = g_list_append(items, "25%");
+  gtk_combo_set_popdown_strings (GTK_COMBO(combo), items);   
+  gtk_combo_set_value_in_list(GTK_COMBO(combo), FALSE, FALSE);
+  gtk_combo_disable_activate(GTK_COMBO(combo));
+
+  gtk_signal_connect (GTK_OBJECT (GTK_COMBO(combo)->entry), "activate",
+		      (GtkSignalFunc) zoom_activate_callback,
+		      ddisp);
+
+  gtk_signal_connect (GTK_OBJECT (GTK_COMBO(combo)->list), "unmap",
+		      (GtkSignalFunc) zoom_activate_callback,
+		      ddisp);
+
+  return combo;
+}
+#endif
+
+static gboolean
+display_drop_callback(GtkWidget *widget, GdkDragContext *context,
+		      gint x, gint y, guint time)
+{
+  if (gtk_drag_get_source_widget(context) != NULL) {
+    /* we only accept drops from the same instance of the application,
+     * as the drag data is a pointer in our address space */
+    return TRUE;
+  }
+  gtk_drag_finish (context, FALSE, FALSE, time);
+  return FALSE;
+}
+
+static void
+display_data_received_callback (GtkWidget *widget, GdkDragContext *context,
+				gint x, gint y, GtkSelectionData *data,
+				guint info, guint time, DDisplay *ddisp)
+{
+  if (data->format == 8 && data->length == sizeof(ToolButtonData *) &&
+      gtk_drag_get_source_widget(context) != NULL) {
+    ToolButtonData *tooldata = *(ToolButtonData **)data->data;
+
+    /* g_message("Tool drop %s at (%d, %d)", (gchar *)tooldata->extra_data, x, y);*/
+    ddisplay_drop_object(ddisp, x, y,
+			 object_get_type((gchar *)tooldata->extra_data),
+			 tooldata->user_data);
+    gtk_drag_finish (context, TRUE, FALSE, time);
+  } else
+    gtk_drag_finish (context, FALSE, FALSE, time);
+}
+
 void
 create_display_shell(DDisplay *ddisp,
 		     int width, int height,
@@ -144,6 +235,9 @@ create_display_shell(DDisplay *ddisp,
 {
   GtkWidget *table, *widget;
   GtkWidget *status_hbox;
+#ifndef WITHOUT_ZOOM_COMBO
+  GtkWidget *zoom_hbox, *zoom_label;
+#endif
   int s_width, s_height;
 
   s_width = gdk_screen_width ();
@@ -154,8 +248,8 @@ create_display_shell(DDisplay *ddisp,
     width = s_width;
 
   /*  The adjustment datums  */
-  ddisp->hsbdata = GTK_ADJUSTMENT (gtk_adjustment_new (0, 0, width, 1, 1, width-1));
-  ddisp->vsbdata = GTK_ADJUSTMENT (gtk_adjustment_new (0, 0, height, 1, 1, height-1));
+  ddisp->hsbdata = GTK_ADJUSTMENT (gtk_adjustment_new (0, 0, width, 1, (width-1)/4, width-1));
+  ddisp->vsbdata = GTK_ADJUSTMENT (gtk_adjustment_new (0, 0, height, 1, (height-1)/4, height-1));
 
   /*  The toplevel shell */
   if (top_level_window) {
@@ -250,6 +344,14 @@ create_display_shell(DDisplay *ddisp,
   gtk_signal_connect (GTK_OBJECT (ddisp->canvas), "event",
 		      (GtkSignalFunc) ddisplay_canvas_events,
 		      ddisp);
+
+  gtk_drag_dest_set(ddisp->canvas, GTK_DEST_DEFAULT_ALL,
+		    create_object_targets, 1, GDK_ACTION_COPY);
+  gtk_signal_connect (GTK_OBJECT (ddisp->canvas), "drag_drop",
+		      GTK_SIGNAL_FUNC(display_drop_callback), NULL);
+  gtk_signal_connect (GTK_OBJECT (ddisp->canvas), "drag_data_received",
+		      GTK_SIGNAL_FUNC(display_data_received_callback), ddisp);
+
   gtk_object_set_user_data (GTK_OBJECT (ddisp->canvas), (gpointer) ddisp);
   /*  pack all the widgets  */
   gtk_table_attach (GTK_TABLE (table), ddisp->origin, 0, 1, 0, 1,
@@ -273,18 +375,41 @@ create_display_shell(DDisplay *ddisp,
   /* the statusbars */
   status_hbox = gtk_hbox_new (FALSE, 2);
 
+  /*
+  ddisp->snap_status = snap_status_load_images(ddisp->shell);
+
+  widget = gtk_button_new ();
+  gtk_container_add(GTK_CONTAINER(widget), ddisp->snap_status);
+  gtk_widget_show(ddisp->snap_status);
+  */
+  /*
+    gtk_signal_connect(GTK_OBJECT(ddisp->origin), "button_press_event",
+    GTK_SIGNAL_FUNC(origin_button_press), ddisp);
+  */
+#ifdef WITHOUT_ZOOM_COMBO
   ddisp->zoom_status = gtk_statusbar_new ();
   ddisp->modified_status = gtk_statusbar_new ();
-  
   gtk_box_pack_start (GTK_BOX (status_hbox), ddisp->zoom_status,
 		      FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (status_hbox), ddisp->modified_status,
-		      TRUE, TRUE, 
-		      0);
+#else
+  ddisp->zoom_status = create_zoom_widget(ddisp);
+  ddisp->modified_status = gtk_statusbar_new ();
 
+  zoom_hbox = gtk_hbox_new(FALSE, 0);
+  zoom_label = gtk_label_new(_("Zoom"));
+  gtk_box_pack_start (GTK_BOX(zoom_hbox), zoom_label,
+		      FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX(zoom_hbox), ddisp->zoom_status,
+		      FALSE, FALSE, 0);
+
+  gtk_box_pack_start (GTK_BOX (status_hbox), zoom_hbox, FALSE, FALSE, 0);
+
+#endif
   gtk_table_attach (GTK_TABLE (table), status_hbox, 0, 3, 3, 4,
                     GTK_FILL, GTK_FILL, 0, 0);
 
+  gtk_box_pack_start (GTK_BOX (status_hbox), ddisp->modified_status,
+		      TRUE, TRUE, 0);
 
   gtk_widget_show (ddisp->hsb);
   gtk_widget_show (ddisp->vsb);
@@ -293,6 +418,10 @@ create_display_shell(DDisplay *ddisp,
   gtk_widget_show (ddisp->vrule);
   gtk_widget_show (ddisp->canvas);
   gtk_widget_show (ddisp->zoom_status);
+#ifndef WITHOUT_ZOOM_COMBO
+  gtk_widget_show (zoom_hbox);
+  gtk_widget_show (zoom_label);
+#endif
   gtk_widget_show (ddisp->modified_status);
   gtk_widget_show (status_hbox);
   gtk_widget_show (table);
@@ -334,6 +463,33 @@ tool_button_press (GtkWidget      *w,
   return FALSE;
 }
 
+static void
+tool_drag_data_get (GtkWidget *widget, GdkDragContext *context,
+		    GtkSelectionData *selection_data, guint info,
+		    guint32 time, ToolButtonData *tooldata)
+{
+  if (info == 0) {
+    gtk_selection_data_set(selection_data, selection_data->target,
+			   8, (guchar *)&tooldata, sizeof(ToolButtonData *));
+  }
+}
+
+static void
+tool_setup_drag_source(GtkWidget *button, ToolButtonData *tooldata,
+		       GdkPixmap *pixmap, GdkBitmap *mask)
+{
+  g_return_if_fail(tooldata->type == CREATE_OBJECT_TOOL);
+
+  gtk_drag_source_set(button, GDK_BUTTON1_MASK,
+		      create_object_targets, 1,
+		      GDK_ACTION_DEFAULT|GDK_ACTION_COPY);
+  gtk_signal_connect(GTK_OBJECT(button), "drag_data_get",
+		     GTK_SIGNAL_FUNC(tool_drag_data_get), tooldata);
+  if (pixmap)
+    gtk_drag_source_set_icon(button, gtk_widget_get_colormap(button),
+			     pixmap, mask);
+}
+
 void 
 tool_select_callback(GtkWidget *widget, gpointer data) {
   ToolButtonData *tooldata = (ToolButtonData *)data;
@@ -405,6 +561,13 @@ create_tools(GtkWidget *parent)
 			GTK_SIGNAL_FUNC (tool_button_press),
 			&tool_data[i].callback_data);
 
+    if (tool_data[i].callback_data.type == CREATE_OBJECT_TOOL)
+      tool_setup_drag_source(button, &tool_data[i].callback_data,
+			     pixmap, mask);
+
+    gdk_pixmap_unref(pixmap);
+    if (mask) gdk_bitmap_unref(mask);
+
     tool_data[i].callback_data.widget = button;
 
     gtk_tooltips_set_tip (tool_tips, button,
@@ -550,9 +713,13 @@ fill_sheet_wbox(GtkWidget *menu_item, Sheet *sheet)
 			gtk_widget_get_colormap(sheet_wbox), &mask, 
 			&style->bg[GTK_STATE_NORMAL], sheet_obj->pixmap);
     } else if (sheet_obj->pixmap_file != NULL) {
-      pixmap = gdk_pixmap_colormap_create_from_xpm(NULL,
-			gtk_widget_get_colormap(sheet_wbox), &mask,
-			&style->bg[GTK_STATE_NORMAL], sheet_obj->pixmap_file);
+      GdkPixbuf *pixbuf;
+
+      pixbuf = gdk_pixbuf_new_from_file(sheet_obj->pixmap_file);
+      if (pixbuf != NULL) {
+	gdk_pixbuf_render_pixmap_and_mask(pixbuf, &pixmap, &mask, 1.0);
+	gdk_pixbuf_unref(pixbuf);
+      }
     } else {
       ObjectType *type;
       type = object_get_type(sheet_obj->object_type);
@@ -594,6 +761,8 @@ fill_sheet_wbox(GtkWidget *menu_item, Sheet *sheet)
     gtk_signal_connect (GTK_OBJECT (button), "button_press_event",
 			GTK_SIGNAL_FUNC (tool_button_press), data);
 
+    tool_setup_drag_source(button, data, pixmap, mask);
+
     gtk_tooltips_set_tip (tool_tips, button,
 			  gettext(sheet_obj->description), NULL);
   }
@@ -786,6 +955,71 @@ toolbox_destroy (GtkWidget *widget, gpointer data)
   app_exit();
 }
 
+/* HB: file dnd stuff lent by The Gimp, not fully understood but working ...
+ */
+enum
+{
+  DIA_DND_TYPE_URI_LIST,
+  DIA_DND_TYPE_TEXT_PLAIN,
+};
+
+static GtkTargetEntry toolbox_target_table[] =
+{
+  { "text/uri-list", 0, DIA_DND_TYPE_URI_LIST },
+  { "text/plain", 0, DIA_DND_TYPE_TEXT_PLAIN }
+};
+
+static guint toolbox_n_targets = (sizeof (toolbox_target_table) /
+                                 sizeof (toolbox_target_table[0]));
+
+static void
+dia_dnd_file_drag_data_received (GtkWidget        *widget,
+                                 GdkDragContext   *context,
+                                 gint              x,
+                                 gint              y,
+                                 GtkSelectionData *data,
+                                 guint             info,
+                                 guint             time)
+{
+  switch (context->action)
+    {
+    case GDK_ACTION_DEFAULT:
+    case GDK_ACTION_COPY:
+    case GDK_ACTION_MOVE:
+    case GDK_ACTION_LINK:
+    case GDK_ACTION_ASK:
+    default:
+      {
+        Diagram *diagram = NULL;
+        DDisplay *ddisp;
+        gchar *sPath = NULL, *pFrom, *pTo; 
+
+        pFrom = strstr((gchar *) data->data, "file:");
+        while (pFrom) {
+          pFrom += 5; /* remove 'file:' */
+          pTo = pFrom;
+	    while (*pTo != 0 && *pTo != 0xd && *pTo != 0xa) pTo ++;
+          sPath = g_strndup(pFrom, pTo - pFrom);
+
+          diagram = diagram_load (sPath, NULL);
+          g_free(sPath);
+
+          if (diagram != NULL) {
+            diagram_update_extents(diagram);
+            layer_dialog_set_diagram(diagram);
+
+            ddisp = new_display(diagram);
+          }
+
+          pFrom = strstr(pTo, "file:");
+        } /* while */
+        gtk_drag_finish (context, TRUE, FALSE, time);
+      }
+      break;
+    }
+  return;
+}
+
 void
 create_toolbox ()
 {
@@ -847,6 +1081,15 @@ create_toolbox ()
   create_color_area (wrapbox);
   create_lineprops_area (wrapbox);
 
+  /* Setup toolbox area as file drop destination */
+  gtk_drag_dest_set (wrapbox,
+		     GTK_DEST_DEFAULT_ALL,
+		     toolbox_target_table, toolbox_n_targets,
+		     GDK_ACTION_COPY);
+  gtk_signal_connect (GTK_OBJECT (wrapbox), "drag_data_received",
+                      GTK_SIGNAL_FUNC (dia_dnd_file_drag_data_received),
+                      wrapbox);
+
   /* menus -- initialised afterwards, because initing the display menus
    * uses the tool buttons*/
   menus_get_toolbox_menubar(&menubar, &accel_group);
