@@ -15,6 +15,11 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "linewidth_area.h"
 #include "attributes.h"
 #include "intl.h"
@@ -33,7 +38,7 @@
 #define AREA_WIDTH X_OFFSET(NUMLINES+1)
 #define AREA_HEIGHT 42
 
-static void linewidth_create_dialog();
+static void linewidth_create_dialog(void);
 
 static int active_linewidth = 2;
 static GdkGC *linewidth_area_gc = NULL;
@@ -193,11 +198,13 @@ linewidth_dialog_ok(GtkWidget *widget, gpointer data) {
 }
 
 static void
-linewidth_dialog_cancel(GtkWidget *widget, gpointer data) {
+linewidth_dialog_cancel(GtkWidget *widget, gpointer data)
+{
   gtk_widget_hide(linewidth_dialog);
 }
 
-static void linewidth_create_dialog()
+static void
+linewidth_create_dialog()
 {
   GtkWidget *button;
   GtkWidget *hbox;
@@ -217,7 +224,7 @@ static void linewidth_create_dialog()
   label = gtk_label_new(_("Line width:"));
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
   gtk_widget_show (label);
-  adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.00, 10.0, 0.01, 0.0, 0.0);
+  adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.00, 10.0, 0.01, 0.05, 0.0);
   linewidth_button = gtk_spin_button_new(adj, attributes_get_default_linewidth(), 2);
   gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(linewidth_button), TRUE);
   gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(linewidth_button), TRUE);
