@@ -15,7 +15,11 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+#include <config.h>
+
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -24,7 +28,6 @@
 #include <string.h>
 #include <glib.h>
 
-#include "config.h"
 #include "intl.h"
 
 #include <tree.h>
@@ -51,6 +54,17 @@
 #define fchmod(f,m) (0)
 #endif
 
+static void
+GHFuncUnknownObjects(gpointer key,
+                     gpointer value,
+                     gpointer user_data)
+{
+  GString* s = (GString*)user_data;
+  g_string_append(s, "\n");
+  g_string_append(s, (gchar*)key);
+  g_free(key);
+}
+
 static GList *
 read_objects(xmlNodePtr objects, GHashTable *objects_hash,const char *filename)
 {
@@ -62,6 +76,11 @@ read_objects(xmlNodePtr objects, GHashTable *objects_hash,const char *filename)
   char *versionstr;
   char *id;
   int version;
+  GHashTable* unknown_hash;
+  GString*    unknown_str;
+
+  unknown_hash = g_hash_table_new(g_str_hash, g_str_equal);
+  unknown_str  = g_string_new("Unknown types while reading diagram file");
 
   list = NULL;
 
@@ -79,15 +98,21 @@ read_objects(xmlNodePtr objects, GHashTable *objects_hash,const char *filename)
 	version = atoi(versionstr);
 	free(versionstr);
       }
-      
+
       type = object_get_type((char *)typestr);
+      
+      if (!type) {
+	if (NULL == g_hash_table_lookup(unknown_hash, typestr))
+	    g_hash_table_insert(unknown_hash, strdup(typestr), 0);
+      }
+      else
+      {
+        obj = type->ops->load(obj_node, version, filename);
+        list = g_list_append(list, obj);
+      
+        g_hash_table_insert(objects_hash, (char *)id, obj);
+      }
       if (typestr) free(typestr);
-      
-      obj = type->ops->load(obj_node, version, filename);
-      list = g_list_append(list, obj);
-      
-      g_hash_table_insert(objects_hash, (char *)id, obj);
-   
     } else if (strcmp(obj_node->name, "group")==0) {
       obj = group_create(read_objects(obj_node, objects_hash, filename));
       list = g_list_append(list, obj);
@@ -97,6 +122,16 @@ read_objects(xmlNodePtr objects, GHashTable *objects_hash,const char *filename)
 
     obj_node = obj_node->next;
   }
+  /* show all the unknown types in one message */
+  if (0 < g_hash_table_size(unknown_hash)) {
+    g_hash_table_foreach(unknown_hash,
+			 GHFuncUnknownObjects,
+			 unknown_str);
+    message_error(unknown_str->str);
+  }
+  g_hash_table_destroy(unknown_hash);
+  g_string_free(unknown_str, TRUE);
+
   return list;
 }
 
@@ -176,7 +211,7 @@ hash_free_string(gpointer       key,
 }
 
 static gboolean
-diagram_data_load(const char *filename, DiagramData *data)
+diagram_data_load(const char *filename, DiagramData *data, void* user_data)
 {
   GHashTable *objects_hash;
   int fd;
@@ -525,12 +560,12 @@ diagram_data_save(DiagramData *data, const char *filename)
    
   /* build the temporary and backup file names */
   dirname = g_strdup(filename);
-  p = strrchr((char *)dirname,'/');
+  p = strrchr((char *)dirname,G_DIR_SEPARATOR);
   if (p) {
     *(p+1) = 0;
   } else {
     g_free(dirname);
-    dirname = g_strdup("./");
+    dirname = g_strdup("." G_DIR_SEPARATOR_S);
   }
   tmpname = g_strconcat(dirname,"__diaXXXXXX",NULL);
   bakname = g_strconcat(filename,"~",NULL);
@@ -552,13 +587,14 @@ diagram_data_save(DiagramData *data, const char *filename)
   fclose(file);
 
   doc = xmlNewDoc("1.0");
-  
-  name_space = xmlNewGlobalNs( doc, "http://www.lysator.liu.se/~alla/dia/",
-			       "dia" );
- 
-  doc->root = xmlNewDocNode(doc, name_space, "diagram", NULL);
 
-  tree = xmlNewChild(doc->root, NULL, "diagramdata", NULL);
+  doc->root = xmlNewDocNode(doc, NULL, "diagram", NULL);
+
+  name_space = xmlNewNs(doc->root, "http://www.lysator.liu.se/~alla/dia/",
+			"dia");
+  xmlSetNs(doc->root, name_space);
+
+  tree = xmlNewChild(doc->root, name_space, "diagramdata", NULL);
   
   attr = new_attribute((ObjectNode)tree, "background");
   data_add_color(attr, &data->bg_color);
@@ -613,7 +649,7 @@ diagram_data_save(DiagramData *data, const char *filename)
   obj_nr = 0;
 
   for (i = 0; i < data->layers->len; i++) {
-    layer_node = xmlNewChild(doc->root, NULL, "layer", NULL);
+    layer_node = xmlNewChild(doc->root, name_space, "layer", NULL);
     layer = (Layer *) g_ptr_array_index(data->layers, i);
     xmlSetProp(layer_node, "name", layer->name);
     if (layer->visible)
@@ -677,7 +713,7 @@ diagram_save(Diagram *dia, const char *filename)
 /* --- filter interfaces --- */
 static void
 export_native(DiagramData *data, const gchar *filename,
-	      const gchar *diafilename)
+	      const gchar *diafilename, void* user_data)
 {
   diagram_data_save(data, filename);
 }
