@@ -15,6 +15,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+#include <config.h>
+
 #include <stdio.h>
 #include <math.h>
 
@@ -127,7 +129,7 @@ click_select_object(DDisplay *ddisp, Point *clickedpoint,
       }
       
       diagram_select(diagram, obj);
-      obj->ops->select(obj, clickedpoint,
+      obj->ops->selectf(obj, clickedpoint,
 		       (Renderer *)ddisp->renderer);
 
       /*
@@ -144,7 +146,7 @@ click_select_object(DDisplay *ddisp, Point *clickedpoint,
       return obj;
     } else { /* Clicked on already selected. */
       /*printf("Already selected\n");*/
-      obj->ops->select(obj, clickedpoint,
+      obj->ops->selectf(obj, clickedpoint,
 		       (Renderer *)ddisp->renderer);
       object_add_updates_list(diagram->data->selected, diagram);
       diagram_flush(diagram);
