@@ -19,8 +19,15 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <stdio.h>
 #include <locale.h>
+#include <string.h> /* strlen */
+#include <signal.h>
+
 #include "intl.h"
 #include "message.h"
 #include "diagram.h"
@@ -33,8 +40,8 @@
 
 #ifdef G_OS_WIN32
 #include <io.h>
-#define popen _popen
-#define pclose _pclose
+#include "win32print.h"
+#define pclose(file) win32_printer_close (file)
 #endif
 
 /* keep track of print options between prints */
@@ -60,7 +67,7 @@ print_page(DiagramData *data, RendererEPS *rend, Rectangle *bounds)
 {
   guint nobjs = 0;
   gfloat tmargin = data->paper.tmargin, bmargin = data->paper.bmargin;
-  gfloat lmargin = data->paper.lmargin, rmargin = data->paper.rmargin;
+  gfloat lmargin = data->paper.lmargin;
   gfloat scale = data->paper.scaling;
 
   /* count the number of objects in this region */
@@ -167,6 +174,14 @@ ok_pressed(GtkButton *button, gboolean *flag)
   gtk_main_quit();
 }
 
+static gboolean sigpipe_received = FALSE;
+static void
+pipe_handler(int signum)
+{
+  sigpipe_received = TRUE;
+}
+
+
 void
 diagram_print_ps(Diagram *dia)
 {
@@ -179,6 +194,13 @@ diagram_print_ps(Diagram *dia)
 
   FILE *file;
   gboolean is_pipe;
+#ifndef G_OS_WIN32
+  /* all the signal stuff below doesn't compile on win32, but it isn't 
+   * needed anymore because the pipe handling - which never worked on win32
+   * anyway - is replace by "native" postscript printing now ...
+   */
+  struct sigaction pipe_action, old_action;
+#endif
 
   ddisp = ddisplay_active();
   dia = ddisp->diagram;
@@ -247,9 +269,15 @@ diagram_print_ps(Diagram *dia)
   gtk_widget_show(button);
 
   /* Set default or old dialog values: */
+#ifdef G_OS_WIN32
+  gtk_entry_set_text(GTK_ENTRY(cmd), 
+		     last_print_options.command 
+		     ? last_print_options.command : win32_printer_default ());
+#else
   gtk_entry_set_text(GTK_ENTRY(cmd), 
 		     last_print_options.command 
 		     ? last_print_options.command : "lpr");
+#endif
   /* Ought to use filename+extension here */
   gtk_entry_set_text(GTK_ENTRY(ofile), 
 		     last_print_options.output 
@@ -267,17 +295,21 @@ diagram_print_ps(Diagram *dia)
   }
 
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(iscmd))) {
+#ifdef G_OS_WIN32
+    file = win32_printer_open (gtk_entry_get_text(GTK_ENTRY(cmd)));
+#else
     file = popen(gtk_entry_get_text(GTK_ENTRY(cmd)), "w");
+#endif
     is_pipe = TRUE;
   } else {
     gchar *filename = gtk_entry_get_text(GTK_ENTRY(ofile));
-    if (filename[0] != '/') {
+    if (!g_path_is_absolute(filename)) {
       char *diagram_dir;
       char *full_filename;
 
       diagram_dir = g_dirname(dia->filename);
       full_filename = g_malloc(strlen(diagram_dir)+strlen(filename)+2);
-      sprintf(full_filename, "%s/%s", diagram_dir, filename);
+      sprintf(full_filename, "%s" G_DIR_SEPARATOR_S "%s", diagram_dir, filename);
       file = fopen(full_filename, "w");
       g_free(full_filename);
     } else {
@@ -295,17 +327,32 @@ diagram_print_ps(Diagram *dia)
   
   if (!file) {
     if (is_pipe)
-      message_warning("Could not run command '%s'",
+      message_warning(_("Could not run command '%s'"),
 		      gtk_entry_get_text(GTK_ENTRY(cmd)));
     else
-      message_warning("Could not open '%s' for writing",
+      message_warning(_("Could not open '%s' for writing"),
 		      gtk_entry_get_text(GTK_ENTRY(ofile)));
     return;
   }
+
+#ifndef G_OS_WIN32
+  /* set up a SIGPIPE handler to catch IO errors, rather than segfaulting */
+  memset(&pipe_action, '\0', sizeof(pipe_action));
+  sigpipe_received = FALSE;
+  pipe_action.sa_handler = pipe_handler;
+  sigaction(SIGPIPE, &pipe_action, &old_action);
+#endif
+
   paginate_psprint(dia, file);
   gtk_widget_destroy(dialog);
   if (is_pipe)
     pclose(file);
   else
     fclose(file);
+
+#ifndef G_OS_WIN32
+  sigaction(SIGPIPE, &old_action, NULL);
+#endif
+  if (sigpipe_received)
+    message_error(_("Error occured while printing"));
 }
