@@ -26,6 +26,10 @@ typedef struct _RendererEPS RendererEPS;
 #include "render.h"
 #include "display.h"
 #include "filter.h"
+#ifdef HAVE_UNICODE
+#include "ps-utf8.h"
+#endif
+
 
 struct _RendererEPS {
   Renderer renderer;
@@ -37,6 +41,10 @@ struct _RendererEPS {
   LineStyle saved_line_style;
   real dash_length;
   real dot_length;
+#ifdef HAVE_UNICODE
+  PSUnicoder *psu;
+#endif
+  Color lcolor;
 };
 
 RendererEPS *new_eps_renderer(Diagram *dia, char *filename);
