@@ -18,10 +18,13 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <assert.h>
 #include <math.h>
+#include <string.h> /* memcpy() */
 
 #include "bezier_conn.h"
 #include "intl.h"
@@ -560,41 +563,113 @@ bezierconn_update_boundingbox(BezierConn *bez)
   Rectangle *bb;
   BezPoint *points;
   int i;
-  
+  Point pt1,pt0,vec;
+  BezierConnBBExtras *extra;
+  real trans;
+
   assert(bez != NULL);
 
+  extra = &bez->extra_spacing;
   bb = &bez->object.bounding_box;
   points = &bez->points[0];
 
   bb->right = bb->left = points[0].p1.x;
   bb->top = bb->bottom = points[0].p1.y;
-  for (i = 1; i < bez->numpoints; i++) {
-    if (points[i].p1.x < bb->left)
-      bb->left = points[i].p1.x;
-    if (points[i].p1.x > bb->right)
-      bb->right = points[i].p1.x;
-    if (points[i].p1.y < bb->top)
-      bb->top = points[i].p1.y;
-    if (points[i].p1.y > bb->bottom)
-      bb->bottom = points[i].p1.y;
+
+  trans = MAX(extra->start_trans,extra->middle_trans);
+  vec = points[0].p1;
+  if (points[1].type == BEZ_CURVE_TO) {
+    point_sub(&vec,&points[1].p1);
+  } else {
+    point_sub(&vec,&points[1].p1); 
+  }
+  check_bb_x(bb,points[0].p1.x + (extra->start_long * vec.x),vec.x);
+  check_bb_y(bb,points[0].p1.y + (extra->start_long * vec.y),vec.y);
+  check_bb_x(bb,points[0].p1.x + (trans * vec.y),vec.y);
+  check_bb_x(bb,points[0].p1.x - (trans * vec.y),vec.y);
+  check_bb_y(bb,points[0].p1.y + (trans * vec.x),vec.x);
+  check_bb_y(bb,points[0].p1.y - (trans * vec.x),vec.x);
+
+  
+  for (i=0;i<(bez->numpoints-1);i++) {
     if (points[i].type == BEZ_CURVE_TO) {
-      if (points[i].p2.x < bb->left)
-	bb->left = points[i].p2.x;
-      if (points[i].p2.x > bb->right)
-	bb->right = points[i].p2.x;
-      if (points[i].p2.y < bb->top)
-	bb->top = points[i].p2.y;
-      if (points[i].p2.y > bb->bottom)
-	bb->bottom = points[i].p2.y;
-      if (points[i].p3.x < bb->left)
-	bb->left = points[i].p3.x;
-      if (points[i].p3.x > bb->right)
-	bb->right = points[i].p3.x;
-      if (points[i].p3.y < bb->top)
-	bb->top = points[i].p3.y;
-      if (points[i].p3.y > bb->bottom)
-	bb->bottom = points[i].p3.y;
+      pt1 = points[i].p3;
+    } else {
+      pt1 = points[i].p1;
     }
+    if (points[i+1].type == BEZ_CURVE_TO) {
+      point_sub(&pt1,&(points[i+1].p3));
+    } else {
+      point_sub(&pt1,&(points[i+1].p1));
+    }
+    point_normalize(&pt1);
+
+    if (i!=0) {
+      if (points[i].type == BEZ_CURVE_TO) {
+        /* FIXME : dig up the math, do something closer to the curve (CC) */        
+        check_bb_x(bb,points[i].p1.x + extra->middle_trans * 1.41,1);
+        check_bb_x(bb,points[i].p1.x - extra->middle_trans * 1.41,1);
+        check_bb_x(bb,points[i].p2.x + extra->middle_trans * 1.41,1);
+        check_bb_x(bb,points[i].p2.x - extra->middle_trans * 1.41,1);
+        check_bb_x(bb,points[i].p3.x + extra->middle_trans * 1.41,1);
+        check_bb_x(bb,points[i].p3.x - extra->middle_trans * 1.41,1);
+
+        check_bb_y(bb,points[i].p1.y + extra->middle_trans * 1.41,1);
+        check_bb_y(bb,points[i].p1.y - extra->middle_trans * 1.41,1);
+        check_bb_y(bb,points[i].p2.y + extra->middle_trans * 1.41,1);
+        check_bb_y(bb,points[i].p2.y - extra->middle_trans * 1.41,1);
+        check_bb_y(bb,points[i].p3.y + extra->middle_trans * 1.41,1);
+        check_bb_y(bb,points[i].p3.y - extra->middle_trans * 1.41,1);
+      } else {
+        real co = point_dot(&pt1,&pt0);
+
+        check_bb_x(bb, points[i].p1.x + (extra->middle_trans * pt1.y),pt1.y);
+        check_bb_x(bb, points[i].p1.x - (extra->middle_trans * pt1.y),pt1.y);
+        check_bb_y(bb, points[i].p1.y + (extra->middle_trans * pt1.x),pt1.x);
+        check_bb_y(bb, points[i].p1.y - (extra->middle_trans * pt1.x),pt1.x);
+
+        if (co > -0.9816) { /* 0.9816 == cos(11deg) */
+          real alpha = fabs(acos(-co));
+          real overshoot = extra->middle_trans / (sin(alpha/2.0));
+          Point pts;
+
+          pts = pt1; point_sub(&pts,&pt0);
+          point_normalize(&pts);
+          
+          check_bb_x(bb,points[i].p1.x + (overshoot * pts.x),pts.x);
+          check_bb_y(bb,points[i].p1.y + (overshoot * pts.y),pts.y);
+        } else { /* this works because i>0 */
+          check_bb_x(bb, points[i].p1.x + (extra->middle_trans * pt0.y),pt0.y);
+          check_bb_x(bb, points[i].p1.x - (extra->middle_trans * pt0.y),pt0.y);
+          check_bb_y(bb, points[i].p1.y + (extra->middle_trans * pt0.x),pt0.x);
+          check_bb_y(bb, points[i].p1.y - (extra->middle_trans * pt0.x),pt0.x);
+        } 
+      }
+    }
+    pt0=pt1;
+  }  
+
+  trans = MAX(extra->start_trans,extra->middle_trans);
+  if (points[bez->numpoints-1].type == BEZ_CURVE_TO) {
+    vec = points[bez->numpoints-1].p3;
+    point_sub(&vec,&points[bez->numpoints-1].p2);
+
+    check_bb_x(bb,points[bez->numpoints-1].p3.x + (extra->end_long * vec.x),vec.x);
+    check_bb_y(bb,points[bez->numpoints-1].p3.y + (extra->end_long * vec.y),vec.y);
+    check_bb_x(bb,points[bez->numpoints-1].p3.x + (trans * vec.y),vec.y);
+    check_bb_x(bb,points[bez->numpoints-1].p3.x - (trans * vec.y),vec.y);
+    check_bb_y(bb,points[bez->numpoints-1].p3.y + (trans * vec.x),vec.x);
+    check_bb_y(bb,points[bez->numpoints-1].p3.y - (trans * vec.x),vec.x);
+  } else {
+    vec = points[bez->numpoints-1].p1;
+    point_sub(&vec,&points[bez->numpoints-2].p1); 
+
+    check_bb_x(bb,points[bez->numpoints-1].p1.x + (extra->end_long * vec.x),vec.x);
+    check_bb_y(bb,points[bez->numpoints-1].p1.y + (extra->end_long * vec.y),vec.y);
+    check_bb_x(bb,points[bez->numpoints-1].p1.x + (trans * vec.y),vec.y);
+    check_bb_x(bb,points[bez->numpoints-1].p1.x - (trans * vec.y),vec.y);
+    check_bb_y(bb,points[bez->numpoints-1].p1.y + (trans * vec.x),vec.x);
+    check_bb_y(bb,points[bez->numpoints-1].p1.y - (trans * vec.x),vec.x);
   }
 }
 
@@ -718,6 +793,7 @@ bezierconn_copy(BezierConn *from, BezierConn *to)
   *to->object.handles[to->object.num_handles-1] =
     *from->object.handles[to->object.num_handles-1];
 
+  memcpy(&to->extra_spacing,&from->extra_spacing,sizeof(to->extra_spacing));
   bezierconn_update_data(to);
 }
 
