@@ -15,6 +15,11 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+#include <config.h>
+
+#include <string.h> /* strlen() */
+
 #include "dia_dirs.h"
 #ifdef G_OS_WIN32
 #include <windows.h>
@@ -42,7 +47,10 @@ dia_get_data_directory (const gchar* subdir)
   return g_strconcat (sLoc , subdir, NULL); 
 
 #else
-  return g_strconcat (DATADIR, G_DIR_SEPARATOR_S, subdir, NULL);
+  if (strlen (subdir) == 0)		
+    return g_strconcat (DATADIR, NULL);
+  else
+    return g_strconcat (DATADIR, G_DIR_SEPARATOR_S, subdir, NULL);
 #endif
 }
 
