@@ -45,11 +45,14 @@
  * Author: Federico Mena <federico@nuclecu.unam.mx>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h> /* strlen */
 
-#include "config.h"
 #include "intl.h"
 #include "utils.h"
 #include "font.h"
@@ -349,7 +352,7 @@ static void
 init_x11_font(FontPrivate *font)
 {
   int i;
-  GdkFont *gdk_font;
+  GdkFont *gdk_font = NULL;
   int bufsize;
   char *buffer;
   char *x11_font;
@@ -550,6 +553,7 @@ font_string_width(const char *string, Font *font, real height)
   double width_height;
   GdkFont *gdk_font;
 
+
   /* Note: This is an ugly hack. It tries to overestimate the width with
      some magic stuff. No guarantees. */
   gdk_font = font_get_gdkfont(font, 100);
@@ -594,7 +598,7 @@ suck_font (GdkFont *font)
 	GdkColor black, white;
 	GdkImage *image;
 	GdkGC *gc;
-	guchar *bitmap, *line;
+	guchar *line;
 	int width, height;
 	int black_pixel, pixel;
 
