@@ -21,6 +21,16 @@
 #include <glib.h>
 #include <math.h>
 
+#ifdef _MSC_VER
+   /* there are some things more in the gcc headers */
+#  include <float.h>
+#  define finite(a) _finite(a)
+#  define isnan(a) _isnan(a)
+#  define M_PI      3.14159265358979323846
+#  define M_SQRT2	1.41421356237309504880	/* sqrt(2) */
+#  define M_SQRT1_2 0.70710678118654752440	/* 1/sqrt(2) */
+#endif
+
 /*
   Coordinate system used:
    +---> x
@@ -207,6 +217,32 @@ point_copy_add_scaled(Point *dst, Point *src, Point *vct, real alpha)
 }
 #endif
 
+G_INLINE_FUNC void check_bb_x( Rectangle *bb, real val, real check );
+#ifdef G_CAN_INLINE
+G_INLINE_FUNC void 
+check_bb_x( Rectangle *bb, real val, real check )
+{
+  /* if x is outside the bounding box, and check is real, adjust the bb */
+  if (finite(check)) { 
+    if (bb->left > val) bb->left = val; 
+    if (bb->right < val) bb->right = val; 
+  }
+}
+#endif
+
+G_INLINE_FUNC void check_bb_y( Rectangle *bb, real val, real check );
+#ifdef G_CAN_INLINE
+G_INLINE_FUNC void 
+check_bb_y( Rectangle *bb, real val, real check )
+{
+  /* if y is outside the bounding box, and check is real, adjust the bb */
+  if (finite(check)) { 
+    if (bb->top > val) bb->top = val; 
+    if (bb->bottom < val) bb->bottom = (val); 
+  } 
+}
+#endif
+
 void rectangle_union(Rectangle *r1, Rectangle *r2);
 void int_rectangle_union(IntRectangle *r1, IntRectangle *r2);
 void rectangle_intersection(Rectangle *r1, Rectangle *r2);
@@ -255,4 +291,16 @@ real distance_bez_shape_point(BezPoint *b, guint npoints,
 real distance_ellipse_point(Point *centre, real width, real height,
 			    real line_width, Point *point);
 
+typedef real  Vector[3];
+typedef Vector  Matrix[3];
+
+void          transform_point             (Matrix, Point *, Point *);
+void          mult_matrix                 (Matrix, Matrix);
+void          identity_matrix             (Matrix);
+void          translate_matrix            (Matrix, real, real);
+void          scale_matrix                (Matrix, real, real);
+void          rotate_matrix               (Matrix, real);
+void          xshear_matrix               (Matrix, real);
+void          yshear_matrix               (Matrix, real);
+
 #endif /* GEOMETRY_H */
