@@ -18,10 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+#include <config.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <assert.h>
 #include <math.h>
+#include <string.h> /* memcpy() */
 
 #include "neworth_conn.h"
 #include "connectionpoint.h"
@@ -231,7 +234,8 @@ neworthconn_distance_from(NewOrthConn *orth, Point *point, real line_width)
   return dist;
 }
 
-void neworthconn_update_midpoints(NewOrthConn *orth)
+static void
+neworthconn_update_midpoints(NewOrthConn *orth)
 {
   int i;
   GSList *elem;
@@ -276,6 +280,8 @@ neworthconn_update_boundingbox(NewOrthConn *orth)
   Rectangle *bb;
   Point *points;
   int i;
+  Point pt1,pt0;
+  NewOrthConnBBExtras *extra = &orth->extra_spacing;
   
   assert(orth != NULL);
 
@@ -284,15 +290,45 @@ neworthconn_update_boundingbox(NewOrthConn *orth)
 
   bb->right = bb->left = points[0].x;
   bb->top = bb->bottom = points[0].y;
-  for (i=1;i<orth->numpoints;i++) {
-    if (points[i].x < bb->left)
-      bb->left = points[i].x;
-    if (points[i].x > bb->right)
-      bb->right = points[i].x;
-    if (points[i].y < bb->top)
-      bb->top = points[i].y;
-    if (points[i].y > bb->bottom)
-      bb->bottom = points[i].y;
+
+  for (i=0;i<(orth->numpoints-1);i++) {
+    pt1 = points[i];
+    point_sub(&pt1,&(points[i+1]));
+    point_normalize(&pt1);
+
+    if (i==0) {
+      real trans = MAX(extra->start_trans,extra->middle_trans);
+      check_bb_x(bb,points[i].x + (extra->start_long * pt1.x),pt1.x);
+      check_bb_x(bb,points[i].x + (trans * pt1.y),pt1.y); 
+      check_bb_x(bb,points[i].x - (trans * pt1.y),pt1.y); 
+
+      check_bb_y(bb,points[i].y + (trans * pt1.x),pt1.x); 
+      check_bb_y(bb,points[i].y - (trans * pt1.x),pt1.x);       
+    } else {
+      check_bb_x(bb, points[i].x + (extra->middle_trans * pt1.y),pt1.y);
+      check_bb_x(bb, points[i].x - (extra->middle_trans * pt1.y),pt1.y);
+      check_bb_y(bb, points[i].y + (extra->middle_trans * pt1.x),pt1.x);
+      check_bb_y(bb, points[i].y - (extra->middle_trans * pt1.x),pt1.x);
+    }
+    if (i!=0) {
+      real overshoot = extra->middle_trans / (sin(M_PI/4.0));
+      Point pts;
+
+      pts = pt1; point_sub(&pts,&pt0);
+      point_normalize(&pts);
+
+      check_bb_x(bb,points[i].x + (overshoot * pts.x),pts.x);
+      check_bb_y(bb,points[i].y + (overshoot * pts.y),pts.y);
+    }
+    if (i==(orth->numpoints-2)) {
+      check_bb_x(bb,points[i+1].x - (extra->end_long * pt1.x),pt1.x);
+      check_bb_x(bb,points[i+1].x - (extra->end_trans * pt1.y),pt1.y);
+      check_bb_x(bb,points[i+1].x + (extra->end_trans * pt1.y),pt1.y);
+
+      check_bb_y(bb,points[i+1].y - (extra->end_trans * pt1.x),pt1.y);
+      check_bb_y(bb,points[i+1].y + (extra->end_trans * pt1.x),pt1.y);
+    }
+    pt0=pt1;
   }
 }
 
@@ -437,6 +473,7 @@ neworthconn_copy(NewOrthConn *from, NewOrthConn *to)
   }
   rcc = 0;
   to->midpoints = connpointline_copy(toobj,from->midpoints,&rcc);
+  memcpy(&to->extra_spacing,&from->extra_spacing,sizeof(to->extra_spacing));
 }
 
 
@@ -670,7 +707,7 @@ delete_point(NewOrthConn *orth, int pos)
     orth->points[i] = orth->points[i+1];
   }
   
-  orth->points = realloc(orth->points, orth->numpoints*sizeof(Point));
+  orth->points = g_realloc(orth->points, orth->numpoints*sizeof(Point));
 }
 
 /* Make sure numpoints have been decreased before calling this function.
@@ -689,9 +726,9 @@ remove_handle(NewOrthConn *orth, int segment)
     orth->orientation[i] = orth->orientation[i+1];
   }
 
-  orth->orientation = realloc(orth->orientation,
+  orth->orientation = g_realloc(orth->orientation,
 			      (orth->numpoints-1)*sizeof(Orientation));
-  orth->handles = realloc(orth->handles,
+  orth->handles = g_realloc(orth->handles,
 			  (orth->numpoints-1)*sizeof(Handle *));
   
   object_remove_handle(&orth->object, handle);
@@ -705,7 +742,7 @@ add_point(NewOrthConn *orth, int pos, Point *point)
   
   orth->numpoints++;
 
-  orth->points = realloc(orth->points, orth->numpoints*sizeof(Point));
+  orth->points = g_realloc(orth->points, orth->numpoints*sizeof(Point));
   for (i=orth->numpoints-1;i>pos;i--) {
     orth->points[i] = orth->points[i-1];
   }
@@ -721,9 +758,9 @@ insert_handle(NewOrthConn *orth, int segment,
 {
   int i;
   
-  orth->orientation = realloc(orth->orientation,
+  orth->orientation = g_realloc(orth->orientation,
 			      (orth->numpoints-1)*sizeof(Orientation));
-  orth->handles = realloc(orth->handles,
+  orth->handles = g_realloc(orth->handles,
 			  (orth->numpoints-1)*sizeof(Handle *));
   for (i=orth->numpoints-2;i>segment;i--) {
     orth->handles[i] = orth->handles[i-1];
