@@ -15,10 +15,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+#include <config.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <assert.h>
 #include <math.h>
+#include <string.h> /* memcpy() */
 
 #include "orth_conn.h"
 #include "message.h"
@@ -58,6 +61,8 @@ static ObjectChange *
 endsegment_create_change(OrthConn *orth, enum change_type type,
 			 int segment, Point *point,
 			 Handle *handle);
+static void 
+place_handle_by_swapping(OrthConn *orth, int index, Handle *handle);
 
 struct EndSegmentChange {
   ObjectChange obj_change;
@@ -248,6 +253,8 @@ orthconn_update_boundingbox(OrthConn *orth)
   Rectangle *bb;
   Point *points;
   int i;
+  Point pt1,pt0;
+  OrthConnBBExtras *extra = &orth->extra_spacing;
   
   assert(orth != NULL);
 
@@ -256,15 +263,45 @@ orthconn_update_boundingbox(OrthConn *orth)
 
   bb->right = bb->left = points[0].x;
   bb->top = bb->bottom = points[0].y;
-  for (i=1;i<orth->numpoints;i++) {
-    if (points[i].x < bb->left)
-      bb->left = points[i].x;
-    if (points[i].x > bb->right)
-      bb->right = points[i].x;
-    if (points[i].y < bb->top)
-      bb->top = points[i].y;
-    if (points[i].y > bb->bottom)
-      bb->bottom = points[i].y;
+  
+  for (i=0;i<(orth->numpoints-1);i++) {
+    pt1 = points[i];
+    point_sub(&pt1,&(points[i+1]));
+    point_normalize(&pt1);
+
+    if (i==0) {
+      real trans = MAX(extra->start_trans,extra->middle_trans);
+      check_bb_x(bb,points[i].x + (extra->start_long * pt1.x),pt1.x);
+      check_bb_x(bb,points[i].x + (trans * pt1.y),pt1.y); 
+      check_bb_x(bb,points[i].x - (trans * pt1.y),pt1.y); 
+
+      check_bb_y(bb,points[i].y + (trans * pt1.x),pt1.x); 
+      check_bb_y(bb,points[i].y - (trans * pt1.x),pt1.x);       
+    } else {
+      check_bb_x(bb, points[i].x + (extra->middle_trans * pt1.y),pt1.y);
+      check_bb_x(bb, points[i].x - (extra->middle_trans * pt1.y),pt1.y);
+      check_bb_y(bb, points[i].y + (extra->middle_trans * pt1.x),pt1.x);
+      check_bb_y(bb, points[i].y - (extra->middle_trans * pt1.x),pt1.x);
+    }
+    if (i!=0) {
+      real overshoot = extra->middle_trans / (sin(M_PI/4.0));
+      Point pts;
+
+      pts = pt1; point_sub(&pts,&pt0);
+      point_normalize(&pts);
+
+      check_bb_x(bb,points[i].x + (overshoot * pts.x),pts.x);
+      check_bb_y(bb,points[i].y + (overshoot * pts.y),pts.y);
+    }
+    if (i==(orth->numpoints-2)) {
+      check_bb_x(bb,points[i+1].x - (extra->end_long * pt1.x),pt1.x);
+      check_bb_x(bb,points[i+1].x - (extra->end_trans * pt1.y),pt1.y);
+      check_bb_x(bb,points[i+1].x + (extra->end_trans * pt1.y),pt1.y);
+
+      check_bb_y(bb,points[i+1].y - (extra->end_trans * pt1.x),pt1.y);
+      check_bb_y(bb,points[i+1].y + (extra->end_trans * pt1.x),pt1.y);
+    }
+    pt0=pt1;
   }
 }
 
@@ -406,6 +443,7 @@ orthconn_copy(OrthConn *from, OrthConn *to)
     to->handles[i]->connected_to = NULL;
     toobj->handles[i] = to->handles[i];
   }
+  memcpy(&to->extra_spacing,&from->extra_spacing,sizeof(to->extra_spacing));
 }
 
 void
@@ -424,7 +462,8 @@ orthconn_destroy(OrthConn *orth)
   g_free(orth->handles);
 }
 
-void place_handle_by_swapping(OrthConn *orth, int index, Handle *handle)
+static void 
+place_handle_by_swapping(OrthConn *orth, int index, Handle *handle)
 {
   Object *obj;
   Handle *tmp;
@@ -634,7 +673,7 @@ delete_point(OrthConn *orth, int pos)
     orth->points[i] = orth->points[i+1];
   }
   
-  orth->points = realloc(orth->points, orth->numpoints*sizeof(Point));
+  orth->points = g_realloc(orth->points, orth->numpoints*sizeof(Point));
 }
 
 /* Make sure numpoints have been decreased before calling this function.
@@ -653,9 +692,9 @@ remove_handle(OrthConn *orth, int segment)
     orth->orientation[i] = orth->orientation[i+1];
   }
 
-  orth->orientation = realloc(orth->orientation,
+  orth->orientation = g_realloc(orth->orientation,
 			      (orth->numpoints-1)*sizeof(Orientation));
-  orth->handles = realloc(orth->handles,
+  orth->handles = g_realloc(orth->handles,
 			  (orth->numpoints-1)*sizeof(Handle *));
   
   object_remove_handle(&orth->object, handle);
@@ -669,7 +708,7 @@ add_point(OrthConn *orth, int pos, Point *point)
   
   orth->numpoints++;
 
-  orth->points = realloc(orth->points, orth->numpoints*sizeof(Point));
+  orth->points = g_realloc(orth->points, orth->numpoints*sizeof(Point));
   for (i=orth->numpoints-1;i>pos;i--) {
     orth->points[i] = orth->points[i-1];
   }
@@ -685,9 +724,9 @@ insert_handle(OrthConn *orth, int segment,
 {
   int i;
   
-  orth->orientation = realloc(orth->orientation,
+  orth->orientation = g_realloc(orth->orientation,
 			      (orth->numpoints-1)*sizeof(Orientation));
-  orth->handles = realloc(orth->handles,
+  orth->handles = g_realloc(orth->handles,
 			  (orth->numpoints-1)*sizeof(Handle *));
   for (i=orth->numpoints-2;i>segment;i--) {
     orth->handles[i] = orth->handles[i-1];
