@@ -23,6 +23,15 @@
 #define HANDLE_CORNER (HANDLE_CUSTOM1)
 
 typedef struct _PolyConn PolyConn;
+typedef struct _PolyConnBBExtras PolyConnBBExtras;
+
+/* This holds some extra spacing to be used by polyconn_update_boundingbox.
+ What this space means is up to the subclass. */
+struct _PolyConnBBExtras {
+  real start_long, start_trans;
+  real middle_trans;
+  real end_long, end_trans;
+};
 
 /* This is a subclass of Object used to help implementing objects
  * that connect points with polygonal line-segments.
@@ -33,8 +42,11 @@ struct _PolyConn {
 
   int numpoints; /* >= 2 */
   Point *points;
+
+  PolyConnBBExtras extra_spacing;
 };
 
+
 void polyconn_update_data(PolyConn *poly);
 void polyconn_update_boundingbox(PolyConn *poly);
 void polyconn_simple_draw(PolyConn *poly, Renderer *renderer, real width);
