@@ -16,10 +16,13 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#include <config.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <assert.h>
 #include <math.h>
+#include <string.h> /* memcpy() */
 
 #include "polyshape.h"
 #include "message.h"
@@ -149,7 +152,7 @@ add_handle(PolyShape *poly, int pos, Point *point, Handle *handle,
   Object *obj;
   
   poly->numpoints++;
-  poly->points = realloc(poly->points, poly->numpoints*sizeof(Point));
+  poly->points = g_realloc(poly->points, poly->numpoints*sizeof(Point));
 
   for (i=poly->numpoints-1; i > pos; i--) {
     poly->points[i] = poly->points[i-1];
@@ -177,7 +180,7 @@ remove_handle(PolyShape *poly, int pos)
   for (i=pos; i < poly->numpoints; i++) {
     poly->points[i] = poly->points[i+1];
   }
-  poly->points = realloc(poly->points, poly->numpoints*sizeof(Point));
+  poly->points = g_realloc(poly->points, poly->numpoints*sizeof(Point));
 
   old_handle = obj->handles[pos];
   old_cp1 = obj->connections[2*pos];
@@ -266,6 +269,8 @@ polyshape_update_boundingbox(PolyShape *poly)
   Rectangle *bb;
   Point *points;
   int i;
+  Point pt1,pt0;
+  PolyShapeBBExtras *extra = &poly->extra_spacing;
   
   assert(poly != NULL);
 
@@ -274,15 +279,40 @@ polyshape_update_boundingbox(PolyShape *poly)
 
   bb->right = bb->left = points[0].x;
   bb->top = bb->bottom = points[0].y;
-  for (i=1;i<poly->numpoints;i++) {
-    if (points[i].x < bb->left)
-      bb->left = points[i].x;
-    if (points[i].x > bb->right)
-      bb->right = points[i].x;
-    if (points[i].y < bb->top)
-      bb->top = points[i].y;
-    if (points[i].y > bb->bottom)
-      bb->bottom = points[i].y;
+
+  for (i=0;i<=(poly->numpoints);i++) {
+    int imod = i % poly->numpoints;
+    int ipomod = (i+1) % poly->numpoints;
+    pt1 = points[imod];
+    point_sub(&pt1,&(points[ipomod]));
+    point_normalize(&pt1);
+
+    if (i!=0) {
+      real co = point_dot(&pt1,&pt0);
+
+      check_bb_x(bb, points[imod].x + (extra->border_trans * pt1.y),pt1.y);
+      check_bb_x(bb, points[imod].x - (extra->border_trans * pt1.y),pt1.y);
+      check_bb_y(bb, points[imod].y + (extra->border_trans * pt1.x),pt1.x);
+      check_bb_y(bb, points[imod].y - (extra->border_trans * pt1.x),pt1.x);
+
+      if (co > -0.9816) { /* 0.9816 == cos(11deg) */
+        real alpha = fabs(acos(-co));
+        real overshoot = extra->border_trans / (sin(alpha/2.0));
+        Point pts;
+
+        pts = pt1; point_sub(&pts,&pt0);
+        point_normalize(&pts);
+
+        check_bb_x(bb,points[imod].x + (overshoot * pts.x),pts.x);
+        check_bb_y(bb,points[imod].y + (overshoot * pts.y),pts.y);
+      } else { 
+          check_bb_x(bb, points[imod].x + (extra->border_trans * pt0.y),pt0.y);
+          check_bb_x(bb, points[imod].x - (extra->border_trans * pt0.y),pt0.y);
+          check_bb_y(bb, points[imod].y + (extra->border_trans * pt0.x),pt0.x);
+          check_bb_y(bb, points[imod].y - (extra->border_trans * pt0.x),pt0.x);
+      } 
+    }
+    pt0=pt1;
   }
 }
 
@@ -382,6 +412,7 @@ polyshape_copy(PolyShape *from, PolyShape *to)
     to->object.connections[2*i+1]->object = &to->object;
   }
 
+  memcpy(&to->extra_spacing,&from->extra_spacing,sizeof(to->extra_spacing));
   polyshape_update_data(to);
 }
 
