@@ -18,15 +18,15 @@
 #ifndef RENDER_H
 #define RENDER_H
 
+typedef struct _RenderOps RenderOps;
+typedef struct _InteractiveRenderOps InteractiveRenderOps;
+typedef struct _Renderer Renderer;
+
 #include "geometry.h"
 #include "color.h"
 #include "font.h"
 #include "dia_image.h"
 
-typedef struct _RenderOps RenderOps;
-typedef struct _InteractiveRenderOps InteractiveRenderOps;
-typedef struct _Renderer Renderer;
-
 typedef enum {
   LINECAPS_BUTT,
   LINECAPS_ROUND,
@@ -149,17 +149,6 @@ typedef void (*FillEllipseFunc) (Renderer *renderer,
 				 real width, real height,
 				 Color *color);
 
-/* BezPoint types:
- *   BEZ_MOVE_TO: move to point p1
- *   BEZ_LINE_TO: line to point p1
- *   BEZ_CURVE_TO: curve to point p3 using p1 and p2 as control points.
- */
-typedef struct _BezPoint BezPoint;
-struct _BezPoint {
-  enum {BEZ_MOVE_TO, BEZ_LINE_TO, BEZ_CURVE_TO} type;
-  Point p1, p2, p3;
-};
-
 /* Draw a bezier curve, given it's control points.
  * The first BezPoint must be of type MOVE_TO, and no other ones may be
  * MOVE_TO's.
@@ -237,7 +226,6 @@ typedef void (*FillPixelRectangleFunc) (Renderer *renderer,
 					int width, int height,
 					Color *color);
 
-
 struct _RenderOps {
   /* Control ops: */
   BeginRenderFunc   begin_render;
