@@ -15,6 +15,8 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+#include <config.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -35,7 +37,8 @@ enum change_type {
   TYPE_DELETE_FORWARD,
   TYPE_INSERT_CHAR,
   TYPE_JOIN_ROW,
-  TYPE_SPLIT_ROW
+  TYPE_SPLIT_ROW,
+  TYPE_DELETE_ALL
 };
 
 struct TextObjectChange {
@@ -46,6 +49,7 @@ struct TextObjectChange {
   char ch;
   int pos;
   int row;
+  char *str;
 };
 
 static ObjectChange *text_create_change(Text *text, enum change_type type,
@@ -298,6 +302,10 @@ text_calc_boundingbox(Text *text, Rectangle *box)
 
   box->right = box->left + text->max_width;
   
+  if (text->cursor_pos == 0) {
+    box->left -= 0.05; /* Half the cursor width */
+  }
+
   box->top = text->position.y - text->ascent;
 
   box->bottom = box->top + text->height*text->numlines;
@@ -834,6 +842,19 @@ int text_is_empty(Text *text)
   return TRUE;
 }
 
+int
+text_delete_all(Text *text, ObjectChange **change)
+{
+  if (!text_is_empty(text)) {
+    *change = text_create_change(text, TYPE_DELETE_ALL,
+				 0, text->cursor_pos, text->cursor_row);
+    
+    text_set_string(text, "");
+    return TRUE;
+  }
+  return FALSE;
+}
+
 void
 data_add_text(AttributeNode attr, Text *text)
 {
@@ -903,7 +924,7 @@ data_text(AttributeNode text_attr)
     align = data_enum(attribute_first_data(attr));
   
   text = new_text(string, font, height, &pos, &col, align);
-  if (string) free(string);
+  if (string) g_free(string);
   return text;
 }
 
@@ -955,8 +976,14 @@ text_change_apply(struct TextObjectChange *change, Object *obj)
   case TYPE_JOIN_ROW:
     text_join_lines(text, change->row);
     break;
+  case TYPE_DELETE_ALL:
+    set_string(text, "");
+    text->cursor_pos = 0;
+    text->cursor_row = 0;
+    break;
   }
 }
+
 static void
 text_change_revert(struct TextObjectChange *change, Object *obj)
 {
@@ -987,9 +1014,18 @@ text_change_revert(struct TextObjectChange *change, Object *obj)
     text->cursor_row = change->row;
     text_split_line(text);
     break;
+  case TYPE_DELETE_ALL:
+    set_string(text, change->str);
+    text->cursor_pos = change->pos;
+    text->cursor_row = change->row;
+    break;
   }
 }
 
+static void
+text_change_free(struct TextObjectChange *change) {
+  g_free(change->str);
+}
 
 static ObjectChange *
 text_create_change(Text *text, enum change_type type,
@@ -1001,12 +1037,16 @@ text_create_change(Text *text, enum change_type type,
 
   change->obj_change.apply = (ObjectChangeApplyFunc) text_change_apply;
   change->obj_change.revert = (ObjectChangeRevertFunc) text_change_revert;
-  change->obj_change.free = (ObjectChangeFreeFunc) NULL;
+  change->obj_change.free = (ObjectChangeFreeFunc) text_change_free;
 
   change->text = text;
   change->type = type;
   change->ch = ch;
   change->pos = pos;
   change->row = row;
+  if (type == TYPE_DELETE_ALL)
+    change->str = text_get_string_copy(text);
+  else
+    change->str = NULL;
   return (ObjectChange *)change;
 }
