@@ -15,12 +15,16 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
 #include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 
-#include "config.h"
 #include "intl.h"
 #include "object.h"
 #include "element.h"
@@ -28,6 +32,7 @@
 #include "render.h"
 #include "attributes.h"
 #include "widgets.h"
+#include "properties.h"
 
 #include "pixmaps/entity.xpm"
 
@@ -39,19 +44,6 @@
 #define FONT_HEIGHT 0.8
 
 typedef struct _Entity Entity;
-typedef struct _EntityPropertiesDialog EntityPropertiesDialog;
-typedef struct _EntityState EntityState;
-
-struct _EntityState {
-  ObjectState obj_state;
-
-  real border_width;
-  Color border_color;
-  Color inner_color;
-  char *name;
-  real name_width;  
-  int weak;
-};  
 
 struct _Entity {
   Element element;
@@ -68,17 +60,6 @@ struct _Entity {
   
   int weak;
   
-  EntityPropertiesDialog *properties_dialog;
-};
-
-struct _EntityPropertiesDialog {
-  GtkWidget *vbox;
-
-  GtkToggleButton *weak;
-  GtkEntry *name;
-  GtkSpinButton *border_width;
-  DiaColorSelector *fg_color;
-  DiaColorSelector *bg_color;
 };
 
 static real entity_distance_from(Entity *entity, Point *point);
@@ -95,15 +76,17 @@ static Object *entity_create(Point *startpoint,
 			     Handle **handle2);
 static void entity_destroy(Entity *entity);
 static Object *entity_copy(Entity *entity);
-static GtkWidget *entity_get_properties(Entity *entity);
-static ObjectChange *entity_apply_properties(Entity *entity);
+static PropDescription *
+entity_describe_props(Entity *entity);
+static void
+entity_get_props(Entity *entity, Property *props, guint nprops);
+static void
+entity_set_props(Entity *entity, Property *props, guint nprops);
 
 static void entity_save(Entity *entity, ObjectNode obj_node,
 			const char *filename);
 static Object *entity_load(ObjectNode obj_node, int version,
 			   const char *filename);
-static EntityState *entity_get_state(Entity *Entity);
-static void entity_set_state(Entity *Entity, EntityState *state);
 
 static ObjectTypeOps entity_type_ops =
 {
@@ -131,114 +114,58 @@ static ObjectOps entity_ops = {
   (CopyFunc)            entity_copy,
   (MoveFunc)            entity_move,
   (MoveHandleFunc)      entity_move_handle,
-  (GetPropertiesFunc)   entity_get_properties,
-  (ApplyPropertiesFunc) entity_apply_properties,
-  (ObjectMenuFunc)      NULL
+  (GetPropertiesFunc)   object_create_props_dialog,
+  (ApplyPropertiesFunc) object_apply_props_from_dialog,
+  (ObjectMenuFunc)      NULL,
+  (DescribePropsFunc)   entity_describe_props,
+  (GetPropsFunc)        entity_get_props,
+  (SetPropsFunc)        entity_set_props,
 };
 
-static ObjectChange *
-entity_apply_properties(Entity *entity)
+static PropDescription entity_props[] = {
+  ELEMENT_COMMON_PROPERTIES,
+  { "name", PROP_TYPE_STRING, PROP_FLAG_VISIBLE,
+    N_("Name:"), NULL, NULL },
+  { "weak", PROP_TYPE_BOOL, PROP_FLAG_VISIBLE,
+    N_("Weak:"), NULL, NULL },
+  PROP_STD_LINE_WIDTH,
+  PROP_STD_LINE_COLOUR,
+  PROP_STD_FILL_COLOUR,
+  PROP_DESC_END
+};
+
+static PropDescription *
+entity_describe_props(Entity *entity)
 {
-  ObjectState *old_state;
-  EntityPropertiesDialog *prop_dialog;
-
-  prop_dialog = entity->properties_dialog;
-
-  old_state = (ObjectState *)entity_get_state(entity);
-
-  entity->border_width = gtk_spin_button_get_value_as_float(prop_dialog->border_width);
-  dia_color_selector_get_color(prop_dialog->fg_color, &entity->border_color);
-  dia_color_selector_get_color(prop_dialog->bg_color, &entity->inner_color);
-  g_free(entity->name);
-  entity->name = strdup(gtk_entry_get_text(prop_dialog->name));
-  entity->weak = prop_dialog->weak->active;
-
-  entity->name_width =
-    font_string_width(entity->name, entity->font, FONT_HEIGHT);
-  
-  entity_update_data(entity);
-
-  return new_object_state_change((Object *)entity, old_state,
-				 (GetStateFunc)entity_get_state,
-				 (SetStateFunc)entity_set_state);
+  if (entity_props[0].quark == 0)
+    prop_desc_list_calculate_quarks(entity_props);
+  return entity_props;
 }
 
-static GtkWidget *
-entity_get_properties(Entity *entity)
+static PropOffset entity_offsets[] = {
+  ELEMENT_COMMON_PROPERTIES_OFFSETS,
+  { "name", PROP_TYPE_STRING, offsetof(Entity, name) },
+  { "weak", PROP_TYPE_BOOL, offsetof(Entity, weak) },
+  { "line_width", PROP_TYPE_REAL, offsetof(Entity, border_width) },
+  { "line_colour", PROP_TYPE_COLOUR, offsetof(Entity, border_color) },
+  { "fill_colour", PROP_TYPE_COLOUR, offsetof(Entity, inner_color) },
+  { NULL, 0, 0}
+};
+
+
+static void
+entity_get_props(Entity *entity, Property *props, guint nprops)
 {
-  EntityPropertiesDialog *prop_dialog;
-  GtkWidget *vbox;
-  GtkWidget *hbox;
-  GtkWidget *label;
-  GtkWidget *color;
-  GtkWidget *border_width;
-  GtkWidget *entry;
-  GtkWidget *checkbox;
-  GtkAdjustment *adj;
+  object_get_props_from_offsets(&entity->element.object, 
+                                entity_offsets, props, nprops);
+}
 
-  if (entity->properties_dialog == NULL) {
-  
-    prop_dialog = g_new(EntityPropertiesDialog, 1);
-    entity->properties_dialog = prop_dialog;
-
-    vbox = gtk_vbox_new(FALSE, 5);
-    gtk_object_ref(GTK_OBJECT(vbox));
-    gtk_object_sink(GTK_OBJECT(vbox));
-    prop_dialog->vbox = vbox;
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    label = gtk_label_new(_("Name:"));
-    entry = gtk_entry_new();
-    prop_dialog->name = GTK_ENTRY(entry);
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    checkbox = gtk_check_button_new_with_label(_("Weak"));
-    prop_dialog->weak = GTK_TOGGLE_BUTTON(checkbox);
-    gtk_box_pack_start (GTK_BOX (hbox), checkbox, TRUE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    label = gtk_label_new(_("Border width:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.00, 10.0, 0.01, 0.0, 0.0);
-    border_width = gtk_spin_button_new(adj, 1.0, 2);
-    gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(border_width), TRUE);
-    gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(border_width), TRUE);
-    prop_dialog->border_width = GTK_SPIN_BUTTON(border_width);
-    gtk_box_pack_start(GTK_BOX (hbox), border_width, TRUE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    label = gtk_label_new(_("Foreground color:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    color = dia_color_selector_new();
-    prop_dialog->fg_color = DIACOLORSELECTOR(color);
-    gtk_box_pack_start (GTK_BOX (hbox), color, TRUE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-
-    hbox = gtk_hbox_new(FALSE, 5);
-    label = gtk_label_new(_("Background color:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    color = dia_color_selector_new();
-    prop_dialog->bg_color = DIACOLORSELECTOR(color);
-    gtk_box_pack_start (GTK_BOX (hbox), color, TRUE, TRUE, 0);
-    gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
-
-    gtk_widget_show_all (vbox);
-  }
-
-  prop_dialog = entity->properties_dialog;
-    
-  gtk_spin_button_set_value(prop_dialog->border_width, entity->border_width);
-  dia_color_selector_set_color(prop_dialog->fg_color, &entity->border_color);
-  dia_color_selector_set_color(prop_dialog->bg_color, &entity->inner_color);
-  gtk_entry_set_text(prop_dialog->name, entity->name);
-  gtk_toggle_button_set_active(prop_dialog->weak, entity->weak);
-  
-  return prop_dialog->vbox;
+static void
+entity_set_props(Entity *entity, Property *props, guint nprops)
+{
+  object_set_props_from_offsets(&entity->element.object, 
+                                entity_offsets, props, nprops);
+  entity_update_data(entity);
 }
 
 static real
@@ -341,8 +268,12 @@ static void
 entity_update_data(Entity *entity)
 {
   Element *elem = &entity->element;
-  Object *obj = (Object *) entity;
-  
+  Object *obj = &elem->object;
+  ElementBBExtras *extra = &elem->extra_spacing;
+
+  entity->name_width =
+    font_string_width(entity->name, entity->font, FONT_HEIGHT);
+
   elem->width = entity->name_width + 2*TEXT_BORDER_WIDTH_X;
   elem->height = FONT_HEIGHT + 2*TEXT_BORDER_WIDTH_Y;
 
@@ -363,51 +294,14 @@ entity_update_data(Entity *entity)
   entity->connections[7].pos.x = elem->corner.x + elem->width;
   entity->connections[7].pos.y = elem->corner.y + elem->height;
 
+  extra->border_trans = entity->border_width/2.0;
   element_update_boundingbox(elem);
-  /* fix boundingentity for line_width: */
-  obj->bounding_box.top -= entity->border_width/2;
-  obj->bounding_box.left -= entity->border_width/2;
-  obj->bounding_box.bottom += entity->border_width/2;
-  obj->bounding_box.right += entity->border_width/2;
   
   obj->position = elem->corner;
   
   element_update_handles(elem);
 }
 
-static EntityState *
-entity_get_state(Entity *entity)
-{
-  EntityState *state = g_new(EntityState, 1);
-  
-  state->obj_state.free = NULL;
-
-  state->border_width = entity->border_width;
-  state->border_color = entity->border_color;
-  state->inner_color = entity->inner_color;
-  state->name = g_strdup(entity->name);
-  state->name_width = entity->name_width;
-  state->weak = entity->weak;
-
-  return state;
-}
-
-static void 
-entity_set_state(Entity *entity, EntityState *state)
-{
-  entity->border_width = state->border_width;
-  entity->border_color = state->border_color;
-  entity->inner_color = state->inner_color;
-  g_free(entity->name);
-  entity->name = g_strdup(state->name);
-  entity->name_width = state->name_width;
-  entity->weak = state->weak;
-
-  g_free(state);
-
-  entity_update_data(entity);
-}
-
 static Object *
 entity_create(Point *startpoint,
 	      void *user_data,
@@ -419,9 +313,9 @@ entity_create(Point *startpoint,
   Object *obj;
   int i;
 
-  entity = g_malloc(sizeof(Entity));
+  entity = g_malloc0(sizeof(Entity));
   elem = &entity->element;
-  obj = (Object *) entity;
+  obj = &elem->object;
   
   obj->type = &entity_type;
 
@@ -431,8 +325,6 @@ entity_create(Point *startpoint,
   elem->width = DEFAULT_WIDTH;
   elem->height = DEFAULT_WIDTH;
 
-  entity->properties_dialog = NULL;
-  
   entity->border_width =  attributes_get_default_linewidth();
   entity->border_color = attributes_get_foreground();
   entity->inner_color = attributes_get_background();
@@ -460,16 +352,12 @@ entity_create(Point *startpoint,
 
   *handle1 = NULL;
   *handle2 = obj->handles[0];  
-  return (Object *)entity;
+  return &entity->element.object;
 }
 
 static void
 entity_destroy(Entity *entity)
 {
-  if (entity->properties_dialog != NULL) {
-    gtk_widget_destroy(entity->properties_dialog->vbox);
-    g_free(entity->properties_dialog);
-  }
   element_destroy(&entity->element);
   g_free(entity->name);
 }
@@ -484,9 +372,9 @@ entity_copy(Entity *entity)
   
   elem = &entity->element;
   
-  newentity = g_malloc(sizeof(Entity));
+  newentity = g_malloc0(sizeof(Entity));
   newelem = &newentity->element;
-  newobj = (Object *) newentity;
+  newobj = &newelem->object;
 
   element_copy(elem, newelem);
 
@@ -508,9 +396,7 @@ entity_copy(Entity *entity)
 
   newentity->weak = entity->weak;
 
-  newentity->properties_dialog = NULL;
-
-  return (Object *)newentity;
+  return &newentity->element.object;
 }
 
 static void
@@ -539,17 +425,15 @@ entity_load(ObjectNode obj_node, int version, const char *filename)
   int i;
   AttributeNode attr;
 
-  entity = g_malloc(sizeof(Entity));
+  entity = g_malloc0(sizeof(Entity));
   elem = &entity->element;
-  obj = (Object *) entity;
+  obj = &elem->object;
   
   obj->type = &entity_type;
   obj->ops = &entity_ops;
 
   element_load(elem, obj_node);
   
-  entity->properties_dialog = NULL;
-
   entity->border_width = 0.1;
   attr = object_find_attribute(obj_node, "border_width");
   if (attr != NULL)
@@ -593,5 +477,5 @@ entity_load(ObjectNode obj_node, int version, const char *filename)
     obj->handles[i]->type = HANDLE_NON_MOVABLE;
   }
 
-  return (Object *)entity;
+  return &entity->element.object;
 }
