@@ -20,13 +20,16 @@
  * please send e-mail to David Thompson <dcthomp@mail.utexas.edu>
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
 #include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 #include <stdio.h>
 
-#include "config.h"
 #include "intl.h"
 #include "object.h"
 #include "objchange.h"
@@ -37,6 +40,8 @@
 #include "diamenu.h"
 #include "text.h"
 #include "orth_conn.h"
+#include "element.h"
+#include "properties.h"
 
 #include "pixmaps/orthflow.xpm"
 
@@ -98,7 +103,6 @@ Color orthflow_color_signal   = { 0.0f, 0.0f, 1.0f };
 
 static Font *orthflow_font = NULL;
 
-static OrthflowDialog *properties_dialog;
 static OrthflowDialog *defaults_dialog;
 
 /* Remember the most recently applied orthflow type and use it to
@@ -121,10 +125,13 @@ static real orthflow_distance_from(Orthflow *orthflow, Point *point);
 static void orthflow_update_data(Orthflow *orthflow);
 static void orthflow_destroy(Orthflow *orthflow);
 static Object *orthflow_copy(Orthflow *orthflow);
-static GtkWidget *orthflow_get_properties(Orthflow *orthflow);
-static ObjectChange *orthflow_apply_properties(Orthflow *orthflow);
 static GtkWidget *orthflow_get_defaults(void);
 static void orthflow_apply_defaults(void);
+static PropDescription *orthflow_describe_props(Orthflow *mes);
+static void
+orthflow_get_props(Orthflow * orthflow, Property *props, guint nprops);
+static void
+orthflow_set_props(Orthflow * orthflow, Property *props, guint nprops);
 static void orthflow_save(Orthflow *orthflow, ObjectNode obj_node,
 			  const char *filename);
 static Object *orthflow_load(ObjectNode obj_node, int version,
@@ -158,11 +165,60 @@ static ObjectOps orthflow_ops = {
   (CopyFunc)            orthflow_copy,
   (MoveFunc)            orthflow_move,
   (MoveHandleFunc)      orthflow_move_handle,
-  (GetPropertiesFunc)   orthflow_get_properties,
-  (ApplyPropertiesFunc) orthflow_apply_properties,
+  (GetPropertiesFunc)   object_create_props_dialog,
+  (ApplyPropertiesFunc) object_apply_props_from_dialog,
   (ObjectMenuFunc)      orthflow_get_object_menu,
+  (DescribePropsFunc)   orthflow_describe_props,
+  (GetPropsFunc)        orthflow_get_props,
+  (SetPropsFunc)        orthflow_set_props,
 };
 
+static PropEnumData prop_orthflow_type_data[] = {
+  { N_("Energy"),ORTHFLOW_ENERGY },
+  { N_("Material"),ORTHFLOW_MATERIAL },
+  { N_("Signal"),ORTHFLOW_SIGNAL },
+  { NULL, 0 } 
+};
+
+static PropDescription orthflow_props[] = {
+  ELEMENT_COMMON_PROPERTIES,
+  { "type", PROP_TYPE_ENUM, PROP_FLAG_VISIBLE,
+    N_("Type:"), NULL, prop_orthflow_type_data },
+  PROP_DESC_END
+};
+
+static PropDescription *
+orthflow_describe_props(Orthflow *mes)
+{
+  if (orthflow_props[0].quark == 0)
+    prop_desc_list_calculate_quarks(orthflow_props);
+  return orthflow_props;
+
+}
+
+static PropOffset orthflow_offsets[] = {
+  OBJECT_COMMON_PROPERTIES_OFFSETS,
+  { "type", PROP_TYPE_ENUM, offsetof(Orthflow, type) },
+  { NULL, 0, 0 }
+};
+
+static void
+orthflow_get_props(Orthflow * orthflow, Property *props, guint nprops)
+{
+  object_get_props_from_offsets(&orthflow->orth.object, 
+                                orthflow_offsets, props, nprops);
+}
+
+static void
+orthflow_set_props(Orthflow *orthflow, Property *props, guint nprops)
+{
+  object_set_props_from_offsets(&orthflow->orth.object, 
+                                orthflow_offsets, props, nprops);
+  orthflow_update_data(orthflow);
+}
+
+
+
 static void
 orthflow_change_apply_revert(ObjectChange* objchg, Object* obj)
 {
@@ -199,7 +255,7 @@ orthflow_create_change( enum OrthflowChangeType change_type,
 			OrthflowType type, Text* text )
 {
   struct _OrthflowChange* change ;
-  change = g_new( struct _OrthflowChange, 1 ) ;
+  change = g_new0( struct _OrthflowChange, 1 ) ;
   change->obj_change.apply = (ObjectChangeApplyFunc) orthflow_change_apply_revert ;
   change->obj_change.revert =  (ObjectChangeRevertFunc) orthflow_change_apply_revert ;
   change->obj_change.free =  (ObjectChangeFreeFunc) orthflow_change_free ;
@@ -233,7 +289,7 @@ orthflow_select(Orthflow *orthflow, Point *clicked_point,
 		Renderer *interactive_renderer)
 {
   text_set_cursor(orthflow->text, clicked_point, interactive_renderer);
-  text_grab_focus(orthflow->text, (Object *)orthflow);
+  text_grab_focus(orthflow->text, &orthflow->orth.object);
 
   orthconn_update_data(&orthflow->orth);
 }
@@ -338,13 +394,14 @@ orthflow_create(Point *startpoint,
   OrthConn *orth;
   Object *obj;
   Point p;
-
-  orthflow = g_new(Orthflow,1);
+  OrthConnBBExtras *extra;
+  orthflow = g_new0(Orthflow,1);
 
   orth = &orthflow->orth ;
   orthconn_init( orth, startpoint ) ;
  
-  obj = (Object *) orthflow;
+  obj = &orth->object;
+  extra = &orth->extra_spacing;
   
   obj->type = &orthflow_type;
   obj->ops = &orthflow_ops;
@@ -385,13 +442,19 @@ orthflow_create(Point *startpoint,
   orthflow->text_handle.connect_type = HANDLE_NONCONNECTABLE;
   orthflow->text_handle.connected_to = NULL;
   object_add_handle( obj, &orthflow->text_handle ) ;
-  
+
+  extra->start_long = 
+    extra->start_trans = 
+    extra->middle_trans = ORTHFLOW_WIDTH/2.0;
+  extra->end_long = 
+    extra->end_trans = ORTHFLOW_WIDTH/2 + ORTHFLOW_ARROWLEN;
+    
   orthflow_update_data(orthflow);
 
   /*obj->handles[1] = orth->handles[2] ;*/
   *handle1 = obj->handles[0];
   *handle2 = obj->handles[2];
-  return (Object *)orthflow;
+  return &orthflow->orth.object;
 }
 
 
@@ -411,19 +474,19 @@ orthflow_copy(Orthflow *orthflow)
 
   orth = &orthflow->orth;
   
-  neworthflow = g_malloc(sizeof(Orthflow));
+  neworthflow = g_malloc0(sizeof(Orthflow));
   neworth = &neworthflow->orth;
-  newobj = (Object *) neworthflow;
+  newobj = &neworth->object;
 
   orthconn_copy(orth, neworth);
 
   neworthflow->text_handle = orthflow->text_handle;
-  newobj->handles[2] = &neworthflow->text_handle;
+  newobj->handles[orth->numpoints-1] = &neworthflow->text_handle;
 
   neworthflow->text = text_copy(orthflow->text);
   neworthflow->type = orthflow->type;
 
-  return (Object *)neworthflow;
+  return &neworthflow->orth.object;
 }
 
 
@@ -431,7 +494,7 @@ static void
 orthflow_update_data(Orthflow *orthflow)
 {
   OrthConn *orth = &orthflow->orth ;
-  Object *obj = (Object *) orthflow;
+  Object *obj = &orth->object;
   Rectangle rect;
   Color* color = &orthflow_color_signal;
   
@@ -459,12 +522,6 @@ orthflow_update_data(Orthflow *orthflow)
   /* Add boundingbox for text: */
   text_calc_boundingbox(orthflow->text, &rect) ;
   rectangle_union(&obj->bounding_box, &rect);
-
-  /* fix boundingbox for orthflow_width and arrow: */
-  obj->bounding_box.top -= ORTHFLOW_WIDTH/2 + ORTHFLOW_ARROWLEN;
-  obj->bounding_box.left -= ORTHFLOW_WIDTH/2 + ORTHFLOW_ARROWLEN;
-  obj->bounding_box.bottom += ORTHFLOW_WIDTH/2 + ORTHFLOW_ARROWLEN;
-  obj->bounding_box.right += ORTHFLOW_WIDTH/2 + ORTHFLOW_ARROWLEN;
 }
 
 
@@ -486,15 +543,17 @@ orthflow_load(ObjectNode obj_node, int version, const char *filename)
   AttributeNode attr;
   OrthConn *orth;
   Object *obj;
+  OrthConnBBExtras *extra;
 
   if (orthflow_font == NULL)
     orthflow_font = font_getfont("Helvetica-Oblique");
 
-  orthflow = g_malloc(sizeof(Orthflow));
+  orthflow = g_malloc0(sizeof(Orthflow));
 
   orth = &orthflow->orth;
-  obj = (Object *) orthflow;
-
+  obj = &orth->object;
+  extra = &orth->extra_spacing;
+  
   obj->type = &orthflow_type;
   obj->ops = &orthflow_ops;
 
@@ -513,14 +572,57 @@ orthflow_load(ObjectNode obj_node, int version, const char *filename)
   orthflow->text_handle.type = HANDLE_MINOR_CONTROL;
   orthflow->text_handle.connect_type = HANDLE_NONCONNECTABLE;
   orthflow->text_handle.connected_to = NULL;
-  obj->handles[2] = &orthflow->text_handle;
+  /* Mein Gott!  The extra handle was never added */
+  object_add_handle(obj, &orthflow->text_handle);
+  obj->handles[orth->numpoints-1] = &orthflow->text_handle;
+
+  extra->start_long = 
+    extra->start_trans = 
+    extra->middle_trans = ORTHFLOW_WIDTH/2.0;
+  extra->end_long = 
+    extra->end_trans = ORTHFLOW_WIDTH/2 + ORTHFLOW_ARROWLEN;
   
   orthflow_update_data(orthflow);
   
-  return (Object *)orthflow;
+  return &orthflow->orth.object;
 }
 
 
+
+static void
+fill_in_defaults_dialog()
+{
+  OrthflowDialog *prop_dialog;
+  GtkToggleButton *button=NULL;
+
+  prop_dialog = defaults_dialog;
+
+  if ( orthflow_default_label ) {
+    gtk_text_set_point( GTK_TEXT(prop_dialog->text), 0 ) ;
+    gtk_text_forward_delete( GTK_TEXT(prop_dialog->text), 
+			     gtk_text_get_length(GTK_TEXT(prop_dialog->text))) ;
+    gtk_text_insert( GTK_TEXT(prop_dialog->text),
+		     NULL, NULL, NULL,
+		     text_get_string_copy(orthflow_default_label),
+		     -1) ;
+  }
+
+  switch (orthflow_most_recent_type) {
+  case ORTHFLOW_ENERGY:
+    button = GTK_TOGGLE_BUTTON(prop_dialog->m_energy);
+    break;
+  case ORTHFLOW_MATERIAL:
+    button = GTK_TOGGLE_BUTTON(prop_dialog->m_material);
+    break;
+  case ORTHFLOW_SIGNAL:
+    button = GTK_TOGGLE_BUTTON(prop_dialog->m_signal);
+    break;
+  }
+  if (button)
+    gtk_toggle_button_set_active(button, TRUE);
+}
+
+#if 0
 static ObjectChange *
 orthflow_apply_properties(Orthflow *orthflow)
 {
@@ -579,39 +681,6 @@ fill_in_dialog(Orthflow *orthflow)
 }
 
 
-static void
-fill_in_defaults_dialog()
-{
-  OrthflowDialog *prop_dialog;
-  GtkToggleButton *button=NULL;
-
-  prop_dialog = defaults_dialog;
-
-  if ( orthflow_default_label ) {
-    gtk_text_set_point( GTK_TEXT(prop_dialog->text), 0 ) ;
-    gtk_text_forward_delete( GTK_TEXT(prop_dialog->text), 
-			     gtk_text_get_length(GTK_TEXT(prop_dialog->text))) ;
-    gtk_text_insert( GTK_TEXT(prop_dialog->text),
-		     NULL, NULL, NULL,
-		     text_get_string_copy(orthflow_default_label),
-		     -1) ;
-  }
-
-  switch (orthflow_most_recent_type) {
-  case ORTHFLOW_ENERGY:
-    button = GTK_TOGGLE_BUTTON(prop_dialog->m_energy);
-    break;
-  case ORTHFLOW_MATERIAL:
-    button = GTK_TOGGLE_BUTTON(prop_dialog->m_material);
-    break;
-  case ORTHFLOW_SIGNAL:
-    button = GTK_TOGGLE_BUTTON(prop_dialog->m_signal);
-    break;
-  }
-  if (button)
-    gtk_toggle_button_set_active(button, TRUE);
-}
-
 
 static GtkWidget *
 orthflow_get_properties(Orthflow *orthflow)
@@ -682,6 +751,7 @@ orthflow_get_properties(Orthflow *orthflow)
 
   return properties_dialog->dialog;
 }
+#endif
 
 static void
 orthflow_update_defaults( Orthflow* orthflow, char update_text )
