@@ -20,6 +20,11 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
 #include <gtk/gtk.h>
 #include <math.h>
@@ -27,7 +32,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "config.h"
 #include "intl.h"
 #include "object.h"
 #include "element.h"
@@ -151,8 +155,10 @@ static Object *step_load(ObjectNode obj_node, int version,
 			   const char *filename);
 static StepState *step_get_state(Step *step);
 static void step_set_state(Step *step, StepState *state);
-static PROPDLG_TYPE step_get_defaults();
-static void step_apply_defaults();
+static PROPDLG_TYPE step_get_defaults(void);
+static void step_apply_defaults(void);
+static void step_init_defaults(void);
+
 
 
 static ObjectTypeOps step_type_ops =
@@ -242,7 +248,8 @@ step_apply_properties(Step *step)
 
   step_been_renamed(step->id);
   step_update_data(step);
-  return new_object_state_change((Object *)step, old_state,
+  return new_object_state_change(&step->element.object, 
+                                 old_state,
 				 (GetStateFunc)step_get_state,
 				 (SetStateFunc)step_set_state);
 }
@@ -277,7 +284,7 @@ step_get_properties(Step *step)
 static Color color_red = { 1.0f, 0.0f, 0.0f };
 
 static void 
-step_init_defaults()
+step_init_defaults(void)
 {
   static int defaults_initialised = 0;
   if (!defaults_initialised) {
@@ -289,7 +296,7 @@ step_init_defaults()
 }
 
 static void 
-step_apply_defaults()
+step_apply_defaults(void)
 {
   StepDefaultsDialog *dlg = step_defaults_dialog;  
 
@@ -299,7 +306,7 @@ step_apply_defaults()
 }
 
 static PROPDLG_TYPE
-step_get_defaults()
+step_get_defaults(void)
 {
   StepDefaultsDialog *dlg = step_defaults_dialog;
 
@@ -445,7 +452,8 @@ static void
 step_update_data(Step *step)
 {
   Element *elem = &step->element;
-  Object *obj = (Object *) step;
+  Object *obj = &elem->object;
+  ElementBBExtras *extra = &elem->extra_spacing;
   Point *p,ulc;
 
   ulc = elem->corner;
@@ -528,22 +536,16 @@ step_update_data(Step *step)
   step->connections[3].pos = step->H;
 
   /* recalc the bounding box : */
+  if ((step->type == STEP_INITIAL) || (step->type == STEP_SUBPCALL)) {
+    extra->border_trans = 2.5 * STEP_LINE_WIDTH;
+  } else {
+    extra->border_trans = STEP_LINE_WIDTH / 2;
+  }
   
   element_update_boundingbox(elem);
   rectangle_add_point(&obj->bounding_box,&step->north.pos);
   rectangle_add_point(&obj->bounding_box,&step->south.pos);
 
-  /* fix boundingbox for line_width: */
-  obj->bounding_box.top -= STEP_LINE_WIDTH/2;
-  obj->bounding_box.left -= STEP_LINE_WIDTH/2 ;
-  obj->bounding_box.bottom += STEP_LINE_WIDTH/2;
-  obj->bounding_box.right += STEP_LINE_WIDTH/2;
-
-  if ((step->type == STEP_INITIAL) || (step->type == STEP_SUBPCALL)) {
-    obj->bounding_box.left -= 2*STEP_LINE_WIDTH;
-    obj->bounding_box.right += 2*STEP_LINE_WIDTH;
-  }  
-
   obj->position = elem->corner;
   
   element_update_handles(elem);
@@ -552,7 +554,7 @@ step_update_data(Step *step)
 static StepState *
 step_get_state(Step *step)
 {
-  StepState *state = g_new(StepState, 1);
+  StepState *state = g_new0(StepState, 1);
   state->obj_state.free = NULL;
 
   state->id = g_strdup(step->id);
@@ -597,7 +599,7 @@ step_create(Point *startpoint,
 
   step = g_new0(Step,1);
   elem = &step->element;
-  obj = (Object *) step;
+  obj = &elem->object;
   
   obj->type = &step_type;
   obj->ops = &step_ops;
@@ -652,7 +654,7 @@ step_create(Point *startpoint,
 
   *handle1 = NULL;
   *handle2 = obj->handles[0];  
-  return (Object *)step;
+  return &step->element.object;
 }
 
 static void
@@ -668,13 +670,13 @@ step_copy(Step *step)
   int i;
   Step *newstep;
   Element *elem, *newelem;
-  Object *newobj,*obj;
+  Object *newobj;
 
   elem = &step->element;
   
   newstep = g_new0(Step,1);
   newelem = &newstep->element;
-  newobj = (Object *) newstep; obj = (Object *)obj;
+  newobj = &newelem->object;
 
   element_copy(elem, newelem);
 
@@ -705,7 +707,7 @@ step_copy(Step *step)
 
   step_update_data(newstep);
 
-  return (Object *)newstep;
+  return &newstep->element.object;
 }
 
 static void
@@ -736,7 +738,7 @@ step_load(ObjectNode obj_node, int version, const char *filename)
 
   step = g_new0(Step,1);
   elem = &step->element;
-  obj = (Object *)step;
+  obj = &elem->object;
   
   obj->type = &step_type;
   obj->ops = &step_ops;
@@ -777,7 +779,7 @@ step_load(ObjectNode obj_node, int version, const char *filename)
   obj->handles[8] = &step->north;
   obj->handles[9] = &step->south;
 
-  return (Object *)step;
+  return &step->element.object;
 }
 
 
