@@ -21,13 +21,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
 #include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 #include <glib.h>
 
-#include "config.h"
 #include "intl.h"
 #include "object.h"
 #include "element.h"
@@ -136,8 +140,8 @@ static void sadtbox_set_state(Box *box, BoxState *state);
 
 static void sadtbox_save(Box *box, ObjectNode obj_node, const char *filename);
 static Object *sadtbox_load(ObjectNode obj_node, int version, const char *filename);
-static PROPDLG_TYPE sadtbox_get_defaults();
-static void sadtbox_apply_defaults();
+static PROPDLG_TYPE sadtbox_get_defaults(void);
+static void sadtbox_apply_defaults(void);
 
 static DiaMenu *sadtbox_get_object_menu(Box *box, Point *clickedpoint);
 
@@ -189,7 +193,8 @@ sadtbox_apply_properties(Box *box)
   PROPDLG_APPLY_STRING(dlg,id);
 
   sadtbox_update_data(box, ANCHOR_MIDDLE, ANCHOR_MIDDLE);
-  return new_object_state_change((Object *)box, old_state, 
+  return new_object_state_change(&box->element.object, 
+                                 old_state, 
 				 (GetStateFunc)sadtbox_get_state,
 				 (SetStateFunc)sadtbox_set_state);
 }
@@ -213,7 +218,7 @@ sadtbox_get_properties(Box *box)
   PROPDLG_RETURN(dlg);
 }
 static void
-sadtbox_apply_defaults()
+sadtbox_apply_defaults(void)
 {
   BoxDefaultsDialog *dlg = sadtbox_defaults_dialog;
 
@@ -224,7 +229,7 @@ sadtbox_apply_defaults()
 }
 
 static void
-init_default_values() {
+init_default_values(void) {
   static int defaults_initialized = 0;
 
   if (!defaults_initialized) {
@@ -237,7 +242,7 @@ init_default_values() {
 }
 
 static GtkWidget *
-sadtbox_get_defaults()
+sadtbox_get_defaults(void)
 {
   BoxDefaultsDialog *dlg = sadtbox_defaults_dialog;
 
@@ -271,7 +276,7 @@ sadtbox_select(Box *box, Point *clicked_point,
 	   Renderer *interactive_renderer)
 {
   text_set_cursor(box->text, clicked_point, interactive_renderer);
-  text_grab_focus(box->text, (Object *)box);
+  text_grab_focus(box->text, &box->element.object);
   element_update_handles(&box->element);
 }
 
@@ -374,7 +379,7 @@ sadtbox_free_state(ObjectState *objstate)
 static BoxState *
 sadtbox_get_state(Box *box)
 {
-  BoxState *state = g_new(BoxState, 1);
+  BoxState *state = g_new0(BoxState, 1);
 
   state->obj_state.free = sadtbox_free_state;
 
@@ -402,7 +407,8 @@ static void
 sadtbox_update_data(Box *box, AnchorShape horiz, AnchorShape vert)
 {
   Element *elem = &box->element;
-  Object *obj = (Object *) box;
+  ElementBBExtras *extra = &elem->extra_spacing;
+  Object *obj = &elem->object;
   Point center, bottom_right;
   Point p;
   real width, height;
@@ -415,9 +421,8 @@ sadtbox_update_data(Box *box, AnchorShape horiz, AnchorShape vert)
   center.y += elem->height/2;
   bottom_right.y += elem->height;
 
-  width = box->text->max_width + box->padding*2 + SADTBOX_LINE_WIDTH;
-  height = box->text->height * box->text->numlines + box->padding*2 +
-    SADTBOX_LINE_WIDTH;
+  width = box->text->max_width + box->padding*2;
+  height = box->text->height * box->text->numlines + box->padding*2;
 
   if (width > elem->width) elem->width = width;
   if (height > elem->height) elem->height = height;
@@ -446,12 +451,8 @@ sadtbox_update_data(Box *box, AnchorShape horiz, AnchorShape vert)
     font_ascent(box->text->font, box->text->height);
   text_set_position(box->text, &p);
 
+  extra->border_trans = SADTBOX_LINE_WIDTH / 2.0;
   element_update_boundingbox(elem);
-  /* fix boundingbox for line_width: */
-  obj->bounding_box.top -= SADTBOX_LINE_WIDTH/2;
-  obj->bounding_box.left -= SADTBOX_LINE_WIDTH/2;
-  obj->bounding_box.bottom += SADTBOX_LINE_WIDTH/2;
-  obj->bounding_box.right += SADTBOX_LINE_WIDTH/2;
   
   obj->position = elem->corner;
   
@@ -575,7 +576,7 @@ sadtbox_create(Point *startpoint,
 
   box = g_malloc0(sizeof(Box));
   elem = &box->element;
-  obj = (Object *) box;
+  obj = &elem->object;
   
   obj->type = &sadtbox_type;
 
@@ -604,11 +605,12 @@ sadtbox_create(Point *startpoint,
   box->south = connpointline_create(obj,1); 
   box->east = connpointline_create(obj,3);
 
+  box->element.extra_spacing.border_trans = SADTBOX_LINE_WIDTH/2.0;
   sadtbox_update_data(box, ANCHOR_MIDDLE, ANCHOR_MIDDLE);
 
   *handle1 = NULL;
   *handle2 = obj->handles[7];  
-  return (Object *)box;
+  return &box->element.object;
 }
 
 static void
@@ -636,9 +638,9 @@ sadtbox_copy(Box *box)
 
   elem = &box->element;
   
-  newbox = g_malloc(sizeof(Box));
+  newbox = g_malloc0(sizeof(Box));
   newelem = &newbox->element;
-  newobj = (Object *) newbox;
+  newobj = &newelem->object;
 
   element_copy(elem, newelem);
   rcc = 0;
@@ -650,10 +652,9 @@ sadtbox_copy(Box *box)
   
   newbox->id = g_strdup(box->id);
   newbox->padding = box->padding;
-  newbox->text = text_copy(box->text);
-  
+  newbox->text = text_copy(box->text);  
 
-  return (Object *)newbox;
+  return &newbox->element.object;
 }
 
 
@@ -680,9 +681,9 @@ sadtbox_load(ObjectNode obj_node, int version, const char *filename)
   int realconncount;
   Point p = {0.0,0.0};
 
-  box = g_malloc(sizeof(Box));
+  box = g_malloc0(sizeof(Box));
   elem = &box->element;
-  obj = (Object *) box;
+  obj = &elem->object;
   
   obj->type = &sadtbox_type;
   obj->ops = &sadtbox_ops;
@@ -708,9 +709,10 @@ sadtbox_load(ObjectNode obj_node, int version, const char *filename)
   box->id = load_string(obj_node,"id",NULL);
   if (!box->id) box->id = g_strdup("A0");
   
+  box->element.extra_spacing.border_trans = SADTBOX_LINE_WIDTH/2.0;
   sadtbox_update_data(box, ANCHOR_MIDDLE, ANCHOR_MIDDLE);
 
-  return (Object *)box;
+  return &box->element.object;
 }
 
 
