@@ -15,12 +15,16 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
 #include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 
-#include "config.h"
 #include "intl.h"
 #include "object.h"
 #include "orth_conn.h"
@@ -356,7 +360,7 @@ association_get_state(Association *assoc)
   int i;
   AssociationEnd *end;
 
-  AssociationState *state = g_new(AssociationState, 1);
+  AssociationState *state = g_new0(AssociationState, 1);
 
   state->obj_state.free = association_state_free;
 
@@ -421,20 +425,26 @@ static void
 association_update_data(Association *assoc)
 {
   OrthConn *orth = &assoc->orth;
-  Object *obj = (Object *) assoc;
+  Object *obj = &orth->object;
+  OrthConnBBExtras *extra = &orth->extra_spacing;
   int num_segm, i, n;
   Point *points;
   Rectangle rect;
   AssociationEnd *end;
   
-  orthconn_update_data(orth);
+  orthconn_update_data(orth);  
   
+  extra->start_trans = 
+    extra->start_long = (assoc->end[0].aggregate == AGGREGATE_NONE?
+                         ASSOCIATION_WIDTH/2.0:
+                         (ASSOCIATION_WIDTH + ASSOCIATION_DIAMONDLEN)/2.0);
+  extra->middle_trans = ASSOCIATION_WIDTH/2.0;
+  extra->end_trans = 
+    extra->end_long = (assoc->end[1].aggregate == AGGREGATE_NONE?
+                         ASSOCIATION_WIDTH/2.0:
+                         (ASSOCIATION_WIDTH + ASSOCIATION_DIAMONDLEN)/2.0);
+
   orthconn_update_boundingbox(orth);
-  /* fix boundingassociation for linewidth and triangle: */
-  obj->bounding_box.top -= ASSOCIATION_WIDTH/2.0 + ASSOCIATION_DIAMONDLEN;
-  obj->bounding_box.left -= ASSOCIATION_WIDTH/2.0 + ASSOCIATION_DIAMONDLEN;
-  obj->bounding_box.bottom += ASSOCIATION_WIDTH/2.0 + ASSOCIATION_DIAMONDLEN;
-  obj->bounding_box.right += ASSOCIATION_WIDTH/2.0 + ASSOCIATION_DIAMONDLEN;
   
   /* Calc text pos: */
   num_segm = assoc->orth.numpoints - 1;
@@ -519,7 +529,7 @@ association_update_data(Association *assoc)
     end->text_align = ALIGN_LEFT;
     break;
   }
-  /* Add the text recangle to the bounding box: */
+  /* Add the text rectangle to the bounding box: */
   rect.left = end->text_pos.x;
   rect.right = rect.left + end->text_width;
   rect.top = end->text_pos.y - font_ascent(assoc_font, ASSOCIATION_FONTHEIGHT);
@@ -544,10 +554,10 @@ association_create(Point *startpoint,
     assoc_font = font_getfont("Courier");
   }
   
-  assoc = g_malloc(sizeof(Association));
+  assoc = g_malloc0(sizeof(Association));
   orth = &assoc->orth;
-  obj = (Object *) assoc;
-  
+  obj = &orth->object;
+
   obj->type = &association_type;
 
   obj->ops = &association_ops;
@@ -582,7 +592,7 @@ association_create(Point *startpoint,
   *handle1 = orth->handles[0];
   *handle2 = orth->handles[orth->numpoints-2];
 
-  return (Object *)assoc;
+  return &assoc->orth.object;
 }
 
 static ObjectChange *
@@ -658,9 +668,9 @@ association_copy(Association *assoc)
   
   orth = &assoc->orth;
   
-  newassoc = g_malloc(sizeof(Association));
+  newassoc = g_malloc0(sizeof(Association));
   neworth = &newassoc->orth;
-  newobj = (Object *) newassoc;
+  newobj = &neworth->object;
 
   orthconn_copy(orth, neworth);
 
@@ -679,7 +689,7 @@ association_copy(Association *assoc)
   
   association_update_data(newassoc);
   
-  return (Object *)newassoc;
+  return &newassoc->orth.object;
 }
 
 
@@ -727,10 +737,10 @@ association_load(ObjectNode obj_node, int version, const char *filename)
     assoc_font = font_getfont("Courier");
   }
 
-  assoc = g_new(Association, 1);
+  assoc = g_new0(Association, 1);
 
   orth = &assoc->orth;
-  obj = (Object *) assoc;
+  obj = &orth->object;
 
   obj->type = &association_type;
   obj->ops = &association_ops;
@@ -796,7 +806,7 @@ association_load(ObjectNode obj_node, int version, const char *filename)
   
   association_update_data(assoc);
 
-  return (Object *)assoc;
+  return &assoc->orth.object;
 }
 
 static ObjectChange *
@@ -872,7 +882,7 @@ association_apply_properties(Association *assoc)
   }
 
   association_update_data(assoc);
-  return new_object_state_change((Object *)assoc, old_state, 
+  return new_object_state_change(&assoc->orth.object, old_state, 
 				 (GetStateFunc)association_get_state,
 				 (SetStateFunc)association_set_state);
 }
