@@ -16,12 +16,15 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
 #include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 
-#include "config.h"
 #include "object.h"
 #include "objchange.h"
 #include "intl.h"
@@ -94,7 +97,7 @@ umlclass_store_disconnects(UMLClassDialog *prop_dialog,
     
     for (i=0;i<connected_obj->num_handles;i++) {
       if (connected_obj->handles[i]->connected_to == cp) {
-	dis = g_new(Disconnect, 1);
+	dis = g_new0(Disconnect, 1);
 	dis->cp = cp;
 	dis->other_object = connected_obj;
 	dis->other_handle = connected_obj->handles[i];
@@ -132,6 +135,8 @@ class_read_from_dialog(UMLClass *umlclass, UMLClassDialog *prop_dialog)
   umlclass->visible_operations = prop_dialog->op_vis->active;
   umlclass->suppress_attributes = prop_dialog->attr_supp->active;
   umlclass->suppress_operations = prop_dialog->op_supp->active;
+  umlclass->color_foreground = prop_dialog->fg_color->col;
+  umlclass->color_background = prop_dialog->bg_color->col;
 }
 
 static void
@@ -152,6 +157,8 @@ class_fill_in_dialog(UMLClass *umlclass)
   gtk_toggle_button_set_active(prop_dialog->op_vis, umlclass->visible_operations);
   gtk_toggle_button_set_active(prop_dialog->attr_supp, umlclass->suppress_attributes);
   gtk_toggle_button_set_active(prop_dialog->op_supp, umlclass->suppress_operations);
+  dia_color_selector_set_color(prop_dialog->fg_color, &umlclass->color_foreground);
+  dia_color_selector_set_color(prop_dialog->bg_color, &umlclass->color_background);
 }
 
 static void 
@@ -164,6 +171,8 @@ class_create_page(GtkNotebook *notebook,  UMLClass *umlclass)
   GtkWidget *vbox;
   GtkWidget *entry;
   GtkWidget *checkbox;
+  GtkWidget *color_background;
+  GtkWidget *color_foreground;
 
   prop_dialog = umlclass->properties_dialog;
 
@@ -226,7 +235,27 @@ class_create_page(GtkNotebook *notebook,  UMLClass *umlclass)
 		      checkbox, TRUE, TRUE, 0);
   gtk_box_pack_start (GTK_BOX (vbox),
 		      hbox, FALSE, TRUE, 0);
+
+  hbox = gtk_hbox_new(FALSE, 5);
+  label = gtk_label_new(_("Foreground Color"));
+  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+  gtk_box_pack_start( GTK_BOX(hbox), label, TRUE, TRUE, 0);
+  color_foreground = dia_color_selector_new();
+  dia_color_selector_set_color((DiaColorSelector *)color_foreground, &umlclass->color_foreground);
+  prop_dialog->fg_color = (DiaColorSelector *)color_foreground;
+  gtk_box_pack_end( GTK_BOX(hbox), color_foreground, TRUE, TRUE,0);
+  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);
   
+  hbox = gtk_hbox_new(FALSE, 5);
+  label = gtk_label_new(_("Background Color"));
+  gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+  gtk_box_pack_start( GTK_BOX(hbox), label, TRUE, TRUE, 0);
+  color_background = dia_color_selector_new();
+  dia_color_selector_set_color((DiaColorSelector *)color_background, &umlclass->color_background);
+  prop_dialog->bg_color = (DiaColorSelector *)color_background;
+  gtk_box_pack_end( GTK_BOX(hbox), color_background, TRUE, TRUE,0);
+  gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, TRUE, 0);
+
   gtk_widget_show_all (vbox);
   gtk_widget_show (page_label);
   gtk_notebook_append_page(notebook, vbox, page_label);
@@ -502,7 +531,7 @@ attributes_read_from_dialog(UMLClass *umlclass,
   GList *clear_list;
   Object *obj;
 
-  obj = (Object *) umlclass;
+  obj = &umlclass->element.object;
 
   attributes_get_current_values(prop_dialog); /* if changed, update from widgets */
   /* Free current attributes: */
@@ -528,11 +557,11 @@ attributes_read_from_dialog(UMLClass *umlclass,
     umlclass->attributes = g_list_append(umlclass->attributes, attr);
     
     if (attr->left_connection == NULL) {
-      attr->left_connection = g_new(ConnectionPoint,1);
+      attr->left_connection = g_new0(ConnectionPoint,1);
       attr->left_connection->object = obj;
       attr->left_connection->connected = NULL;
       
-      attr->right_connection = g_new(ConnectionPoint,1);
+      attr->right_connection = g_new0(ConnectionPoint,1);
       attr->right_connection->object = obj;
       attr->right_connection->connected = NULL;
 
@@ -1374,7 +1403,7 @@ operations_read_from_dialog(UMLClass *umlclass,
   GList *clear_list;
   Object *obj;
 
-  obj = (Object *) umlclass;
+  obj = &umlclass->element.object;
 
   operations_get_current_values(prop_dialog); /* if changed, update from widgets */
   /* Free current operations: */
@@ -1400,11 +1429,11 @@ operations_read_from_dialog(UMLClass *umlclass,
     umlclass->operations = g_list_append(umlclass->operations, op);
     
     if (op->left_connection == NULL) {
-      op->left_connection = g_new(ConnectionPoint,1);
+      op->left_connection = g_new0(ConnectionPoint,1);
       op->left_connection->object = obj;
       op->left_connection->connected = NULL;
       
-      op->right_connection = g_new(ConnectionPoint,1);
+      op->right_connection = g_new0(ConnectionPoint,1);
       op->right_connection->object = obj;
       op->right_connection->connected = NULL;
       
@@ -2286,7 +2315,7 @@ umlclass_apply_properties(UMLClass *umlclass)
     num_ops = g_list_length(prop_dialog->operations_list->children);
   else
     num_ops = 0;
-  obj = (Object *) umlclass;
+  obj = &umlclass->element.object;
   obj->num_connections = 8 + num_attrib*2 + num_ops*2;
   obj->connections =
     g_realloc(obj->connections,
@@ -2421,7 +2450,7 @@ umlclass_free_state(UMLClassState *state)
 static UMLClassState *
 umlclass_get_state(UMLClass *umlclass)
 {
-  UMLClassState *state = g_new(UMLClassState, 1);
+  UMLClassState *state = g_new0(UMLClassState, 1);
   GList *list;
 
   state->name = g_strdup(umlclass->name);
@@ -2499,7 +2528,7 @@ umlclass_update_connectionpoints(UMLClass *umlclass)
   else
     num_ops = 0;
   
-  obj = (Object *) umlclass;
+  obj = &umlclass->element.object;
   obj->num_connections = 8 + num_attrib*2 + num_ops*2;
   obj->connections =
     g_realloc(obj->connections,
@@ -2643,7 +2672,7 @@ new_umlclass_change(UMLClass *obj, UMLClassState *saved_state,
 {
   UMLClassChange *change;
 
-  change = g_new(UMLClassChange, 1);
+  change = g_new0(UMLClassChange, 1);
   
   change->obj_change.apply =
     (ObjectChangeApplyFunc) umlclass_change_apply;
