@@ -15,19 +15,23 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 #include <stdio.h>
 
-#include "config.h"
 #include "intl.h"
 #include "object.h"
 #include "element.h"
 #include "render.h"
 #include "attributes.h"
 #include "text.h"
+#include "properties.h"
 
 #include "uml.h"
 
@@ -35,7 +39,6 @@
 
 typedef struct _Classicon Classicon;
 typedef struct _ClassiconState ClassiconState;
-typedef struct _ClassiconPropertiesDialog ClassiconPropertiesDialog;
 
 struct _ClassiconState {
   ObjectState obj_state;
@@ -54,15 +57,6 @@ struct _Classicon {
   Text *text;
 };
 
-struct _ClassiconPropertiesDialog {
-  GtkWidget *dialog;
-  GtkWidget *m_control;
-  GtkWidget *m_bound;
-  GtkWidget *m_entity;  
-  GtkWidget *is_object;
-};
-
-
 enum CLassIconStereotype {
     CLASSICON_CONTROL,
     CLASSICON_BOUNDARY,
@@ -77,8 +71,6 @@ enum CLassIconStereotype {
 #define CLASSICON_AIR 0.25
 #define CLASSICON_ARROW 0.4
 
-static ClassiconPropertiesDialog* properties_dialog = NULL;
-
 static real classicon_distance_from(Classicon *cicon, Point *point);
 static void classicon_select(Classicon *cicon, Point *clicked_point,
 			     Renderer *interactive_renderer);
@@ -96,9 +88,10 @@ static void classicon_save(Classicon *cicon, ObjectNode obj_node,
 			   const char *filename);
 static Object *classicon_load(ObjectNode obj_node, int version,
 			      const char *filename);
+static PropDescription *classicon_describe_props(Classicon *classicon);
+static void classicon_get_props(Classicon *classicon, Property *props, guint nprops);
+static void classicon_set_props(Classicon *classicon, Property *props, guint nprops);
 static void classicon_update_data(Classicon *cicon);
-static GtkWidget *classicon_get_properties(Classicon *cicon);
-static ObjectChange *classicon_apply_properties(Classicon *cicon);
 static ClassiconState *classicon_get_state(Classicon *cicon);
 static void classicon_set_state(Classicon *cicon,
 				ClassiconState *state);
@@ -128,11 +121,117 @@ static ObjectOps classicon_ops = {
   (CopyFunc)            classicon_copy,
   (MoveFunc)            classicon_move,
   (MoveHandleFunc)      classicon_move_handle,
-  (GetPropertiesFunc)   classicon_get_properties,
-  (ApplyPropertiesFunc) classicon_apply_properties,
-  (ObjectMenuFunc)      NULL
+  (GetPropertiesFunc)   object_create_props_dialog,
+  (ApplyPropertiesFunc) object_apply_props_from_dialog,
+  (ObjectMenuFunc)      NULL,
+  (DescribePropsFunc)   classicon_describe_props,
+  (GetPropsFunc)        classicon_get_props,
+  (SetPropsFunc)        classicon_set_props
 };
 
+static PropEnumData prop_classicon_type_data[] = {
+  { N_("Control"), CLASSICON_CONTROL },
+  { N_("Boundary"), CLASSICON_BOUNDARY },
+  { N_("Entity"), CLASSICON_ENTITY },
+  { NULL, 0 }
+};
+
+static PropDescription classicon_props[] = {
+  ELEMENT_COMMON_PROPERTIES,
+  { "stereotype", PROP_TYPE_ENUM, PROP_FLAG_VISIBLE,
+  N_("Stereotype"), NULL,  prop_classicon_type_data},
+  { "is_object", PROP_TYPE_BOOL, PROP_FLAG_VISIBLE,
+  N_("Is object"), NULL, NULL },
+  PROP_STD_TEXT_FONT,
+  PROP_STD_TEXT_HEIGHT,
+  PROP_STD_TEXT_COLOUR,
+  PROP_STD_TEXT,
+  
+  PROP_DESC_END
+};
+
+static PropDescription *
+classicon_describe_props(Classicon *classicon)
+{
+  if (classicon_props[0].quark == 0)
+    prop_desc_list_calculate_quarks(classicon_props);
+  return classicon_props;
+}
+
+static PropOffset classicon_offsets[] = {
+  ELEMENT_COMMON_PROPERTIES_OFFSETS,
+  { "stereotype", PROP_TYPE_ENUM, offsetof(Classicon, stereotype) },
+  { "is_object", PROP_TYPE_BOOL, offsetof(Classicon, is_object) },
+  { NULL, 0, 0 },
+};
+
+static struct { const gchar *name; GQuark q; } quarks[] = {
+  { "text_font" },
+  { "text_height" },
+  { "text_colour" },
+  { "text" }
+};
+
+static void
+classicon_get_props(Classicon * classicon, Property *props, guint nprops)
+{
+  guint i;
+
+  if (object_get_props_from_offsets(&classicon->element.object, 
+                                    classicon_offsets, props, nprops))
+    return;
+  /* these props can't be handled as easily */
+  if (quarks[0].q == 0)
+    for (i = 0; i < 4; i++)
+      quarks[i].q = g_quark_from_static_string(quarks[i].name);
+  for (i = 0; i < nprops; i++) {
+    GQuark pquark = g_quark_from_string(props[i].name);
+
+    if (pquark == quarks[0].q) {
+      props[i].type = PROP_TYPE_FONT;
+      PROP_VALUE_FONT(props[i]) = classicon->text->font;
+    } else if (pquark == quarks[1].q) {
+      props[i].type = PROP_TYPE_REAL;
+      PROP_VALUE_REAL(props[i]) = classicon->text->height;
+    } else if (pquark == quarks[2].q) {
+      props[i].type = PROP_TYPE_COLOUR;
+      PROP_VALUE_COLOUR(props[i]) = classicon->text->color;
+    } else if (pquark == quarks[3].q) {
+      props[i].type = PROP_TYPE_STRING;
+      g_free(PROP_VALUE_STRING(props[i]));
+      PROP_VALUE_STRING(props[i]) = text_get_string_copy(classicon->text);
+    }
+  }
+}
+
+static void
+classicon_set_props(Classicon *classicon, Property *props, guint nprops)
+{
+  if (!object_set_props_from_offsets(&classicon->element.object, 
+                                     classicon_offsets, props, nprops)) {
+    guint i;
+
+    if (quarks[0].q == 0)
+      for (i = 0; i < 4; i++)
+	quarks[i].q = g_quark_from_static_string(quarks[i].name);
+
+    for (i = 0; i < nprops; i++) {
+      GQuark pquark = g_quark_from_string(props[i].name);
+
+      if (pquark == quarks[0].q && props[i].type == PROP_TYPE_FONT) {
+	text_set_font(classicon->text, PROP_VALUE_FONT(props[i]));
+      } else if (pquark == quarks[1].q && props[i].type == PROP_TYPE_REAL) {
+	text_set_height(classicon->text, PROP_VALUE_REAL(props[i]));
+      } else if (pquark == quarks[2].q && props[i].type == PROP_TYPE_COLOUR) {
+	text_set_color(classicon->text, &PROP_VALUE_COLOUR(props[i]));
+      } else if (pquark == quarks[3].q && props[i].type == PROP_TYPE_STRING) {
+	text_set_string(classicon->text, PROP_VALUE_STRING(props[i]));
+      }
+    }
+  }
+  classicon_update_data(classicon);
+}
+
 static real
 classicon_distance_from(Classicon *cicon, Point *point)
 {
@@ -145,7 +244,7 @@ classicon_select(Classicon *cicon, Point *clicked_point,
 		 Renderer *interactive_renderer)
 {
   text_set_cursor(cicon->text, clicked_point, interactive_renderer);
-  text_grab_focus(cicon->text, (Object *)cicon);
+  text_grab_focus(cicon->text, &cicon->element.object);
   element_update_handles(&cicon->element);
 }
 
@@ -279,7 +378,7 @@ static void
 classicon_update_data(Classicon *cicon)
 {
   Element *elem = &cicon->element;
-  Object *obj = (Object *) cicon;
+  Object *obj = &elem->object;
   Font *font;
   Point p1;
   real h, wt, w = 0;
@@ -339,12 +438,6 @@ classicon_update_data(Classicon *cicon)
   
   element_update_boundingbox(elem);
 
-  /* fix boundingclassicon for line width and top rectangle: */
-  /*  obj->bounding_box.top -= CLASSICON_BORDERWIDTH/2.0;
-  obj->bounding_box.left -= CLASSICON_BORDERWIDTH/2.0;
-  obj->bounding_box.bottom += CLASSICON_BORDERWIDTH/2.0;
-  obj->bounding_box.right += CLASSICON_BORDERWIDTH/2.0;
-  */
   obj->position = elem->corner;
   obj->position.x += (elem->width + ((is_boundary)?CLASSICON_RADIOUS:0))/2.0;
   obj->position.y += CLASSICON_RADIOUS + CLASSICON_ARROW;
@@ -365,9 +458,9 @@ classicon_create(Point *startpoint,
   Font *font;
   int i;
   
-  cicon = g_malloc(sizeof(Classicon));
+  cicon = g_malloc0(sizeof(Classicon));
   elem = &cicon->element;
-  obj = (Object *) cicon;
+  obj = &elem->object;
   
   obj->type = &classicon_type;
 
@@ -392,6 +485,7 @@ classicon_create(Point *startpoint,
     cicon->connections[i].object = obj;
     cicon->connections[i].connected = NULL;
   }
+  elem->extra_spacing.border_trans = 0.0;
   classicon_update_data(cicon);
 
   for (i=0;i<8;i++) {
@@ -401,7 +495,7 @@ classicon_create(Point *startpoint,
   *handle1 = NULL;
   *handle2 = NULL;
 
-  return (Object *)cicon;
+  return &cicon->element.object;
 }
 
 static void
@@ -421,9 +515,9 @@ classicon_copy(Classicon *cicon)
   
   elem = &cicon->element;
   
-  newcicon = g_malloc(sizeof(Classicon));
+  newcicon = g_malloc0(sizeof(Classicon));
   newelem = &newcicon->element;
-  newobj = (Object *) newcicon;
+  newobj = &newelem->object;
 
   element_copy(elem, newelem);
 
@@ -442,13 +536,13 @@ classicon_copy(Classicon *cicon)
 
   classicon_update_data(newcicon);
   
-  return (Object *)newcicon;
+  return &newcicon->element.object;
 }
 
 static ClassiconState *
 classicon_get_state(Classicon *cicon)
 {
-  ClassiconState *state = g_new(ClassiconState, 1);
+  ClassiconState *state = g_new0(ClassiconState, 1);
 
   state->obj_state.free = NULL;
 
@@ -493,9 +587,9 @@ classicon_load(ObjectNode obj_node, int version, const char *filename)
   Object *obj;
   int i;
   
-  cicon = g_malloc(sizeof(Classicon));
+  cicon = g_malloc0(sizeof(Classicon));
   elem = &cicon->element;
-  obj = (Object *) cicon;
+  obj = &elem->object;
   
   obj->type = &classicon_type;
   obj->ops = &classicon_ops;
@@ -524,114 +618,13 @@ classicon_load(ObjectNode obj_node, int version, const char *filename)
     cicon->connections[i].object = obj;
     cicon->connections[i].connected = NULL;
   }
+  elem->extra_spacing.border_trans = 0.0;
   classicon_update_data(cicon);
 
   for (i=0;i<8;i++) {
     obj->handles[i]->type = HANDLE_NON_MOVABLE;
   }
 
-  return (Object *) cicon;
-}
-
-
-static ObjectChange *
-classicon_apply_properties(Classicon *cicon)
-{
-  ClassiconPropertiesDialog *prop_dialog;
-  ObjectState *old_state;
-
-  prop_dialog = properties_dialog;
-
-  old_state = (ObjectState *)classicon_get_state(cicon);
-
-  cicon->is_object = GTK_TOGGLE_BUTTON(prop_dialog->is_object)->active;
-
-  /* Read from dialog and put in object: */
-  if (GTK_TOGGLE_BUTTON(prop_dialog->m_control)->active) {
-      cicon->stereotype = CLASSICON_CONTROL;
-  } else if (GTK_TOGGLE_BUTTON(prop_dialog->m_bound)->active) {
-    cicon->stereotype = CLASSICON_BOUNDARY;
-  } else if (GTK_TOGGLE_BUTTON(prop_dialog->m_entity)->active) {
-    cicon->stereotype = CLASSICON_ENTITY;
-  }
-
-  classicon_update_data(cicon);
-
-  return new_object_state_change((Object *)cicon, old_state, 
-				 (GetStateFunc)classicon_get_state,
-				 (SetStateFunc)classicon_set_state);
-}
-
-static void
-fill_in_dialog(Classicon *cicon)
-{
-  ClassiconPropertiesDialog *prop_dialog;
-  GtkToggleButton *button = NULL;
-
-  prop_dialog = properties_dialog;
-
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prop_dialog->is_object), 
-			       cicon->is_object);
-
-  switch (cicon->stereotype) {
-  case CLASSICON_CONTROL:
-      button = GTK_TOGGLE_BUTTON(prop_dialog->m_control);
-      break;
-  case CLASSICON_BOUNDARY:
-      button = GTK_TOGGLE_BUTTON(prop_dialog->m_bound);
-      break;
-  case CLASSICON_ENTITY:
-      button = GTK_TOGGLE_BUTTON(prop_dialog->m_entity);
-      break;
-  }
-  gtk_toggle_button_set_active(button, TRUE);
-}
-
-static GtkWidget *
-classicon_get_properties(Classicon *cicon)
-{
-  ClassiconPropertiesDialog *prop_dialog;
-  GtkWidget *dialog;
-  GtkWidget *label;
-  GSList *group;
-
-  if (properties_dialog == NULL) {
-
-    prop_dialog = g_new(ClassiconPropertiesDialog, 1);
-    properties_dialog = prop_dialog;
-
-    dialog = gtk_vbox_new(FALSE, 0);
-    gtk_object_ref(GTK_OBJECT(dialog));
-    gtk_object_sink(GTK_OBJECT(dialog));
-    prop_dialog->dialog = dialog;
-    
-    prop_dialog->m_control = gtk_radio_button_new_with_label (NULL, _("Control"));
-    gtk_box_pack_start (GTK_BOX (dialog), prop_dialog->m_control, TRUE, TRUE, 0);
-    gtk_widget_show (prop_dialog->m_control);
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prop_dialog->m_control), TRUE);
-
-    group = gtk_radio_button_group (GTK_RADIO_BUTTON (prop_dialog->m_control));
-    prop_dialog->m_bound = gtk_radio_button_new_with_label(group, _("Boundary"));
-    gtk_box_pack_start (GTK_BOX (dialog), prop_dialog->m_bound, TRUE, TRUE, 0);
-    gtk_widget_show (prop_dialog->m_bound);
-
-    group = gtk_radio_button_group (GTK_RADIO_BUTTON (prop_dialog->m_bound));
-    prop_dialog->m_entity = gtk_radio_button_new_with_label(group, _("Entity"));
-    gtk_box_pack_start (GTK_BOX (dialog), prop_dialog->m_entity, TRUE, TRUE, 0);
-    gtk_widget_show (prop_dialog->m_entity);
-
-    label = gtk_hseparator_new ();
-    gtk_box_pack_start (GTK_BOX (dialog), label, FALSE, TRUE, 0);
-    gtk_widget_show (label);
-
-    prop_dialog->is_object = gtk_check_button_new_with_label(_("Is an object"));
-    gtk_widget_show(prop_dialog->is_object);
-    gtk_box_pack_start (GTK_BOX (dialog), prop_dialog->is_object, TRUE, TRUE, 0);
-  }
-  
-  fill_in_dialog(cicon);
-  gtk_widget_show (properties_dialog->dialog);
-
-  return properties_dialog->dialog;
+  return &cicon->element.object;
 }
 
