@@ -15,18 +15,25 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
 #include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 
-#include "config.h"
 #include "intl.h"
 #include "object.h"
 #include "element.h"
 #include "render.h"
 #include "attributes.h"
 #include "text.h"
+#include "properties.h"
+
+#include "stereotype.h"
 
 #include "pixmaps/component.xpm"
 
@@ -37,7 +44,6 @@ struct _Component {
   ConnectionPoint connections[8];
 
   char *stereotype;
-  char *name;
   Text *text;
 };
 
@@ -66,6 +72,10 @@ static void component_save(Component *cmp, ObjectNode obj_node,
 static Object *component_load(ObjectNode obj_node, int version,
 				 const char *filename);
 
+static PropDescription *component_describe_props(Component *component);
+static void component_get_props(Component *component, Property *props, guint nprops);
+static void component_set_props(Component *component, Property *props, guint nprops);
+
 static void component_update_data(Component *cmp);
 
 static ObjectTypeOps component_type_ops =
@@ -92,11 +102,125 @@ static ObjectOps component_ops = {
   (CopyFunc)            component_copy,
   (MoveFunc)            component_move,
   (MoveHandleFunc)      component_move_handle,
-  (GetPropertiesFunc)   object_return_null,
-  (ApplyPropertiesFunc) object_return_void,
-  (ObjectMenuFunc)      NULL
+  (GetPropertiesFunc)   object_create_props_dialog,
+  (ApplyPropertiesFunc) object_apply_props_from_dialog,
+  (ObjectMenuFunc)      NULL,
+  (DescribePropsFunc)   component_describe_props,
+  (GetPropsFunc)        component_get_props,
+  (SetPropsFunc)        component_set_props
 };
 
+static PropDescription component_props[] = {
+  ELEMENT_COMMON_PROPERTIES,
+  { "stereotype", PROP_TYPE_STRING, PROP_FLAG_VISIBLE,
+  N_("Stereotype"), NULL, NULL },
+  PROP_STD_TEXT_FONT,
+  PROP_STD_TEXT_HEIGHT,
+  PROP_STD_TEXT_COLOUR,
+  PROP_STD_TEXT,
+  
+  PROP_DESC_END
+};
+
+static PropDescription *
+component_describe_props(Component *component)
+{
+  if (component_props[0].quark == 0)
+    prop_desc_list_calculate_quarks(component_props);
+  return component_props;
+}
+
+static PropOffset component_offsets[] = {
+  ELEMENT_COMMON_PROPERTIES_OFFSETS,
+  /*{ "stereotype", PROP_TYPE_STRING, offsetof(Component , stereotype) },*/
+  { NULL, 0, 0 },
+};
+
+static struct { const gchar *name; GQuark q; } quarks[] = {
+  { "text_font" },
+  { "text_height" },
+  { "text_colour" },
+  { "text" },
+  { "stereotype" }
+};
+
+static void
+component_get_props(Component * component, Property *props, guint nprops)
+{
+  guint i;
+
+  if (object_get_props_from_offsets(&component->element.object, 
+                                    component_offsets, props, nprops))
+    return;
+  /* these props can't be handled as easily */
+  if (quarks[0].q == 0)
+    for (i = 0; i < sizeof(quarks)/sizeof(*quarks); i++)
+      quarks[i].q = g_quark_from_static_string(quarks[i].name);
+  for (i = 0; i < nprops; i++) {
+    GQuark pquark = g_quark_from_string(props[i].name);
+
+    if (pquark == quarks[0].q) {
+      props[i].type = PROP_TYPE_FONT;
+      PROP_VALUE_FONT(props[i]) = component->text->font;
+    } else if (pquark == quarks[1].q) {
+      props[i].type = PROP_TYPE_REAL;
+      PROP_VALUE_REAL(props[i]) = component->text->height;
+    } else if (pquark == quarks[2].q) {
+      props[i].type = PROP_TYPE_COLOUR;
+      PROP_VALUE_COLOUR(props[i]) = component->text->color;
+    } else if (pquark == quarks[3].q) {
+      props[i].type = PROP_TYPE_STRING;
+      g_free(PROP_VALUE_STRING(props[i]));
+      PROP_VALUE_STRING(props[i]) = text_get_string_copy(component->text);
+    } else if (pquark == quarks[4].q) {
+      props[i].type = PROP_TYPE_STRING;
+      g_free(PROP_VALUE_STRING(props[i]));
+      if (component->stereotype &&
+	  component->stereotype[0] != '\0')
+	PROP_VALUE_STRING(props[i]) =
+	  stereotype_to_string(component->stereotype);
+      else
+	PROP_VALUE_STRING(props[i]) = NULL;
+    }
+  }
+}
+
+static void
+component_set_props(Component *component, Property *props, guint nprops)
+{
+  if (!object_set_props_from_offsets(&component->element.object, 
+                                     component_offsets, props, nprops)) {
+    guint i;
+
+    if (quarks[0].q == 0)
+      for (i = 0; i < sizeof(quarks)/sizeof(*quarks); i++)
+	quarks[i].q = g_quark_from_static_string(quarks[i].name);
+
+    for (i = 0; i < nprops; i++) {
+      GQuark pquark = g_quark_from_string(props[i].name);
+
+      if (pquark == quarks[0].q && props[i].type == PROP_TYPE_FONT) {
+	text_set_font(component->text, PROP_VALUE_FONT(props[i]));
+      } else if (pquark == quarks[1].q && props[i].type == PROP_TYPE_REAL) {
+	text_set_height(component->text, PROP_VALUE_REAL(props[i]));
+      } else if (pquark == quarks[2].q && props[i].type == PROP_TYPE_COLOUR) {
+	text_set_color(component->text, &PROP_VALUE_COLOUR(props[i]));
+      } else if (pquark == quarks[3].q && props[i].type == PROP_TYPE_STRING) {
+	text_set_string(component->text, PROP_VALUE_STRING(props[i]));
+      } else if (pquark == quarks[4].q && props[i].type == PROP_TYPE_STRING) {
+	g_free(component->stereotype);
+	if (PROP_VALUE_STRING(props[i]) &&
+	    PROP_VALUE_STRING(props[i])[0] != '\0')
+	  component->stereotype =
+	    string_to_stereotype(PROP_VALUE_STRING(props[i]));
+	else 
+	  component->stereotype = NULL;
+      }
+    }
+  }
+  component_update_data(component);
+}
+
 static real
 component_distance_from(Component *cmp, Point *point)
 {
@@ -109,7 +233,7 @@ component_select(Component *cmp, Point *clicked_point,
 	       Renderer *interactive_renderer)
 {
   text_set_cursor(cmp->text, clicked_point, interactive_renderer);
-  text_grab_focus(cmp->text, (Object *)cmp);
+  text_grab_focus(cmp->text, &cmp->element.object);
   element_update_handles(&cmp->element);
 }
 
@@ -127,15 +251,8 @@ component_move_handle(Component *cmp, Handle *handle,
 static void
 component_move(Component *cmp, Point *to)
 {
-  Point p;
-  
   cmp->element.corner = *to;
 
-  p = *to;
-  p.x += COMPONENT_CWIDTH + COMPONENT_MARGIN_X;
-  p.y += 2*COMPONENT_CHEIGHT;
-  text_set_position(cmp->text, &p);
-  
   component_update_data(cmp);
 }
 
@@ -160,7 +277,6 @@ component_draw(Component *cmp, Renderer *renderer)
   renderer->ops->set_linewidth(renderer, COMPONENT_BORDERWIDTH);
   renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
 
-
   p1.x = x + COMPONENT_CWIDTH/2; p1.y = y;
   p2.x = x+w; p2.y = y+h;
 
@@ -193,7 +309,13 @@ component_draw(Component *cmp, Renderer *renderer)
 			   &p1, &p2,
 			   &color_black);
 
-
+  if (cmp->stereotype != NULL &&
+      cmp->stereotype[0] != '\0') {
+    p1 = cmp->text->position;
+    p1.y -= cmp->text->height;
+    renderer->ops->draw_string(renderer, cmp->stereotype, &p1, 
+			       ALIGN_LEFT, &color_black);
+  }
 
   text_draw(cmp->text, renderer);
 }
@@ -202,14 +324,34 @@ static void
 component_update_data(Component *cmp)
 {
   Element *elem = &cmp->element;
-  Object *obj = (Object *) cmp;
-  
+  Object *obj = &elem->object;
+  Point p;
+
   elem->width = cmp->text->max_width + 2*COMPONENT_MARGIN_X + COMPONENT_CWIDTH;
   elem->width = MAX(elem->width, 2*COMPONENT_CWIDTH);
   elem->height =  cmp->text->height*cmp->text->numlines +
     cmp->text->descent + 0.1 + 2*COMPONENT_MARGIN_Y ;
   elem->height = MAX(elem->height, 5*COMPONENT_CHEIGHT);
 
+  p = elem->corner;
+  p.x += COMPONENT_CWIDTH + COMPONENT_MARGIN_X;
+  p.y += COMPONENT_CHEIGHT;
+  p.y += cmp->text->ascent;
+  if (cmp->stereotype &&
+      cmp->stereotype[0] != '\0')
+    p.y += cmp->text->height;
+  text_set_position(cmp->text, &p);
+
+  if (cmp->stereotype &&
+      cmp->stereotype[0] != '\0') {
+    Font *font;
+    font = cmp->text->font;
+    elem->height += cmp->text->height;
+    elem->width = MAX(elem->width, font_string_width(cmp->stereotype,
+						     font, cmp->text->height) +
+		      2*COMPONENT_MARGIN_X + COMPONENT_CWIDTH);
+  }
+
   /* Update connections: */
   cmp->connections[0].pos = elem->corner;
   cmp->connections[1].pos.x = elem->corner.x + elem->width / 2.0;
@@ -228,11 +370,6 @@ component_update_data(Component *cmp)
   cmp->connections[7].pos.y = elem->corner.y + elem->height;
   
   element_update_boundingbox(elem);
-  /* fix boundingcomponent for line width and top rectangle: */
-  obj->bounding_box.top -= COMPONENT_BORDERWIDTH/2.0;
-  obj->bounding_box.left -= COMPONENT_BORDERWIDTH/2.0;
-  obj->bounding_box.bottom += COMPONENT_BORDERWIDTH/2.0;
-  obj->bounding_box.right += COMPONENT_BORDERWIDTH/2.0;
 
   obj->position = elem->corner;
 
@@ -252,9 +389,9 @@ component_create(Point *startpoint,
   Font *font;
   int i;
   
-  cmp = g_malloc(sizeof(Component));
+  cmp = g_malloc0(sizeof(Component));
   elem = &cmp->element;
-  obj = (Object *) cmp;
+  obj = &elem->object;
   
   obj->type = &component_type;
 
@@ -276,6 +413,9 @@ component_create(Point *startpoint,
     cmp->connections[i].object = obj;
     cmp->connections[i].connected = NULL;
   }
+  elem->extra_spacing.border_trans = COMPONENT_BORDERWIDTH/2.0;
+  cmp->stereotype = NULL;
+
   component_update_data(cmp);
 
   for (i=0;i<8;i++) {
@@ -284,7 +424,7 @@ component_create(Point *startpoint,
 
   *handle1 = NULL;
   *handle2 = NULL;
-  return (Object *)cmp;
+  return &cmp->element.object;
 }
 
 static void
@@ -293,6 +433,7 @@ component_destroy(Component *cmp)
   text_destroy(cmp->text);
 
   element_destroy(&cmp->element);
+  g_free(cmp->stereotype);
 }
 
 static Object *
@@ -305,9 +446,9 @@ component_copy(Component *cmp)
   
   elem = &cmp->element;
   
-  newcmp = g_malloc(sizeof(Component));
+  newcmp = g_malloc0(sizeof(Component));
   newelem = &newcmp->element;
-  newobj = (Object *) newcmp;
+  newobj = &newelem->object;
 
   element_copy(elem, newelem);
 
@@ -320,10 +461,13 @@ component_copy(Component *cmp)
     newcmp->connections[i].pos = cmp->connections[i].pos;
     newcmp->connections[i].last_pos = cmp->connections[i].last_pos;
   }
-
   component_update_data(newcmp);
+  if (cmp->stereotype != NULL)
+    newcmp->stereotype = g_strdup(cmp->stereotype);
+  else
+    newcmp->stereotype = NULL;
   
-  return (Object *)newcmp;
+  return &newcmp->element.object;
 }
 
 
@@ -335,6 +479,8 @@ component_save(Component *cmp, ObjectNode obj_node,
 
   data_add_text(new_attribute(obj_node, "text"),
 		cmp->text);
+  data_add_string(new_attribute(obj_node, "stereotype"),
+		  cmp->stereotype);
 }
 
 static Object *
@@ -346,9 +492,9 @@ component_load(ObjectNode obj_node, int version, const char *filename)
   Object *obj;
   int i;
   
-  cmp = g_malloc(sizeof(Component));
+  cmp = g_malloc0(sizeof(Component));
   elem = &cmp->element;
-  obj = (Object *) cmp;
+  obj = &elem->object;
   
   obj->type = &component_type;
   obj->ops = &component_ops;
@@ -360,6 +506,11 @@ component_load(ObjectNode obj_node, int version, const char *filename)
   if (attr != NULL)
     cmp->text = data_text(attribute_first_data(attr));
   
+  cmp->stereotype = NULL;
+  attr = object_find_attribute(obj_node, "stereotype");
+  if (attr != NULL)
+    cmp->stereotype = data_string(attribute_first_data(attr));
+
   element_init(elem, 8, 8);
 
   for (i=0;i<8;i++) {
@@ -367,13 +518,14 @@ component_load(ObjectNode obj_node, int version, const char *filename)
     cmp->connections[i].object = obj;
     cmp->connections[i].connected = NULL;
   }
+  elem->extra_spacing.border_trans = COMPONENT_BORDERWIDTH/2.0;
   component_update_data(cmp);
 
   for (i=0;i<8;i++) {
     obj->handles[i]->type = HANDLE_NON_MOVABLE;
   }
 
-  return (Object *) cmp;
+  return &cmp->element.object;
 }
 
 
