@@ -15,20 +15,26 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
 #include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 
-#include "config.h"
 #include "intl.h"
 #include "object.h"
 #include "element.h"
 #include "render.h"
 #include "attributes.h"
 #include "text.h"
+#include "properties.h"
 
 #include "uml.h"
+#include "stereotype.h"
 
 #include "pixmaps/largepackage.xpm"
 
@@ -92,6 +98,10 @@ static void largepackage_update_data(LargePackage *pkg);
 static GtkWidget *largepackage_get_properties(LargePackage *pkg);
 static ObjectChange *largepackage_apply_properties(LargePackage *pkg);
 
+static PropDescription *largepackage_describe_props(LargePackage *largepackage);
+static void largepackage_get_props(LargePackage *largepackage, Property *props, guint nprops);
+static void largepackage_set_props(LargePackage *largepackage, Property *props, guint nprops);
+
 static LargePackageState *largepackage_get_state(LargePackage *pkg);
 static void largepackage_set_state(LargePackage *pkg,
 				 LargePackageState *state);
@@ -122,9 +132,48 @@ static ObjectOps largepackage_ops = {
   (MoveHandleFunc)      largepackage_move_handle,
   (GetPropertiesFunc)   largepackage_get_properties,
   (ApplyPropertiesFunc) largepackage_apply_properties,
-  (ObjectMenuFunc)      NULL
+  (ObjectMenuFunc)      NULL,
+  (DescribePropsFunc)   largepackage_describe_props,
+  (GetPropsFunc)        largepackage_get_props,
+  (SetPropsFunc)        largepackage_set_props
 };
 
+static PropDescription largepackage_props[] = {
+  ELEMENT_COMMON_PROPERTIES,
+  /* XXX */
+  
+  PROP_DESC_END
+};
+
+static PropDescription *
+largepackage_describe_props(LargePackage *largepackage)
+{
+  if (largepackage_props[0].quark == 0)
+    prop_desc_list_calculate_quarks(largepackage_props);
+  return largepackage_props;
+}
+
+static PropOffset largepackage_offsets[] = {
+  ELEMENT_COMMON_PROPERTIES_OFFSETS,
+
+  { NULL, 0, 0 },
+};
+
+static void
+largepackage_get_props(LargePackage * largepackage, Property *props, guint nprops)
+{
+  object_get_props_from_offsets(&largepackage->element.object, 
+                                largepackage_offsets, props, nprops);
+}
+
+static void
+largepackage_set_props(LargePackage *largepackage, Property *props, guint nprops)
+{
+  object_set_props_from_offsets(&largepackage->element.object, 
+                                largepackage_offsets, props, nprops);
+  largepackage_update_data(largepackage);
+}
+
 static real
 largepackage_distance_from(LargePackage *pkg, Point *point)
 {
@@ -210,23 +259,24 @@ largepackage_draw(LargePackage *pkg, Renderer *renderer)
   p1.x = x + 0.1;
   p1.y = y - LARGEPACKAGE_FONTHEIGHT - font_descent(pkg->font, LARGEPACKAGE_FONTHEIGHT) - 0.1;
 
+
+
   if (pkg->stereotype) {
     renderer->ops->draw_string(renderer, pkg->stereotype, &p1,
 			       ALIGN_LEFT, &color_black);
   }
   p1.y += LARGEPACKAGE_FONTHEIGHT;
 
-  renderer->ops->draw_string(renderer, pkg->name, &p1,
-			     ALIGN_LEFT, &color_black);
-
-
+  if (pkg->name)
+    renderer->ops->draw_string(renderer, pkg->name, &p1,
+			       ALIGN_LEFT, &color_black);
 }
 
 static void
 largepackage_update_data(LargePackage *pkg)
 {
   Element *elem = &pkg->element;
-  Object *obj = (Object *) pkg;
+  Object *obj = &elem->object;
 
   if (elem->width < (pkg->topwidth + 0.2))
     elem->width = pkg->topwidth + 0.2;
@@ -251,11 +301,8 @@ largepackage_update_data(LargePackage *pkg)
   pkg->connections[7].pos.y = elem->corner.y + elem->height;
   
   element_update_boundingbox(elem);
-  /* fix boundinglargepackage for line width and top rectangle: */
-  obj->bounding_box.top -= LARGEPACKAGE_BORDERWIDTH/2.0 + pkg->topheight;
-  obj->bounding_box.left -= LARGEPACKAGE_BORDERWIDTH/2.0;
-  obj->bounding_box.bottom += LARGEPACKAGE_BORDERWIDTH/2.0;
-  obj->bounding_box.right += LARGEPACKAGE_BORDERWIDTH/2.0;
+  /* fix boundingbox for top rectangle: */
+  obj->bounding_box.top -= pkg->topheight;
 
   obj->position = elem->corner;
 
@@ -273,9 +320,9 @@ largepackage_create(Point *startpoint,
   Object *obj;
   int i;
   
-  pkg = g_malloc(sizeof(LargePackage));
+  pkg = g_malloc0(sizeof(LargePackage));
   elem = &pkg->element;
-  obj = (Object *) pkg;
+  obj = &elem->object;
   
   obj->type = &largepackage_type;
 
@@ -303,11 +350,12 @@ largepackage_create(Point *startpoint,
     pkg->connections[i].object = obj;
     pkg->connections[i].connected = NULL;
   }
+  pkg->element.extra_spacing.border_trans = LARGEPACKAGE_BORDERWIDTH/2.0;
   largepackage_update_data(pkg);
 
   *handle1 = NULL;
   *handle2 = obj->handles[0];
-  return (Object *)pkg;
+  return &pkg->element.object;
 }
 
 static void
@@ -337,9 +385,9 @@ largepackage_copy(LargePackage *pkg)
   
   elem = &pkg->element;
   
-  newpkg = g_malloc(sizeof(LargePackage));
+  newpkg = g_malloc0(sizeof(LargePackage));
   newelem = &newpkg->element;
-  newobj = (Object *) newpkg;
+  newobj = &newelem->object;
 
   element_copy(elem, newelem);
 
@@ -365,7 +413,7 @@ largepackage_copy(LargePackage *pkg)
 
   largepackage_update_data(newpkg);
   
-  return (Object *)newpkg;
+  return &newpkg->element.object;
 }
 
 static void
@@ -379,7 +427,7 @@ largepackage_state_free(ObjectState *ostate)
 static LargePackageState *
 largepackage_get_state(LargePackage *pkg)
 {
-  LargePackageState *state = g_new(LargePackageState, 1);
+  LargePackageState *state = g_new0(LargePackageState, 1);
 
   state->obj_state.free = largepackage_state_free;
 
@@ -423,9 +471,9 @@ largepackage_load(ObjectNode obj_node, int version, const char *filename)
   Object *obj;
   int i;
   
-  pkg = g_malloc(sizeof(LargePackage));
+  pkg = g_malloc0(sizeof(LargePackage));
   elem = &pkg->element;
-  obj = (Object *) pkg;
+  obj = &elem->object;
   
   obj->type = &largepackage_type;
   obj->ops = &largepackage_ops;
@@ -445,9 +493,10 @@ largepackage_load(ObjectNode obj_node, int version, const char *filename)
   pkg->font = font_getfont("Courier");
 
   pkg->topwidth = 2.0;
-  pkg->topwidth = MAX(pkg->topwidth,
-		      font_string_width(pkg->name, pkg->font,
-					LARGEPACKAGE_FONTHEIGHT)+2*0.1);
+  if (pkg->name != NULL)
+    pkg->topwidth = MAX(pkg->topwidth,
+			font_string_width(pkg->name, pkg->font,
+					  LARGEPACKAGE_FONTHEIGHT)+2*0.1);
   if (pkg->stereotype != NULL)
     pkg->topwidth = MAX(pkg->topwidth,
 			font_string_width(pkg->stereotype, pkg->font,
@@ -466,7 +515,7 @@ largepackage_load(ObjectNode obj_node, int version, const char *filename)
   }
   largepackage_update_data(pkg);
 
-  return (Object *) pkg;
+  return &pkg->element.object;
 }
 
 static ObjectChange *
@@ -491,12 +540,7 @@ largepackage_apply_properties(LargePackage *pkg)
   str = gtk_entry_get_text(prop_dialog->stereotype);
   
   if (strlen(str) != 0) {
-    pkg->stereotype = g_malloc(sizeof(char)*strlen(str)+2+1);
-    pkg->stereotype[0] = UML_STEREOTYPE_START;
-    pkg->stereotype[1] = 0;
-    strcat(pkg->stereotype, str);
-    pkg->stereotype[strlen(str)+1] = UML_STEREOTYPE_END;
-    pkg->stereotype[strlen(str)+2] = 0;
+    pkg->stereotype = string_to_stereotype(str);
   } else {
     pkg->stereotype = NULL;
   }
@@ -511,9 +555,10 @@ largepackage_apply_properties(LargePackage *pkg)
 					  LARGEPACKAGE_FONTHEIGHT)+2*0.1);
   
   pkg->topheight = LARGEPACKAGE_FONTHEIGHT*2 + 0.1*2;
+  pkg->element.extra_spacing.border_trans = LARGEPACKAGE_BORDERWIDTH/2.0;
 
   largepackage_update_data(pkg);
-  return new_object_state_change((Object *)pkg, old_state, 
+  return new_object_state_change(&pkg->element.object, old_state, 
 				 (GetStateFunc)largepackage_get_state,
 				 (SetStateFunc)largepackage_set_state);
 }
@@ -529,9 +574,7 @@ fill_in_dialog(LargePackage *pkg)
   gtk_entry_set_text(prop_dialog->name, pkg->name);
   
   if (pkg->stereotype != NULL) {
-    str = strdup(pkg->stereotype);
-    strcpy(str, pkg->stereotype+1);
-    str[strlen(str)-1] = 0;
+    str = stereotype_to_string(pkg->stereotype);
     gtk_entry_set_text(prop_dialog->stereotype, str);
     g_free(str);
   } else {
