@@ -15,19 +15,24 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
 #include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 #include <stdio.h>
 
-#include "config.h"
 #include "intl.h"
 #include "object.h"
 #include "connection.h"
 #include "render.h"
 #include "handle.h"
 #include "arrows.h"
+#include "properties.h"
 
 #include "pixmaps/message.xpm"
 
@@ -90,8 +95,6 @@ struct _MessageDialog {
 
 static Font *message_font = NULL;
 
-static MessageDialog *properties_dialog;
-
 static void message_move_handle(Message *message, Handle *handle,
 				   Point *to, HandleMoveReason reason, ModifierKeys modifiers);
 static void message_move(Message *message, Point *to);
@@ -106,8 +109,7 @@ static real message_distance_from(Message *message, Point *point);
 static void message_update_data(Message *message);
 static void message_destroy(Message *message);
 static Object *message_copy(Message *message);
-static GtkWidget *message_get_properties(Message *message);
-static ObjectChange *message_apply_properties(Message *message);
+static ObjectChange *message_apply_properties(Message *message, GtkWidget *widget);
 
 static MessageState *message_get_state(Message *message);
 static void message_set_state(Message *message,
@@ -118,6 +120,9 @@ static void message_save(Message *message, ObjectNode obj_node,
 static Object *message_load(ObjectNode obj_node, int version,
 			    const char *filename);
 
+static PropDescription *message_describe_props(Message *mes);
+static void message_get_props(Message * message, Property *props, guint nprops);
+static void message_set_props(Message *message, Property *props, guint nprops);
 
 static ObjectTypeOps message_type_ops =
 {
@@ -142,11 +147,66 @@ static ObjectOps message_ops = {
   (CopyFunc)            message_copy,
   (MoveFunc)            message_move,
   (MoveHandleFunc)      message_move_handle,
-  (GetPropertiesFunc)   message_get_properties,
+  (GetPropertiesFunc)   object_create_props_dialog,/*message_get_properties,*/
   (ApplyPropertiesFunc) message_apply_properties,
-  (ObjectMenuFunc)      NULL
+  (ObjectMenuFunc)      NULL,
+  (DescribePropsFunc)   message_describe_props,
+  (GetPropsFunc)        message_get_props,
+  (SetPropsFunc)        message_set_props
 };
 
+static PropEnumData prop_message_type_data[] = {
+  { N_("Call"), MESSAGE_CALL },
+  { N_("Create"), MESSAGE_CREATE },
+  { N_("Destroy"), MESSAGE_DESTROY },
+  { N_("Simple"), MESSAGE_SIMPLE },
+  { N_("Return"), MESSAGE_RETURN },
+  { N_("Send"), MESSAGE_SEND },
+  { N_("Recursive"), MESSAGE_RECURSIVE },
+  { NULL, 0}
+};
+
+static PropDescription message_props[] = {
+  OBJECT_COMMON_PROPERTIES,
+  { "message", PROP_TYPE_STRING, PROP_FLAG_VISIBLE,
+    N_("Message:"), NULL, NULL },
+  { "message_type", PROP_TYPE_ENUM, PROP_FLAG_VISIBLE,
+    N_("Message type:"), NULL, prop_message_type_data },
+  PROP_DESC_END
+};
+
+static PropDescription *
+message_describe_props(Message *mes)
+{
+  if (message_props[0].quark == 0)
+    prop_desc_list_calculate_quarks(message_props);
+  return message_props;
+
+}
+
+static PropOffset message_offsets[] = {
+  OBJECT_COMMON_PROPERTIES_OFFSETS,
+  { "message", PROP_TYPE_STRING, offsetof(Message, text) },
+  { "message_type", PROP_TYPE_ENUM, offsetof(Message, type) },
+  { NULL, 0, 0 }
+};
+
+static void
+message_get_props(Message * message, Property *props, guint nprops)
+{
+  object_get_props_from_offsets(&message->connection.object, 
+                                message_offsets, props, nprops);
+}
+
+static void
+message_set_props(Message *message, Property *props, guint nprops)
+{
+  object_set_props_from_offsets(&message->connection.object, 
+                                message_offsets, props, nprops);
+  message_update_data(message);
+}
+
+
 static real
 message_distance_from(Message *message, Point *point)
 {
@@ -287,7 +347,7 @@ message_draw(Message *message, Renderer *renderer)
   else
       mname = message->text;
 
-  if (mname) 
+  if (mname && strlen(mname) != 0) 
       renderer->ops->draw_string(renderer,
 				 mname, /*message->text,*/
 				 &message->text_pos, ALIGN_CENTER,
@@ -302,26 +362,28 @@ message_create(Point *startpoint,
 {
   Message *message;
   Connection *conn;
+  ConnectionBBExtras *extra;
   Object *obj;
 
   if (message_font == NULL)
     message_font = font_getfont("Helvetica");
   
-  message = g_malloc(sizeof(Message));
+  message = g_malloc0(sizeof(Message));
 
   conn = &message->connection;
   conn->endpoints[0] = *startpoint;
   conn->endpoints[1] = *startpoint;
   conn->endpoints[1].x += 1.5;
  
-  obj = (Object *) message;
-  
+  obj = &conn->object;
+  extra = &conn->extra_spacing;
+
   obj->type = &message_type;
   obj->ops = &message_ops;
   
   connection_init(conn, 3, 0);
 
-  message->text = NULL;
+  message->text = strdup("");
   message->text_width = 0.0;
   message->text_pos.x = 0.5*(conn->endpoints[0].x + conn->endpoints[1].x);
   message->text_pos.y = 0.5*(conn->endpoints[0].y + conn->endpoints[1].y);
@@ -332,11 +394,16 @@ message_create(Point *startpoint,
   message->text_handle.connected_to = NULL;
   obj->handles[2] = &message->text_handle;
   
+  extra->start_long = 
+    extra->start_trans = 
+    extra->end_long = MESSAGE_WIDTH/2.0;
+  extra->end_trans = MAX(MESSAGE_WIDTH,MESSAGE_ARROWLEN)/2.0;
+  
   message_update_data(message);
 
   *handle1 = obj->handles[0];
   *handle2 = obj->handles[1];
-  return (Object *)message;
+  return &message->connection.object;
 }
 
 
@@ -357,22 +424,22 @@ message_copy(Message *message)
   
   conn = &message->connection;
   
-  newmessage = g_malloc(sizeof(Message));
+  newmessage = g_malloc0(sizeof(Message));
   newconn = &newmessage->connection;
-  newobj = (Object *) newmessage;
+  newobj = &newconn->object;
 
   connection_copy(conn, newconn);
 
   newmessage->text_handle = message->text_handle;
   newobj->handles[2] = &newmessage->text_handle;
 
-  newmessage->text = (message->text) ? strdup(message->text): NULL;
+  newmessage->text = strdup(message->text);
   newmessage->text_pos = message->text_pos;
   newmessage->text_width = message->text_width;
 
   newmessage->type = message->type;
 
-  return (Object *)newmessage;
+  return &newmessage->connection.object;
 }
 
 static void
@@ -385,7 +452,7 @@ message_state_free(ObjectState *ostate)
 static MessageState *
 message_get_state(Message *message)
 {
-  MessageState *state = g_new(MessageState, 1);
+  MessageState *state = g_new0(MessageState, 1);
 
   state->obj_state.free = message_state_free;
 
@@ -412,7 +479,7 @@ static void
 message_update_data(Message *message)
 {
   Connection *conn = &message->connection;
-  Object *obj = (Object *) message;
+  Object *obj = &conn->object;
   Rectangle rect;
   
   obj->position = conn->endpoints[0];
@@ -420,22 +487,17 @@ message_update_data(Message *message)
   message->text_handle.pos = message->text_pos;
 
   connection_update_handles(conn);
-
-  /* Boundingbox: */
   connection_update_boundingbox(conn);
 
+  message->text_width =
+    font_string_width(message->text, message_font, MESSAGE_FONTHEIGHT);
+
   /* Add boundingbox for text: */
-  rect.left = message->text_pos.x;
+  rect.left = message->text_pos.x-message->text_width/2;
   rect.right = rect.left + message->text_width;
   rect.top = message->text_pos.y - font_ascent(message_font, MESSAGE_FONTHEIGHT);
   rect.bottom = rect.top + MESSAGE_FONTHEIGHT;
   rectangle_union(&obj->bounding_box, &rect);
-
-  /* fix boundingbox for message_width and arrow: */
-  obj->bounding_box.top -= MESSAGE_WIDTH/2 + MESSAGE_ARROWLEN;
-  obj->bounding_box.left -= MESSAGE_WIDTH/2 + MESSAGE_ARROWLEN;
-  obj->bounding_box.bottom += MESSAGE_WIDTH/2 + MESSAGE_ARROWLEN;
-  obj->bounding_box.right += MESSAGE_WIDTH/2 + MESSAGE_ARROWLEN;
 }
 
 
@@ -458,15 +520,17 @@ message_load(ObjectNode obj_node, int version, const char *filename)
   Message *message;
   AttributeNode attr;
   Connection *conn;
+  ConnectionBBExtras *extra;
   Object *obj;
 
   if (message_font == NULL)
     message_font = font_getfont("Helvetica");
 
-  message = g_malloc(sizeof(Message));
+  message = g_malloc0(sizeof(Message));
 
   conn = &message->connection;
-  obj = (Object *) message;
+  obj = &conn->object;
+  extra = &conn->extra_spacing;
 
   obj->type = &message_type;
   obj->ops = &message_ops;
@@ -488,11 +552,7 @@ message_load(ObjectNode obj_node, int version, const char *filename)
   if (attr != NULL)
     message->type = (MessageType)data_int(attribute_first_data(attr));
 
-  if (message->text)
-    message->text_width =
-      font_string_width(message->text, message_font, MESSAGE_FONTHEIGHT);
-  else
-    message->text_width = 0;
+  message->text_width = 0;
   
   message->text_handle.id = HANDLE_MOVE_TEXT;
   message->text_handle.type = HANDLE_MINOR_CONTROL;
@@ -500,175 +560,31 @@ message_load(ObjectNode obj_node, int version, const char *filename)
   message->text_handle.connected_to = NULL;
   obj->handles[2] = &message->text_handle;
   
+  extra->start_long = 
+    extra->start_trans = 
+    extra->end_long = MESSAGE_WIDTH/2.0;
+  extra->end_trans = MAX(MESSAGE_WIDTH,MESSAGE_ARROWLEN)/2.0;
+  
   message_update_data(message);
   
-  return (Object *)message;
+  return &message->connection.object;
 }
 
 
 static ObjectChange *
-message_apply_properties(Message *message)
+message_apply_properties(Message *message, GtkWidget *widget)
 {
-  MessageDialog *prop_dialog;
   ObjectState *old_state;
   
-  prop_dialog = properties_dialog;
-
   old_state = (ObjectState *)message_get_state(message);
 
+  object_apply_props_from_dialog((Object *)message, widget);
+
   /* Read from dialog and put in object: */
-  g_free(message->text);
-  message->text = strdup(gtk_entry_get_text(prop_dialog->text));
     
-  message->text_width =
-      font_string_width(message->text, message_font,
-			MESSAGE_FONTHEIGHT);
-  
-  
-  if (GTK_TOGGLE_BUTTON(prop_dialog->m_call)->active) 
-      message->type = MESSAGE_CALL;
-  else if (GTK_TOGGLE_BUTTON( prop_dialog->m_return )->active) 
-      message->type = MESSAGE_RETURN;
-  else if (GTK_TOGGLE_BUTTON( prop_dialog->m_create )->active) 
-      message->type = MESSAGE_CREATE;
-  else if (GTK_TOGGLE_BUTTON( prop_dialog->m_destroy )->active) 
-      message->type = MESSAGE_DESTROY;
-  else if (GTK_TOGGLE_BUTTON( prop_dialog->m_send )->active) 
-      message->type = MESSAGE_SEND;
-  else if (GTK_TOGGLE_BUTTON( prop_dialog->m_simple )->active) 
-      message->type = MESSAGE_SIMPLE;
-  else if (GTK_TOGGLE_BUTTON( prop_dialog->m_recursive )->active) 
-      message->type = MESSAGE_RECURSIVE;
-  
   message_update_data(message);
 
-  return new_object_state_change((Object *)message, old_state, 
+  return new_object_state_change(&message->connection.object, old_state, 
 				 (GetStateFunc)message_get_state,
 				 (SetStateFunc)message_set_state);
 }
-
-static void
-fill_in_dialog(Message *message)
-{
-  MessageDialog *prop_dialog;
-  char *str;
-  GtkToggleButton *button=NULL;
-
-  prop_dialog = properties_dialog;
-
-  if (message->text) {
-      str = strdup(message->text);
-      gtk_entry_set_text(prop_dialog->text, str);
-      g_free(str);
-  } else {
-      gtk_entry_set_text(prop_dialog->text, "");
-  }
-
-  switch (message->type) {
-  case MESSAGE_CALL:
-      button = GTK_TOGGLE_BUTTON(prop_dialog->m_call);
-      break;
-  case MESSAGE_CREATE:
-      button = GTK_TOGGLE_BUTTON(prop_dialog->m_create);
-      break;
-  case MESSAGE_DESTROY:
-      button = GTK_TOGGLE_BUTTON(prop_dialog->m_destroy);
-      break;
-  case MESSAGE_SIMPLE:
-      button = GTK_TOGGLE_BUTTON(prop_dialog->m_simple);
-      break;
-  case MESSAGE_RETURN:
-      button = GTK_TOGGLE_BUTTON(prop_dialog->m_return);
-      break;
-  case MESSAGE_SEND:
-      button = GTK_TOGGLE_BUTTON(prop_dialog->m_send);
-      break;
-  case MESSAGE_RECURSIVE:
-      button = GTK_TOGGLE_BUTTON(prop_dialog->m_recursive);
-      break;
-  }
-  if (button)
-    gtk_toggle_button_set_active(button, TRUE);
-}
-
-static GtkWidget *
-message_get_properties(Message *message)
-{
-  MessageDialog *prop_dialog;
-  GtkWidget *dialog;
-  GtkWidget *entry;
-  GtkWidget *hbox;
-  GtkWidget *label;
-  GSList *group;
-
-  if (properties_dialog == NULL) {
-
-    prop_dialog = g_new(MessageDialog, 1);
-    properties_dialog = prop_dialog;
-    
-    dialog = gtk_vbox_new(FALSE, 0);
-    gtk_object_ref(GTK_OBJECT(dialog));
-    gtk_object_sink(GTK_OBJECT(dialog));
-    prop_dialog->dialog = dialog;
-    
-    hbox = gtk_hbox_new(FALSE, 5);
-
-    label = gtk_label_new(_("Message:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    entry = gtk_entry_new();
-    prop_dialog->text = GTK_ENTRY(entry);
-    gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
-    gtk_widget_show (label);
-    gtk_widget_show (entry);
-    gtk_box_pack_start (GTK_BOX (dialog), hbox, TRUE, TRUE, 0);
-    gtk_widget_show(hbox);
-
-    label = gtk_hseparator_new ();
-    gtk_box_pack_start (GTK_BOX (dialog), label, FALSE, TRUE, 0);
-    gtk_widget_show (label);
-
-    label = gtk_label_new(_("Message type:"));
-    gtk_box_pack_start (GTK_BOX (dialog), label, FALSE, TRUE, 0);
-    gtk_widget_show (label);
-
-    /* */
-    prop_dialog->m_call = gtk_radio_button_new_with_label (NULL, _("Call"));
-    gtk_box_pack_start (GTK_BOX (dialog), prop_dialog->m_call, TRUE, TRUE, 0);
-    gtk_widget_show (prop_dialog->m_call);
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prop_dialog->m_call), TRUE);
-    group = gtk_radio_button_group (GTK_RADIO_BUTTON (prop_dialog->m_call));
-    prop_dialog->m_return = gtk_radio_button_new_with_label(group, _("Return"));
-    gtk_box_pack_start (GTK_BOX (dialog), prop_dialog->m_return, TRUE, TRUE, 0);
-    gtk_widget_show (prop_dialog->m_return);
-
-    group = gtk_radio_button_group (GTK_RADIO_BUTTON (prop_dialog->m_return));
-    prop_dialog->m_send = gtk_radio_button_new_with_label(group, _("Asynchronous"));
-    gtk_box_pack_start (GTK_BOX (dialog), prop_dialog->m_send, TRUE, TRUE, 0);
-    gtk_widget_show (prop_dialog->m_send);
-
-    group = gtk_radio_button_group (GTK_RADIO_BUTTON (prop_dialog->m_send));
-    prop_dialog->m_create = gtk_radio_button_new_with_label(group, _("Create"));
-    gtk_box_pack_start (GTK_BOX (dialog), prop_dialog->m_create, TRUE, TRUE, 0);
-    gtk_widget_show (prop_dialog->m_create);
-
-    group = gtk_radio_button_group (GTK_RADIO_BUTTON (prop_dialog->m_create));
-    prop_dialog->m_destroy = gtk_radio_button_new_with_label(group, _("Destroy"));
-    gtk_box_pack_start (GTK_BOX (dialog), prop_dialog->m_destroy, TRUE, TRUE, 0);
-    gtk_widget_show (prop_dialog->m_destroy);
-
-    group = gtk_radio_button_group (GTK_RADIO_BUTTON (prop_dialog->m_destroy));
-    prop_dialog->m_simple = gtk_radio_button_new_with_label(group, _("Simple"));
-    gtk_box_pack_start (GTK_BOX (dialog), prop_dialog->m_simple, TRUE, TRUE, 0);
-    gtk_widget_show (prop_dialog->m_simple);
-
-    group = gtk_radio_button_group (GTK_RADIO_BUTTON (prop_dialog->m_simple));
-    prop_dialog->m_recursive = gtk_radio_button_new_with_label(group, _("Recursive"));
-    gtk_box_pack_start (GTK_BOX (dialog), prop_dialog->m_recursive, TRUE, TRUE, 0);
-    gtk_widget_show (prop_dialog->m_recursive);
-  }
-  
-  fill_in_dialog(message);
-  gtk_widget_show (properties_dialog->dialog);
-
-  return properties_dialog->dialog;
-}
