@@ -15,18 +15,23 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
 #include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 
-#include "config.h"
 #include "intl.h"
 #include "object.h"
 #include "element.h"
 #include "render.h"
 #include "attributes.h"
 #include "text.h"
+#include "properties.h"
 
 #include "pixmaps/note.xpm"
 
@@ -64,6 +69,10 @@ static void note_save(Note *note, ObjectNode obj_node,
 static Object *note_load(ObjectNode obj_node, int version,
 			 const char *filename);
 
+static PropDescription *note_describe_props(Note *note);
+static void note_get_props(Note *note, Property *props, guint nprops);
+static void note_set_props(Note *note, Property *props, guint nprops);
+
 static void note_update_data(Note *note);
 
 static ObjectTypeOps note_type_ops =
@@ -90,11 +99,104 @@ static ObjectOps note_ops = {
   (CopyFunc)            note_copy,
   (MoveFunc)            note_move,
   (MoveHandleFunc)      note_move_handle,
-  (GetPropertiesFunc)   object_return_null,
-  (ApplyPropertiesFunc) object_return_void,
-  (ObjectMenuFunc)      NULL
+  (GetPropertiesFunc)   object_create_props_dialog,
+  (ApplyPropertiesFunc) object_apply_props_from_dialog,
+  (ObjectMenuFunc)      NULL,
+  (DescribePropsFunc)   note_describe_props,
+  (GetPropsFunc)        note_get_props,
+  (SetPropsFunc)        note_set_props
 };
 
+static PropDescription note_props[] = {
+  ELEMENT_COMMON_PROPERTIES,
+  PROP_STD_TEXT_FONT,
+  PROP_STD_TEXT_HEIGHT,
+  PROP_STD_TEXT_COLOUR,
+  PROP_STD_TEXT,
+  
+  PROP_DESC_END
+};
+
+static PropDescription *
+note_describe_props(Note *note)
+{
+  if (note_props[0].quark == 0)
+    prop_desc_list_calculate_quarks(note_props);
+  return note_props;
+}
+
+static PropOffset note_offsets[] = {
+  ELEMENT_COMMON_PROPERTIES_OFFSETS,
+  { NULL, 0, 0 },
+};
+
+static struct { const gchar *name; GQuark q; } quarks[] = {
+  { "text_font" },
+  { "text_height" },
+  { "text_colour" },
+  { "text" }
+};
+
+static void
+note_get_props(Note * note, Property *props, guint nprops)
+{
+  guint i;
+
+  if (object_get_props_from_offsets(&note->element.object, 
+                                    note_offsets, props, nprops))
+    return;
+  /* these props can't be handled as easily */
+  if (quarks[0].q == 0)
+    for (i = 0; i < sizeof(quarks)/sizeof(*quarks); i++)
+      quarks[i].q = g_quark_from_static_string(quarks[i].name);
+  for (i = 0; i < nprops; i++) {
+    GQuark pquark = g_quark_from_string(props[i].name);
+
+    if (pquark == quarks[0].q) {
+      props[i].type = PROP_TYPE_FONT;
+      PROP_VALUE_FONT(props[i]) = note->text->font;
+    } else if (pquark == quarks[1].q) {
+      props[i].type = PROP_TYPE_REAL;
+      PROP_VALUE_REAL(props[i]) = note->text->height;
+    } else if (pquark == quarks[2].q) {
+      props[i].type = PROP_TYPE_COLOUR;
+      PROP_VALUE_COLOUR(props[i]) = note->text->color;
+    } else if (pquark == quarks[3].q) {
+      props[i].type = PROP_TYPE_STRING;
+      g_free(PROP_VALUE_STRING(props[i]));
+      PROP_VALUE_STRING(props[i]) = text_get_string_copy(note->text);
+    }
+  }
+}
+
+static void
+note_set_props(Note *note, Property *props, guint nprops)
+{
+  if (!object_set_props_from_offsets(&note->element.object, 
+                                     note_offsets, props, nprops)) {
+    guint i;
+
+    if (quarks[0].q == 0)
+      for (i = 0; i < sizeof(quarks)/sizeof(*quarks); i++)
+	quarks[i].q = g_quark_from_static_string(quarks[i].name);
+
+    for (i = 0; i < nprops; i++) {
+      GQuark pquark = g_quark_from_string(props[i].name);
+
+      if (pquark == quarks[0].q && props[i].type == PROP_TYPE_FONT) {
+	text_set_font(note->text, PROP_VALUE_FONT(props[i]));
+      } else if (pquark == quarks[1].q && props[i].type == PROP_TYPE_REAL) {
+	text_set_height(note->text, PROP_VALUE_REAL(props[i]));
+      } else if (pquark == quarks[2].q && props[i].type == PROP_TYPE_COLOUR) {
+	text_set_color(note->text, &PROP_VALUE_COLOUR(props[i]));
+      } else if (pquark == quarks[3].q && props[i].type == PROP_TYPE_STRING) {
+	text_set_string(note->text, PROP_VALUE_STRING(props[i]));
+      }
+    }
+  }
+  note_update_data(note);
+}
+
 static real
 note_distance_from(Note *note, Point *point)
 {
@@ -107,7 +209,7 @@ note_select(Note *note, Point *clicked_point,
 	       Renderer *interactive_renderer)
 {
   text_set_cursor(note->text, clicked_point, interactive_renderer);
-  text_grab_focus(note->text, (Object *)note);
+  text_grab_focus(note->text, &note->element.object);
   element_update_handles(&note->element);
 }
 
@@ -125,14 +227,8 @@ note_move_handle(Note *note, Handle *handle,
 static void
 note_move(Note *note, Point *to)
 {
-  Point p;
-  
   note->element.corner = *to;
 
-  p = *to;
-  p.x += NOTE_BORDERWIDTH/2.0 + NOTE_MARGIN_X;
-  p.y += NOTE_BORDERWIDTH/2.0 + NOTE_CORNER + note->text->ascent;
-  text_set_position(note->text, &p);
   note_update_data(note);
 }
 
@@ -192,12 +288,18 @@ static void
 note_update_data(Note *note)
 {
   Element *elem = &note->element;
-  Object *obj = (Object *) note;
-  
+  Object *obj = &elem->object;
+  Point p;
+
   elem->width = note->text->max_width + NOTE_MARGIN_X + NOTE_CORNER;
   elem->height =
     note->text->height*note->text->numlines + NOTE_MARGIN_Y + NOTE_CORNER;
 
+  p = elem->corner;
+  p.x += NOTE_BORDERWIDTH/2.0 + NOTE_MARGIN_X;
+  p.y += NOTE_BORDERWIDTH/2.0 + NOTE_CORNER + note->text->ascent;
+  text_set_position(note->text, &p);
+
   /* Update connections: */
   note->connections[0].pos = elem->corner;
   note->connections[1].pos.x = elem->corner.x + elem->width / 2.0;
@@ -216,14 +318,8 @@ note_update_data(Note *note)
   note->connections[7].pos.y = elem->corner.y + elem->height;
   
   element_update_boundingbox(elem);
-  /* fix boundingnote for line_width: */
-  obj->bounding_box.top -= NOTE_BORDERWIDTH/2.0;
-  obj->bounding_box.left -= NOTE_BORDERWIDTH/2.0;
-  obj->bounding_box.bottom += NOTE_BORDERWIDTH/2.0;
-  obj->bounding_box.right += NOTE_BORDERWIDTH/2.0;
 
   obj->position = elem->corner;
-
   element_update_handles(elem);
 }
 
@@ -240,9 +336,9 @@ note_create(Point *startpoint,
   Font *font;
   int i;
   
-  note = g_malloc(sizeof(Note));
+  note = g_malloc0(sizeof(Note));
   elem = &note->element;
-  obj = (Object *) note;
+  obj = &elem->object;
   
   obj->type = &note_type;
 
@@ -264,6 +360,7 @@ note_create(Point *startpoint,
     note->connections[i].object = obj;
     note->connections[i].connected = NULL;
   }
+  elem->extra_spacing.border_trans = NOTE_BORDERWIDTH/2.0;
   note_update_data(note);
 
   for (i=0;i<8;i++) {
@@ -272,7 +369,7 @@ note_create(Point *startpoint,
 
   *handle1 = NULL;
   *handle2 = NULL;
-  return (Object *)note;
+  return &note->element.object;
 }
 
 static void
@@ -293,9 +390,9 @@ note_copy(Note *note)
   
   elem = &note->element;
   
-  newnote = g_malloc(sizeof(Note));
+  newnote = g_malloc0(sizeof(Note));
   newelem = &newnote->element;
-  newobj = (Object *) newnote;
+  newobj = &newelem->object;
 
   element_copy(elem, newelem);
 
@@ -308,10 +405,9 @@ note_copy(Note *note)
     newnote->connections[i].pos = note->connections[i].pos;
     newnote->connections[i].last_pos = note->connections[i].last_pos;
   }
-
   note_update_data(newnote);
   
-  return (Object *)newnote;
+  return &newnote->element.object;
 }
 
 
@@ -333,9 +429,9 @@ note_load(ObjectNode obj_node, int version, const char *filename)
   Object *obj;
   int i;
   
-  note = g_malloc(sizeof(Note));
+  note = g_malloc0(sizeof(Note));
   elem = &note->element;
-  obj = (Object *) note;
+  obj = &elem->object;
   
   obj->type = &note_type;
   obj->ops = &note_ops;
@@ -354,13 +450,14 @@ note_load(ObjectNode obj_node, int version, const char *filename)
     note->connections[i].object = obj;
     note->connections[i].connected = NULL;
   }
+  elem->extra_spacing.border_trans = NOTE_BORDERWIDTH/2.0;
   note_update_data(note);
 
   for (i=0;i<8;i++) {
     obj->handles[i]->type = HANDLE_NON_MOVABLE;
   }
 
-  return (Object *)note;
+  return &note->element.object;
 }
 
 
