@@ -15,19 +15,24 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
 #include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 #include <stdio.h>
 
-#include "config.h"
 #include "intl.h"
 #include "object.h"
 #include "element.h"
 #include "render.h"
 #include "attributes.h"
 #include "text.h"
+#include "properties.h"
 
 #include "uml.h"
 
@@ -103,6 +108,9 @@ static void objet_save(Objet *ob, ObjectNode obj_node,
 		       const char *filename);
 static Object *objet_load(ObjectNode obj_node, int version,
 			  const char *filename);
+static PropDescription *objet_describe_props(Objet *objet);
+static void objet_get_props(Objet *objet, Property *props, guint nprops);
+static void objet_set_props(Objet *objet, Property *props, guint nprops);
 static void objet_update_data(Objet *ob);
 static GtkWidget *objet_get_properties(Objet *ob);
 static ObjectChange *objet_apply_properties(Objet *ob);
@@ -147,9 +155,69 @@ static ObjectOps objet_ops = {
   (MoveHandleFunc)      objet_move_handle,
   (GetPropertiesFunc)   objet_get_properties,
   (ApplyPropertiesFunc) objet_apply_properties,
-  (ObjectMenuFunc)      NULL
+  (ObjectMenuFunc)      NULL,
+  (DescribePropsFunc)   objet_describe_props,
+  (GetPropsFunc)        objet_get_props,
+  (SetPropsFunc)        objet_set_props,
 };
 
+
+static PropDescription objet_props[] = {
+  ELEMENT_COMMON_PROPERTIES,
+  { "stereotype", PROP_TYPE_STRING, PROP_FLAG_VISIBLE,
+    N_("Stereotype"), NULL, NULL },
+  PROP_STD_TEXT,
+  { "name", PROP_TYPE_STRING, PROP_FLAG_VISIBLE,
+    N_("Class name"), NULL, NULL },
+
+  /*
+   * XXX Lists: "attributes" "operations" "templates"
+   */
+
+  { NULL, 0, 0, NULL, NULL, NULL, 0}
+};
+
+static PropDescription *
+objet_describe_props(Objet *ob)
+{
+  if (objet_props[0].quark == 0)
+    prop_desc_list_calculate_quarks(objet_props);
+  return objet_props;
+}
+
+static PropOffset objet_offsets[] = {
+  ELEMENT_COMMON_PROPERTIES_OFFSETS,
+  { "name", PROP_TYPE_STRING, offsetof(Objet, exstate) },
+  { "stereotype", PROP_TYPE_STRING, offsetof(Objet, stereotype) },
+  { NULL, 0, 0 },
+};
+
+#ifdef THIS_IS_PROBABLY_DEAD_CODE
+static struct { const gchar *name; GQuark q; } quarks[] = {
+  { "name" },
+  { "stereotype" } 
+};
+#endif
+
+static void
+objet_get_props(Objet * objet, Property *props, guint nprops)
+{
+  if (object_get_props_from_offsets(&objet->element.object, 
+                                    objet_offsets, props, nprops))
+    return;
+  /* XXX ? */
+}
+
+static void
+objet_set_props(Objet *objet, Property *props, guint nprops)
+{
+  if (!object_set_props_from_offsets(&objet->element.object, objet_offsets,
+		     props, nprops)) {
+    /* XXX ? */
+  }
+  objet_update_data(objet);
+}
+
 static real
 objet_distance_from(Objet *ob, Point *point)
 {
@@ -162,7 +230,7 @@ objet_select(Objet *ob, Point *clicked_point,
 	       Renderer *interactive_renderer)
 {
   text_set_cursor(ob->text, clicked_point, interactive_renderer);
-  text_grab_focus(ob->text, (Object *)ob);
+  text_grab_focus(ob->text, &ob->element.object);
   element_update_handles(&ob->element);
 }
 
@@ -286,7 +354,7 @@ static void
 objet_update_data(Objet *ob)
 {
   Element *elem = &ob->element;
-  Object *obj = (Object *) ob;
+  Object *obj = &elem->object;
   Font *font;
   Point p1, p2;
   real h, w = 0;
@@ -362,14 +430,7 @@ objet_update_data(Objet *ob)
   ob->connections[7].pos.y = elem->corner.y + elem->height;
   
   element_update_boundingbox(elem);
-  /* fix boundingobjet for line width and top rectangle: */
-  obj->bounding_box.top -= OBJET_BORDERWIDTH/2.0;
-  obj->bounding_box.left -= OBJET_BORDERWIDTH/2.0;
-  obj->bounding_box.bottom += OBJET_BORDERWIDTH/2.0;
-  obj->bounding_box.right += OBJET_BORDERWIDTH/2.0;
-
   obj->position = elem->corner;
-
   element_update_handles(elem);
 }
 
@@ -386,9 +447,9 @@ objet_create(Point *startpoint,
   Font *font;
   int i;
   
-  ob = g_malloc(sizeof(Objet));
+  ob = g_malloc0(sizeof(Objet));
   elem = &ob->element;
-  obj = (Object *) ob;
+  obj = &elem->object;
   
   obj->type = &umlobject_type;
 
@@ -418,6 +479,7 @@ objet_create(Point *startpoint,
     ob->connections[i].object = obj;
     ob->connections[i].connected = NULL;
   }
+  elem->extra_spacing.border_trans = OBJET_BORDERWIDTH/2.0;
   objet_update_data(ob);
 
   for (i=0;i<8;i++) {
@@ -427,7 +489,7 @@ objet_create(Point *startpoint,
   *handle1 = NULL;
   *handle2 = NULL;
 
-  return (Object *)ob;
+  return &ob->element.object;
 }
 
 static void
@@ -456,9 +518,9 @@ objet_copy(Objet *ob)
   
   elem = &ob->element;
   
-  newob = g_malloc(sizeof(Objet));
+  newob = g_malloc0(sizeof(Objet));
   newelem = &newob->element;
-  newobj = (Object *) newob;
+  newobj = &newelem->object;
 
   element_copy(elem, newelem);
 
@@ -486,7 +548,7 @@ objet_copy(Objet *ob)
   
   objet_update_data(newob);
   
-  return (Object *)newob;
+  return &newob->element.object;
 }
 
 static void
@@ -501,7 +563,7 @@ objet_state_free(ObjectState *ostate)
 static ObjetState *
 objet_get_state(Objet *ob)
 {
-  ObjetState *state = g_new(ObjetState, 1);
+  ObjetState *state = g_new0(ObjetState, 1);
 
   state->obj_state.free = objet_state_free;
 
@@ -571,9 +633,9 @@ objet_load(ObjectNode obj_node, int version, const char *filename)
   Object *obj;
   int i;
   
-  ob = g_malloc(sizeof(Objet));
+  ob = g_malloc0(sizeof(Objet));
   elem = &ob->element;
-  obj = (Object *) ob;
+  obj = &elem->object;
   
   obj->type = &objet_type;
   obj->ops = &objet_ops;
@@ -625,13 +687,14 @@ objet_load(ObjectNode obj_node, int version, const char *filename)
     ob->connections[i].object = obj;
     ob->connections[i].connected = NULL;
   }
+  elem->extra_spacing.border_trans = OBJET_BORDERWIDTH/2.0;
   objet_update_data(ob);
 
   for (i=0;i<8;i++) {
     obj->handles[i]->type = HANDLE_NON_MOVABLE;
   }
 
-  return (Object *) ob;
+  return &ob->element.object;
 }
 
 
@@ -683,7 +746,7 @@ objet_apply_properties(Objet *ob)
 
   objet_update_data(ob);
   
-  return new_object_state_change((Object *)ob, old_state, 
+  return new_object_state_change(&ob->element.object, old_state, 
 				 (GetStateFunc)objet_get_state,
 				 (SetStateFunc)objet_set_state);
 }
