@@ -15,18 +15,25 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
 #include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 
-#include "config.h"
 #include "intl.h"
 #include "object.h"
 #include "element.h"
 #include "render.h"
 #include "attributes.h"
 #include "text.h"
+#include "properties.h"
+
+#include "stereotype.h"
 
 #include "pixmaps/smallpackage.xpm"
 
@@ -37,7 +44,6 @@ struct _SmallPackage {
   ConnectionPoint connections[8];
 
   char *stereotype;
-  char *name;
   Text *text;
 };
 
@@ -66,6 +72,10 @@ static void smallpackage_save(SmallPackage *pkg, ObjectNode obj_node,
 static Object *smallpackage_load(ObjectNode obj_node, int version,
 				 const char *filename);
 
+static PropDescription *smallpackage_describe_props(SmallPackage *smallpackage);
+static void smallpackage_get_props(SmallPackage *smallpackage, Property *props, guint nprops);
+static void smallpackage_set_props(SmallPackage *smallpackage, Property *props, guint nprops);
+
 static void smallpackage_update_data(SmallPackage *pkg);
 
 static ObjectTypeOps smallpackage_type_ops =
@@ -92,11 +102,126 @@ static ObjectOps smallpackage_ops = {
   (CopyFunc)            smallpackage_copy,
   (MoveFunc)            smallpackage_move,
   (MoveHandleFunc)      smallpackage_move_handle,
-  (GetPropertiesFunc)   object_return_null,
-  (ApplyPropertiesFunc) object_return_void,
-  (ObjectMenuFunc)      NULL
+  (GetPropertiesFunc)   object_create_props_dialog,
+  (ApplyPropertiesFunc) object_apply_props_from_dialog,
+  (ObjectMenuFunc)      NULL,
+  (DescribePropsFunc)   smallpackage_describe_props,
+  (GetPropsFunc)        smallpackage_get_props,
+  (SetPropsFunc)        smallpackage_set_props,
 };
 
+static PropDescription smallpackage_props[] = {
+  ELEMENT_COMMON_PROPERTIES,
+  { "stereotype", PROP_TYPE_STRING, PROP_FLAG_VISIBLE,
+  N_("Stereotype"), NULL, NULL },
+  PROP_STD_TEXT_FONT,
+  PROP_STD_TEXT_HEIGHT,
+  PROP_STD_TEXT_COLOUR,
+  PROP_STD_TEXT,
+  
+  PROP_DESC_END
+};
+
+static PropDescription *
+smallpackage_describe_props(SmallPackage *smallpackage)
+{
+  if (smallpackage_props[0].quark == 0)
+    prop_desc_list_calculate_quarks(smallpackage_props);
+  return smallpackage_props;
+}
+
+static PropOffset smallpackage_offsets[] = {
+  ELEMENT_COMMON_PROPERTIES_OFFSETS,
+  /*  { "stereotype", PROP_TYPE_STRING, offsetof(SmallPackage, stereotype) },*/
+  { NULL, 0, 0 },
+};
+
+static struct { const gchar *name; GQuark q; } quarks[] = {
+  { "text_font" },
+  { "text_height" },
+  { "text_colour" },
+  { "text" },
+  { "stereotype" }
+};
+
+static void
+smallpackage_get_props(SmallPackage * smallpackage, Property *props, guint nprops)
+{
+  guint i;
+
+  if (object_get_props_from_offsets(&smallpackage->element.object, 
+                                    smallpackage_offsets, props, nprops))
+    return;
+  /* these props can't be handled as easily */
+  if (quarks[0].q == 0)
+    for (i = 0; i < sizeof(quarks)/sizeof(*quarks); i++)
+      quarks[i].q = g_quark_from_static_string(quarks[i].name);
+  for (i = 0; i < nprops; i++) {
+    GQuark pquark = g_quark_from_string(props[i].name);
+
+    if (pquark == quarks[0].q) {
+      props[i].type = PROP_TYPE_FONT;
+      PROP_VALUE_FONT(props[i]) = smallpackage->text->font;
+    } else if (pquark == quarks[1].q) {
+      props[i].type = PROP_TYPE_REAL;
+      PROP_VALUE_REAL(props[i]) = smallpackage->text->height;
+    } else if (pquark == quarks[2].q) {
+      props[i].type = PROP_TYPE_COLOUR;
+      PROP_VALUE_COLOUR(props[i]) = smallpackage->text->color;
+    } else if (pquark == quarks[3].q) {
+      props[i].type = PROP_TYPE_STRING;
+      g_free(PROP_VALUE_STRING(props[i]));
+      PROP_VALUE_STRING(props[i]) = text_get_string_copy(smallpackage->text);
+    } else if (pquark == quarks[4].q) {
+      props[i].type = PROP_TYPE_STRING;
+      g_free(PROP_VALUE_STRING(props[i]));
+      if (smallpackage->stereotype != NULL &&
+	  smallpackage->stereotype[0] != '\0')
+	PROP_VALUE_STRING(props[i]) =
+	  stereotype_to_string(smallpackage->stereotype);
+      else
+	PROP_VALUE_STRING(props[i]) = NULL;	
+    }
+  }
+}
+
+static void
+smallpackage_set_props(SmallPackage *smallpackage, Property *props, guint nprops)
+{
+  if (!object_set_props_from_offsets(&smallpackage->element.object, 
+                                     smallpackage_offsets, props, nprops)) {
+    guint i;
+
+    if (quarks[0].q == 0)
+      for (i = 0; i < sizeof(quarks)/sizeof(*quarks); i++)
+	quarks[i].q = g_quark_from_static_string(quarks[i].name);
+
+    for (i = 0; i < nprops; i++) {
+      GQuark pquark = g_quark_from_string(props[i].name);
+
+      if (pquark == quarks[0].q && props[i].type == PROP_TYPE_FONT) {
+	text_set_font(smallpackage->text, PROP_VALUE_FONT(props[i]));
+      } else if (pquark == quarks[1].q && props[i].type == PROP_TYPE_REAL) {
+	text_set_height(smallpackage->text, PROP_VALUE_REAL(props[i]));
+      } else if (pquark == quarks[2].q && props[i].type == PROP_TYPE_COLOUR) {
+	text_set_color(smallpackage->text, &PROP_VALUE_COLOUR(props[i]));
+      } else if (pquark == quarks[3].q && props[i].type == PROP_TYPE_STRING) {
+	text_set_string(smallpackage->text, PROP_VALUE_STRING(props[i]));
+      } else if (pquark == quarks[4].q && props[i].type == PROP_TYPE_STRING) {
+	if (smallpackage->stereotype != NULL)
+	  g_free(smallpackage->stereotype);
+	if (PROP_VALUE_STRING(props[i]) != NULL &&
+	    PROP_VALUE_STRING(props[i])[0] != '\0')
+	  smallpackage->stereotype =
+	    string_to_stereotype(PROP_VALUE_STRING(props[i]));
+	else 
+	  smallpackage->stereotype = NULL;
+      }
+    }
+  }
+  smallpackage_update_data(smallpackage);
+}
+
 static real
 smallpackage_distance_from(SmallPackage *pkg, Point *point)
 {
@@ -109,7 +234,7 @@ smallpackage_select(SmallPackage *pkg, Point *clicked_point,
 	       Renderer *interactive_renderer)
 {
   text_set_cursor(pkg->text, clicked_point, interactive_renderer);
-  text_grab_focus(pkg->text, (Object *)pkg);
+  text_grab_focus(pkg->text, &pkg->element.object);
   element_update_handles(&pkg->element);
 }
 
@@ -182,21 +307,44 @@ smallpackage_draw(SmallPackage *pkg, Renderer *renderer)
 			   &p1, &p2,
 			   &color_black);
 
-
   text_draw(pkg->text, renderer);
+
+  if (pkg->stereotype != NULL) {
+    p1 = pkg->text->position;
+    p1.y -= pkg->text->height;
+    renderer->ops->draw_string(renderer, pkg->stereotype, &p1, 
+			       ALIGN_LEFT, &color_black);
+  }
 }
 
 static void
 smallpackage_update_data(SmallPackage *pkg)
 {
   Element *elem = &pkg->element;
-  Object *obj = (Object *) pkg;
-  
+  Object *obj = &elem->object;
+  Point p;
+  Font *font;
+
   elem->width = pkg->text->max_width + 2*SMALLPACKAGE_MARGIN_X;
   elem->width = MAX(elem->width, SMALLPACKAGE_TOPWIDTH+1.0);
   elem->height =
     pkg->text->height*pkg->text->numlines + 2*SMALLPACKAGE_MARGIN_Y;
 
+  p = elem->corner;
+  p.x += SMALLPACKAGE_MARGIN_X;
+  p.y += SMALLPACKAGE_MARGIN_Y + pkg->text->ascent;
+
+  if (pkg->stereotype != NULL) {
+    font = pkg->text->font;
+    elem->height += pkg->text->height;
+    elem->width = MAX(elem->width, font_string_width(pkg->stereotype,
+						     font, pkg->text->height)+
+		      2*SMALLPACKAGE_MARGIN_X);
+    p.y += pkg->text->height;
+  }
+
+  pkg->text->position = p;
+
   /* Update connections: */
   pkg->connections[0].pos = elem->corner;
   pkg->connections[1].pos.x = elem->corner.x + elem->width / 2.0;
@@ -215,11 +363,8 @@ smallpackage_update_data(SmallPackage *pkg)
   pkg->connections[7].pos.y = elem->corner.y + elem->height;
   
   element_update_boundingbox(elem);
-  /* fix boundingsmallpackage for line width and top rectangle: */
-  obj->bounding_box.top -= SMALLPACKAGE_BORDERWIDTH/2.0 + SMALLPACKAGE_TOPHEIGHT;
-  obj->bounding_box.left -= SMALLPACKAGE_BORDERWIDTH/2.0;
-  obj->bounding_box.bottom += SMALLPACKAGE_BORDERWIDTH/2.0;
-  obj->bounding_box.right += SMALLPACKAGE_BORDERWIDTH/2.0;
+  /* fix boundingbox for top rectangle: */
+  obj->bounding_box.top -= SMALLPACKAGE_TOPHEIGHT;
 
   obj->position = elem->corner;
 
@@ -239,9 +384,9 @@ smallpackage_create(Point *startpoint,
   Font *font;
   int i;
   
-  pkg = g_malloc(sizeof(SmallPackage));
+  pkg = g_malloc0(sizeof(SmallPackage));
   elem = &pkg->element;
-  obj = (Object *) pkg;
+  obj = &elem->object;
   
   obj->type = &smallpackage_type;
 
@@ -263,15 +408,18 @@ smallpackage_create(Point *startpoint,
     pkg->connections[i].object = obj;
     pkg->connections[i].connected = NULL;
   }
+  elem->extra_spacing.border_trans = SMALLPACKAGE_BORDERWIDTH/2.0;
   smallpackage_update_data(pkg);
 
   for (i=0;i<8;i++) {
     obj->handles[i]->type = HANDLE_NON_MOVABLE;
   }
 
+  pkg->stereotype = NULL;
+
   *handle1 = NULL;
   *handle2 = NULL;
-  return (Object *)pkg;
+  return &pkg->element.object;
 }
 
 static void
@@ -279,6 +427,9 @@ smallpackage_destroy(SmallPackage *pkg)
 {
   text_destroy(pkg->text);
 
+  if (pkg->stereotype)
+    g_free(pkg->stereotype);
+
   element_destroy(&pkg->element);
 }
 
@@ -292,9 +443,9 @@ smallpackage_copy(SmallPackage *pkg)
   
   elem = &pkg->element;
   
-  newpkg = g_malloc(sizeof(SmallPackage));
+  newpkg = g_malloc0(sizeof(SmallPackage));
   newelem = &newpkg->element;
-  newobj = (Object *) newpkg;
+  newobj = &newelem->object;
 
   element_copy(elem, newelem);
 
@@ -308,9 +459,14 @@ smallpackage_copy(SmallPackage *pkg)
     newpkg->connections[i].last_pos = pkg->connections[i].last_pos;
   }
 
+  if (pkg->stereotype != NULL) 
+    newpkg->stereotype = strdup(pkg->stereotype);
+  else
+    newpkg->stereotype = NULL;
+
   smallpackage_update_data(newpkg);
   
-  return (Object *)newpkg;
+  return &newpkg->element.object;
 }
 
 
@@ -320,6 +476,10 @@ smallpackage_save(SmallPackage *pkg, ObjectNode obj_node,
 {
   element_save(&pkg->element, obj_node);
 
+  if (pkg->stereotype != NULL)
+    data_add_string(new_attribute(obj_node, "stereotype"),
+		    pkg->stereotype);
+
   data_add_text(new_attribute(obj_node, "text"),
 		pkg->text);
 }
@@ -333,15 +493,22 @@ smallpackage_load(ObjectNode obj_node, int version, const char *filename)
   Object *obj;
   int i;
   
-  pkg = g_malloc(sizeof(SmallPackage));
+  pkg = g_malloc0(sizeof(SmallPackage));
   elem = &pkg->element;
-  obj = (Object *) pkg;
+  obj = &elem->object;
   
   obj->type = &smallpackage_type;
   obj->ops = &smallpackage_ops;
 
   element_load(elem, obj_node);
   
+  pkg->stereotype = NULL;
+  attr = object_find_attribute(obj_node, "stereotype");
+  if (attr != NULL)
+    pkg->stereotype = data_string(attribute_first_data(attr));
+  else
+    pkg->stereotype = NULL;
+
   pkg->text = NULL;
   attr = object_find_attribute(obj_node, "text");
   if (attr != NULL)
@@ -354,13 +521,14 @@ smallpackage_load(ObjectNode obj_node, int version, const char *filename)
     pkg->connections[i].object = obj;
     pkg->connections[i].connected = NULL;
   }
+  elem->extra_spacing.border_trans = SMALLPACKAGE_BORDERWIDTH/2.0;
   smallpackage_update_data(pkg);
 
   for (i=0;i<8;i++) {
     obj->handles[i]->type = HANDLE_NON_MOVABLE;
   }
 
-  return (Object *) pkg;
+  return &pkg->element.object;
 }
 
 
