@@ -15,22 +15,25 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 
-#include "config.h"
 #include "intl.h"
 #include "object.h"
 #include "element.h"
 #include "render.h"
 #include "attributes.h"
 #include "text.h"
+#include "properties.h"
 
 #include "pixmaps/state.xpm"
 
-typedef struct _StatePropertiesDialog StatePropertiesDialog;
 typedef struct _State State;
 typedef struct _StateState StateState;
 
@@ -57,15 +60,6 @@ struct _State {
 };
 
 
-struct _StatePropertiesDialog {
-  GtkWidget *dialog;
-
-  GtkWidget *normal;
-  GtkWidget *end;
-  GtkWidget *begin;  
-};
-
-
 #define STATE_WIDTH  4
 #define STATE_HEIGHT 3
 #define STATE_RATIO 1
@@ -74,8 +68,6 @@ struct _StatePropertiesDialog {
 #define STATE_MARGIN_X 0.5
 #define STATE_MARGIN_Y 0.5
 
-static StatePropertiesDialog *properties_dialog;
-
 static real state_distance_from(State *state, Point *point);
 static void state_select(State *state, Point *clicked_point,
 			Renderer *interactive_renderer);
@@ -94,9 +86,10 @@ static void state_save(State *state, ObjectNode obj_node,
 			 const char *filename);
 static Object *state_load(ObjectNode obj_node, int version,
 			    const char *filename);
+static PropDescription *state_describe_props(State *state);
+static void state_get_props(State *state, Property *props, guint nprops);
+static void state_set_props(State *state, Property *props, guint nprops);
 static void state_update_data(State *state);
-static ObjectChange *state_apply_properties(State *state);
-static GtkWidget *state_get_properties(State *state);
 
 static StateState *state_get_state(State *state);
 static void state_set_state(State *state, StateState *sstate);
@@ -129,11 +122,114 @@ static ObjectOps state_ops = {
   (CopyFunc)            state_copy,
   (MoveFunc)            state_move,
   (MoveHandleFunc)      state_move_handle,
-  (GetPropertiesFunc)   state_get_properties,
-  (ApplyPropertiesFunc) state_apply_properties,
-  (ObjectMenuFunc)      NULL
+  (GetPropertiesFunc)   object_create_props_dialog,
+  (ApplyPropertiesFunc) object_apply_props_from_dialog,
+  (ObjectMenuFunc)      NULL,
+  (DescribePropsFunc)   state_describe_props,
+  (GetPropsFunc)        state_get_props,
+  (SetPropsFunc)        state_set_props
 };
 
+static PropEnumData prop_state_type_data[] = {
+  { N_("Normal"), STATE_NORMAL },
+  { N_("Begin"), STATE_BEGIN },
+  { N_("End"), STATE_END },
+  { NULL, 0 }
+};
+
+static PropDescription state_props[] = {
+  ELEMENT_COMMON_PROPERTIES,
+  { "state_type", PROP_TYPE_ENUM, PROP_FLAG_VISIBLE,
+  N_("State Type"), NULL, prop_state_type_data },
+  PROP_STD_TEXT_FONT,
+  PROP_STD_TEXT_HEIGHT,
+  PROP_STD_TEXT_COLOUR,
+  PROP_STD_TEXT,
+  
+  PROP_DESC_END
+};
+
+static PropDescription *
+state_describe_props(State *state)
+{
+  if (state_props[0].quark == 0)
+    prop_desc_list_calculate_quarks(state_props);
+  return state_props;
+}
+
+static PropOffset state_offsets[] = {
+  ELEMENT_COMMON_PROPERTIES_OFFSETS,
+  { "state_type", PROP_TYPE_ENUM, offsetof(State, state_type) },
+  { NULL, 0, 0 },
+};
+
+static struct { const gchar *name; GQuark q; } quarks[] = {
+  { "text_font" },
+  { "text_height" },
+  { "text_colour" },
+  { "text" }
+};
+
+static void
+state_get_props(State * state, Property *props, guint nprops)
+{
+  guint i;
+
+  if (object_get_props_from_offsets(&state->element.object, 
+                                    state_offsets, props, nprops))
+    return;
+  /* these props can't be handled as easily */
+  if (quarks[0].q == 0)
+    for (i = 0; i < 4; i++)
+      quarks[i].q = g_quark_from_static_string(quarks[i].name);
+  for (i = 0; i < nprops; i++) {
+    GQuark pquark = g_quark_from_string(props[i].name);
+
+    if (pquark == quarks[0].q) {
+      props[i].type = PROP_TYPE_FONT;
+      PROP_VALUE_FONT(props[i]) = state->text->font;
+    } else if (pquark == quarks[1].q) {
+      props[i].type = PROP_TYPE_REAL;
+      PROP_VALUE_REAL(props[i]) = state->text->height;
+    } else if (pquark == quarks[2].q) {
+      props[i].type = PROP_TYPE_COLOUR;
+      PROP_VALUE_COLOUR(props[i]) = state->text->color;
+    } else if (pquark == quarks[3].q) {
+      props[i].type = PROP_TYPE_STRING;
+      g_free(PROP_VALUE_STRING(props[i]));
+      PROP_VALUE_STRING(props[i]) = text_get_string_copy(state->text);
+    }
+  }
+}
+
+static void
+state_set_props(State *state, Property *props, guint nprops)
+{
+  if (!object_set_props_from_offsets(&state->element.object, state_offsets,
+		     props, nprops)) {
+    guint i;
+
+    if (quarks[0].q == 0)
+      for (i = 0; i < 4; i++)
+	quarks[i].q = g_quark_from_static_string(quarks[i].name);
+
+    for (i = 0; i < nprops; i++) {
+      GQuark pquark = g_quark_from_string(props[i].name);
+
+      if (pquark == quarks[0].q && props[i].type == PROP_TYPE_FONT) {
+	text_set_font(state->text, PROP_VALUE_FONT(props[i]));
+      } else if (pquark == quarks[1].q && props[i].type == PROP_TYPE_REAL) {
+	text_set_height(state->text, PROP_VALUE_REAL(props[i]));
+      } else if (pquark == quarks[2].q && props[i].type == PROP_TYPE_COLOUR) {
+	text_set_color(state->text, &PROP_VALUE_COLOUR(props[i]));
+      } else if (pquark == quarks[3].q && props[i].type == PROP_TYPE_STRING) {
+	text_set_string(state->text, PROP_VALUE_STRING(props[i]));
+      }
+    }
+  }
+  state_update_data(state);
+}
+
 static real
 state_distance_from(State *state, Point *point)
 {
@@ -146,7 +242,7 @@ state_select(State *state, Point *clicked_point,
 	       Renderer *interactive_renderer)
 {
   text_set_cursor(state->text, clicked_point, interactive_renderer);
-  text_grab_focus(state->text, (Object *)state);
+  text_grab_focus(state->text, &state->element.object);
   element_update_handles(&state->element);
 }
 
@@ -231,7 +327,7 @@ state_update_data(State *state)
   real w, h;
 
   Element *elem = &state->element;
-  Object *obj = (Object *) state;
+  Object *obj = &elem->object;
   Point p;
   
   if (state->state_type==STATE_NORMAL) { 
@@ -286,9 +382,9 @@ state_create(Point *startpoint,
   Font *font;
   int i;
   
-  state = g_malloc(sizeof(State));
+  state = g_malloc0(sizeof(State));
   elem = &state->element;
-  obj = (Object *) state;
+  obj = &elem->object;
   
   obj->type = &state_type;
   obj->ops = &state_ops;
@@ -310,6 +406,7 @@ state_create(Point *startpoint,
     state->connections[i].object = obj;
     state->connections[i].connected = NULL;
   }
+  elem->extra_spacing.border_trans = 0.0;
   state_update_data(state);
 
   for (i=0;i<8;i++) {
@@ -318,7 +415,7 @@ state_create(Point *startpoint,
 
   *handle1 = NULL;
   *handle2 = NULL;
-  return (Object *)state;
+  return &state->element.object;
 }
 
 static void
@@ -339,9 +436,9 @@ state_copy(State *state)
   
   elem = &state->element;
   
-  newstate = g_malloc(sizeof(State));
+  newstate = g_malloc0(sizeof(State));
   newelem = &newstate->element;
-  newobj = (Object *) newstate;
+  newobj = &newelem->object;
 
   element_copy(elem, newelem);
 
@@ -359,13 +456,13 @@ state_copy(State *state)
 
   state_update_data(newstate);
   
-  return (Object *)newstate;
+  return &newstate->element.object;
 }
 
 static StateState *
 state_get_state(State *state)
 {
-  StateState *sstate = g_new(StateState, 1);
+  StateState *sstate = g_new0(StateState, 1);
 
   sstate->obj_state.free = NULL;
 
@@ -406,9 +503,9 @@ state_load(ObjectNode obj_node, int version, const char *filename)
   int i;
   AttributeNode attr;
 
-  state = g_malloc(sizeof(State));
+  state = g_malloc0(sizeof(State));
   elem = &state->element;
-  obj = (Object *) state;
+  obj = &elem->object;
   
   obj->type = &state_type;
   obj->ops = &state_ops;
@@ -429,96 +526,14 @@ state_load(ObjectNode obj_node, int version, const char *filename)
     state->connections[i].object = obj;
     state->connections[i].connected = NULL;
   }
+  elem->extra_spacing.border_trans = 0.0;
   state_update_data(state);
 
   for (i=0;i<8;i++) {
     obj->handles[i]->type = HANDLE_NON_MOVABLE;
   }
 
-  return (Object *)state;
-}
-
-
-static ObjectChange *
-state_apply_properties(State *state)
-{
-  StatePropertiesDialog *prop_dialog;
-  ObjectState *old_state;
-
-  prop_dialog = properties_dialog;
-
-  old_state = (ObjectState *)state_get_state(state);
-
-  if (GTK_TOGGLE_BUTTON(prop_dialog->normal)->active) 
-      state->state_type = STATE_NORMAL;
-  else if (GTK_TOGGLE_BUTTON(prop_dialog->begin)->active) 
-      state->state_type = STATE_BEGIN;
-  else if (GTK_TOGGLE_BUTTON(prop_dialog->end)->active) 
-      state->state_type = STATE_END;
-
-  state_update_data(state);
-  return new_object_state_change((Object *)state, old_state, 
-				 (GetStateFunc)state_get_state,
-				 (SetStateFunc)state_set_state);
-}
-
-static void
-fill_in_dialog(State *state)
-{
-  StatePropertiesDialog *prop_dialog;
-  GtkToggleButton *button=NULL;
-
-  prop_dialog = properties_dialog;
-  switch (state->state_type) {
-  case STATE_NORMAL:
-      button = GTK_TOGGLE_BUTTON(prop_dialog->normal);
-      break;
-  case STATE_BEGIN:
-      button = GTK_TOGGLE_BUTTON(prop_dialog->begin);
-      break;
-  case STATE_END:
-      button = GTK_TOGGLE_BUTTON(prop_dialog->end);
-      break;
-  }
-  if (button)
-      gtk_toggle_button_set_active(button, TRUE);
-}
-
-static GtkWidget *
-state_get_properties(State *state)
-{
-  StatePropertiesDialog *prop_dialog;
-  GtkWidget *dialog;
-  GSList *group;
-
-  if (properties_dialog == NULL) {
-    prop_dialog = g_new(StatePropertiesDialog, 1);
-    properties_dialog = prop_dialog;
-
-    dialog = gtk_vbox_new(FALSE, 0);
-    gtk_object_ref(GTK_OBJECT(dialog));
-    gtk_object_sink(GTK_OBJECT(dialog));
-    prop_dialog->dialog = dialog;
-    
-    prop_dialog->normal = gtk_radio_button_new_with_label (NULL, _("Normal"));
-    gtk_box_pack_start (GTK_BOX (dialog), prop_dialog->normal, TRUE, TRUE, 0);
-    gtk_widget_show (prop_dialog->normal);
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prop_dialog->normal), TRUE);
-    group = gtk_radio_button_group (GTK_RADIO_BUTTON (prop_dialog->normal));
-    prop_dialog->begin = gtk_radio_button_new_with_label(group, _("Begin"));
-    gtk_box_pack_start (GTK_BOX (dialog), prop_dialog->begin, TRUE, TRUE, 0);
-    gtk_widget_show (prop_dialog->begin);
-
-    group = gtk_radio_button_group (GTK_RADIO_BUTTON (prop_dialog->begin));
-    prop_dialog->end = gtk_radio_button_new_with_label(group, _("End"));
-    gtk_box_pack_start (GTK_BOX (dialog), prop_dialog->end, TRUE, TRUE, 0);
-    gtk_widget_show (prop_dialog->end);
-  }
-  
-  fill_in_dialog(state);
-  gtk_widget_show (properties_dialog->dialog);
-
-  return properties_dialog->dialog;
+  return &state->element.object;
 }
 
 
