@@ -15,11 +15,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <string.h>
 #include <stdio.h>
 #include <assert.h>
 
-#include "config.h"
 #include "intl.h"
 #include "object.h"
 #include "uml.h"
@@ -90,7 +94,10 @@ uml_get_attribute_string(UMLAttribute *attribute)
   int len;
   char *str;
 
-  len = 1 + strlen(attribute->name)  + 2 + strlen(attribute->type);
+  len = 1 + strlen(attribute->name) + strlen(attribute->type);
+  if (attribute->name[0] && attribute->type[0]) {
+    len += 2;
+  }
   if (attribute->value != NULL) {
     len += 3 + strlen(attribute->value);
   }
@@ -101,7 +108,9 @@ uml_get_attribute_string(UMLAttribute *attribute)
   str[1] = 0;
   
   strcat(str, attribute->name);
-  strcat(str, ": ");
+  if (attribute->name[0] && attribute->type[0]) {
+    strcat(str, ": ");
+  }
   strcat(str, attribute->type);
   if (attribute->value != NULL) {
     strcat(str, " = ");
@@ -129,7 +138,10 @@ uml_get_operation_string(UMLOperation *operation)
     param = (UMLParameter  *) list->data;
     list = g_list_next(list);
 
-    len += strlen(param->name) + 1 + strlen(param->type);
+    len += strlen(param->name) + strlen(param->type);
+    if (param->type[0] && param->name[0]) {
+      len += 1;
+    }
     if (param->value != NULL) {
       len += 1 + strlen(param->value);
     }
@@ -158,7 +170,9 @@ uml_get_operation_string(UMLOperation *operation)
     list = g_list_next(list);
 
     strcat(str, param->name);
-    strcat(str, ":");
+    if (param->type[0] && param->name[0]) {
+      strcat(str, ":");
+    }
     strcat(str, param->type);
     
     if (param->value != NULL) {
@@ -242,7 +256,7 @@ uml_attribute_copy(UMLAttribute *attr)
 {
   UMLAttribute *newattr;
 
-  newattr = g_new(UMLAttribute, 1);
+  newattr = g_new0(UMLAttribute, 1);
   newattr->name = strdup(attr->name);
   newattr->type = strdup(attr->type);
   if (attr->value != NULL) {
@@ -268,7 +282,7 @@ uml_operation_copy(UMLOperation *op)
   UMLParameter *newparam;
   GList *list;
   
-  newop = g_new(UMLOperation, 1);
+  newop = g_new0(UMLOperation, 1);
   newop->name = strdup(op->name);
   if (op->type != NULL) {
     newop->type = strdup(op->type);
@@ -287,7 +301,7 @@ uml_operation_copy(UMLOperation *op)
   while (list != NULL) {
     param = (UMLParameter *)list->data;
 
-    newparam = g_new(UMLParameter, 1);
+    newparam = g_new0(UMLParameter, 1);
     newparam->name = strdup(param->name);
     newparam->type = strdup(param->type);
     if (param->value != NULL)
@@ -309,7 +323,7 @@ uml_formalparameter_copy(UMLFormalParameter *param)
 {
   UMLFormalParameter *newparam;
 
-  newparam = g_new(UMLFormalParameter, 1);
+  newparam = g_new0(UMLFormalParameter, 1);
 
   newparam->name = strdup(param->name);
   if (param->type != NULL) {
@@ -374,7 +388,7 @@ uml_attribute_new(void)
 {
   UMLAttribute *attr;
   
-  attr = g_new(UMLAttribute, 1);
+  attr = g_new0(UMLAttribute, 1);
   attr->name = g_strdup("");
   attr->type = g_strdup("");
   attr->value = NULL;
@@ -392,7 +406,7 @@ uml_operation_new(void)
 {
   UMLOperation *op;
 
-  op = g_new(UMLOperation, 1);
+  op = g_new0(UMLOperation, 1);
   op->name = g_strdup("");
   op->type = NULL;
   op->visibility = UML_PUBLIC;
@@ -410,7 +424,7 @@ uml_parameter_new(void)
 {
   UMLParameter *param;
 
-  param = g_new(UMLParameter, 1);
+  param = g_new0(UMLParameter, 1);
   param->name = g_strdup("");
   param->type = g_strdup("");
   param->value = NULL;
@@ -424,7 +438,7 @@ uml_formalparameter_new(void)
 {
   UMLFormalParameter *param;
 
-  param = g_new(UMLFormalParameter, 1);
+  param = g_new0(UMLFormalParameter, 1);
   param->name = g_strdup("");
   param->type = NULL;
 
@@ -513,7 +527,7 @@ uml_attribute_read(DataNode composite)
   UMLAttribute *attr;
   AttributeNode attr_node;
   
-  attr = g_new(UMLAttribute, 1);
+  attr = g_new0(UMLAttribute, 1);
 
   attr->name = NULL;
   attr_node = composite_find_attribute(composite, "name");
@@ -561,7 +575,7 @@ uml_operation_read(DataNode composite)
   DataNode composite2;
   int i, num;
 
-  op = g_new(UMLOperation, 1);
+  op = g_new0(UMLOperation, 1);
 
   op->name = NULL;
   attr_node = composite_find_attribute(composite, "name");
@@ -593,7 +607,7 @@ uml_operation_read(DataNode composite)
   num = attribute_num_data(attr_node2);
   composite2 = attribute_first_data(attr_node2);
   for (i=0;i<num;i++) {
-    param = g_new(UMLParameter, 1);
+    param = g_new0(UMLParameter, 1);
     
     param->name = NULL;
     attr_node = composite_find_attribute(composite2, "name");
@@ -631,7 +645,7 @@ uml_formalparameter_read(DataNode composite)
   UMLFormalParameter *param;
   AttributeNode attr_node;
   
-  param = g_new(UMLFormalParameter, 1);
+  param = g_new0(UMLFormalParameter, 1);
 
   param->name = NULL;
   attr_node = composite_find_attribute(composite, "name");
