@@ -21,13 +21,17 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
 #include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 #include <glib.h>
 
-#include "config.h"
 #include "intl.h"
 #include "object.h"
 #include "element.h"
@@ -216,8 +220,9 @@ static void chronoline_set_state(Chronoline *chronoline, ChronolineState *state)
 
 static void chronoline_save(Chronoline *chronoline, ObjectNode obj_node, const char *filename);
 static Object *chronoline_load(ObjectNode obj_node, int version, const char *filename);
-static PROPDLG_TYPE chronoline_get_defaults();
-static void chronoline_apply_defaults();
+static PROPDLG_TYPE chronoline_get_defaults(void);
+static void chronoline_apply_defaults(void);
+static void chronoline_free_state(ObjectState *objstate);
 
 /*static DiaMenu *chronoline_get_object_menu(Chronoline *chronoline, Point *clickedpoint); */
 
@@ -286,7 +291,8 @@ chronoline_apply_properties(Chronoline *chronoline)
      is sovereign. */
   chronoline_update_data(chronoline);
 
-  return new_object_state_change((Object *)chronoline, old_state, 
+  return new_object_state_change(&chronoline->element.object, 
+                                 old_state, 
 				 (GetStateFunc)chronoline_get_state,
 				 (SetStateFunc)chronoline_set_state);
 }
@@ -344,7 +350,7 @@ chronoline_get_properties(Chronoline *chronoline)
   PROPDLG_RETURN(dlg);
 }
 static void
-chronoline_apply_defaults()
+chronoline_apply_defaults(void)
 {
   ChronolineDefaultsDialog *dlg = chronoline_defaults_dialog;
 
@@ -367,7 +373,7 @@ chronoline_apply_defaults()
 }
 
 static void
-chronoline_init_defaults() {
+chronoline_init_defaults(void) {
   static int defaults_initialized = 0;
 
   if (!defaults_initialized) {
@@ -394,7 +400,7 @@ chronoline_init_defaults() {
 }
 
 static GtkWidget *
-chronoline_get_defaults()
+chronoline_get_defaults(void)
 {
   ChronolineDefaultsDialog *dlg = chronoline_defaults_dialog;
 
@@ -437,7 +443,7 @@ chronoline_get_defaults()
 static real
 chronoline_distance_from(Chronoline *chronoline, Point *point)
 {
-  Object *obj = (Object *)chronoline;
+  Object *obj = &chronoline->element.object;
   return distance_rectangle_point(&obj->bounding_box, point);
 }
 
@@ -652,7 +658,7 @@ chronoline_draw(Chronoline *chronoline, Renderer *renderer)
 			     &chronoline->color);
 }
 
-void
+static void
 chronoline_free_state(ObjectState *objstate)
 {
   ChronolineState *state G_GNUC_UNUSED= (ChronolineState *)objstate;
@@ -681,7 +687,7 @@ chronoline_free_state(ObjectState *objstate)
 static ChronolineState *
 chronoline_get_state(Chronoline *chronoline)
 {
-  ChronolineState *state = g_new(ChronolineState, 1);
+  ChronolineState *state = g_new0(ChronolineState, 1);
 
   state->obj_state.free = chronoline_free_state; 
 
@@ -746,7 +752,7 @@ static void
 chronoline_update_data(Chronoline *chronoline)
 {
   Element *elem = &chronoline->element;
-  Object *obj = (Object *) chronoline;
+  Object *obj = &elem->object;
   real time_span;
   Point ur_corner;
   int shouldbe,i;
@@ -754,6 +760,7 @@ chronoline_update_data(Chronoline *chronoline)
   CLEventList *lst;
   CLEvent *evt;  
   GSList *conn_elem;
+  ElementBBExtras *extra = &elem->extra_spacing;
 
   grayify(&chronoline->datagray,&chronoline->data_color);
   grayify(&chronoline->gray,&chronoline->color);
@@ -776,18 +783,16 @@ chronoline_update_data(Chronoline *chronoline)
     chronoline->end_time = chronoline->start_time + time_span;
   }
 
+  extra->border_trans = chronoline->main_lwidth / 2;
   element_update_boundingbox(elem);
-  /* fix boundingbox for line_width: */
+
+  /* fix boundingbox for special extras: */
   realheight = obj->bounding_box.bottom - obj->bounding_box.top;
   realheight = MAX(realheight,chronoline->font_size);
   
-  obj->bounding_box.top -= chronoline->main_lwidth/2;
-  obj->bounding_box.left -= (chronoline->main_lwidth + chronoline->labelwidth);
-
-
+  obj->bounding_box.left -= chronoline->labelwidth;
   obj->bounding_box.bottom = obj->bounding_box.top + realheight + 
     chronoline->main_lwidth;
-  obj->bounding_box.right += (chronoline->main_lwidth)/2;
   
   obj->position = elem->corner;
   
@@ -863,7 +868,7 @@ static void chronoline_alloc(Chronoline **chronoline,
   *chronoline = g_new0(Chronoline,1);
   *elem = &((*chronoline)->element);
 
-  *obj = (Object *) *chronoline;
+  *obj = &((*chronoline)->element.object);
   (*obj)->type = &chronoline_type;
   (*obj)->ops = &chronoline_ops;
 }
@@ -913,7 +918,7 @@ chronoline_create(Point *startpoint,
 
   *handle1 = NULL;
   *handle2 = obj->handles[7];  
-  return (Object *)chronoline;
+  return &chronoline->element.object;
 }
 
 static void
@@ -932,7 +937,8 @@ chronoline_copy(Chronoline *chronoline)
   Object *newobj, *obj;
   int rcc = 0;
 
-  elem = &chronoline->element; obj = (Object *)chronoline;
+  elem = &chronoline->element; 
+  obj = &elem->object;
   chronoline_alloc(&newchronoline,&newelem,&newobj);
   element_copy(elem, newelem);
 
@@ -964,7 +970,7 @@ chronoline_copy(Chronoline *chronoline)
 
   chronoline_update_data(newchronoline);
 
-  return (Object *)newchronoline;
+  return &newchronoline->element.object;
 }
 
 
@@ -1029,5 +1035,5 @@ chronoline_load(ObjectNode obj_node, int version, const char *filename)
 
   chronoline_update_data(chronoline);
 
-  return (Object *)chronoline;
+  return &chronoline->element.object;
 }
