@@ -18,11 +18,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
 #include <gtk/gtk.h>
 #include <math.h>
 
-#include "config.h"
 #include "intl.h"
 #include "shape_info.h"
 #include "object.h"
@@ -44,6 +48,8 @@
 
 #define CORRECT_DISTANCE
 
+void custom_object_new(ShapeInfo *info, ObjectType **otype);
+
 /* used when resizing to decide which side of the shape to expand/shrink */
 typedef enum {
   ANCHOR_MIDDLE,
@@ -235,8 +241,8 @@ custom_get_props(Custom *custom, Property *props, guint nprops)
 {
   guint i;
 
-  if (object_get_props_from_offsets((Object *)custom, custom_offsets,
-				    props, nprops) ||
+  if (object_get_props_from_offsets(&custom->element.object, 
+                                    custom_offsets, props, nprops) ||
       !custom->info->has_text)
     return;
   if (quarks[0].q == 0)
@@ -269,8 +275,8 @@ custom_get_props(Custom *custom, Property *props, guint nprops)
 static void
 custom_set_props(Custom *custom, Property *props, guint nprops)
 {
-  if (!object_set_props_from_offsets((Object *)custom, custom_offsets,
-                                     props, nprops) &&
+  if (!object_set_props_from_offsets(&custom->element.object, 
+                                     custom_offsets, props, nprops) &&
       custom->info->has_text) {
     guint i;
 
@@ -361,7 +367,7 @@ custom_get_defaults(void)
     label = gtk_label_new(_("Text padding:"));
     gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
     gtk_widget_show (label);
-    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.0, 10.0, 0.1, 0.0, 0.0);
+    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.0, 10.0, 0.1, 1.0, 1.0);
     padding = gtk_spin_button_new(adj, 1.0, 2);
     gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(padding), TRUE);
     gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(padding), TRUE);
@@ -397,7 +403,7 @@ custom_get_defaults(void)
     label = gtk_label_new(_("Font size:"));
     gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
     gtk_widget_show (label);
-    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.1, 10.0, 0.1, 0.0, 0.0);
+    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.1, 10.0, 0.1, 1.0, 1.0);
     font_size = gtk_spin_button_new(adj, 1.0, 2);
     gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(font_size), TRUE);
     gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(font_size), TRUE);
@@ -598,7 +604,7 @@ custom_select(Custom *custom, Point *clicked_point,
 {
   if (custom->info->has_text) {
     text_set_cursor(custom->text, clicked_point, interactive_renderer);
-    text_grab_focus(custom->text, (Object *)custom);
+    text_grab_focus(custom->text, &custom->element.object);
   }
 
   element_update_handles(&custom->element);
@@ -861,12 +867,13 @@ custom_update_data(Custom *custom, AnchorShape horiz, AnchorShape vert)
 {
   Element *elem = &custom->element;
   ShapeInfo *info = custom->info;
-  Object *obj = (Object *) custom;
+  Object *obj = &elem->object;
   Point center, bottom_right;
   Point p;
   Rectangle tb;
+  ElementBBExtras *extra = &elem->extra_spacing;
   int i;
-
+  
   /* save starting points */
   center = bottom_right = elem->corner;
   center.x += elem->width/2;
@@ -882,25 +889,27 @@ custom_update_data(Custom *custom, AnchorShape horiz, AnchorShape vert)
 
   /* resize shape if text does not fit inside text_bounds */
   if (info->has_text) {
-    real width, height;
-    real xscale = 0.0, yscale = 0.0;
+    if (info->resize_with_text) {
+      real width, height;
+      real xscale = 0.0, yscale = 0.0;
 
-    transform_rect(custom, &info->text_bounds, &tb);
+      transform_rect(custom, &info->text_bounds, &tb);
 
-    width = custom->text->max_width + 2*custom->padding+custom->border_width;
-    height = custom->text->height * custom->text->numlines +
-      2 * custom->padding + custom->border_width;
+      width = custom->text->max_width + 2*custom->padding+custom->border_width;
+      height = custom->text->height * custom->text->numlines +
+	2 * custom->padding + custom->border_width;
 
-    xscale = width / (tb.right - tb.left);
-    yscale = height / (tb.bottom - tb.top);
+      xscale = width / (tb.right - tb.left);
+      yscale = height / (tb.bottom - tb.top);
 
-    if (xscale > 1.0) {
-      elem->width  *= xscale;
-      custom->xscale *= xscale;
-    }
-    if (yscale > 1.0) {
-      elem->height *= yscale;
-      custom->yscale *= yscale;
+      if (fabs(xscale - 1.0) > 0.00000001) {
+	elem->width  *= xscale;
+	custom->xscale *= xscale;
+      }
+      if (fabs(yscale - 1.0) > 0.00000001) {
+	elem->height *= yscale;
+	custom->yscale *= yscale;
+      }
     }
   }
 
@@ -997,24 +1006,28 @@ custom_update_data(Custom *custom, AnchorShape horiz, AnchorShape vert)
   for (i = 0; i < info->nconnections; i++)
     transform_coord(custom, &info->connections[i],&custom->connections[i].pos);
 
+
+  extra->border_trans = custom->border_width/2;
+
+  {
+    /* FIXME: this block is a bad hack, see BugZilla #52912.
+     I want that crap to go away ASAP after 0.87 is released -- CC */
+    GList *tmp;
+    real worst_lw = 0.0;
+
+    for (tmp = info->display_list; tmp; tmp = tmp->next) {
+      GraphicElement *el = tmp->data;
+      if (el->any.s.line_width > worst_lw) worst_lw = el->any.s.line_width;
+    }
+    extra->border_trans += worst_lw * custom->border_width * 5.24  - custom->border_width/2; 
+  }
+
   element_update_boundingbox(elem);
-  /* fix boundingbox for line_width: */
-  obj->bounding_box.top -= custom->border_width/2;
-  obj->bounding_box.left -= custom->border_width/2;
-  obj->bounding_box.bottom += custom->border_width/2;
-  obj->bounding_box.right += custom->border_width/2;
 
   /* extend bouding box to include text bounds ... */
   if (info->has_text) {
-    transform_rect(custom, &info->text_bounds, &tb);
-    if (obj->bounding_box.top > tb.top)
-      obj->bounding_box.top = tb.top;
-    if (obj->bounding_box.bottom < tb.bottom)
-      obj->bounding_box.bottom = tb.bottom;
-    if (obj->bounding_box.left > tb.left)
-      obj->bounding_box.left = tb.left;
-    if (obj->bounding_box.right < tb.right)
-      obj->bounding_box.right = tb.right;
+    text_calc_boundingbox(custom->text, &tb);
+    rectangle_union(&obj->bounding_box, &tb);
   }
   obj->position = elem->corner;
   
@@ -1042,7 +1055,7 @@ custom_create(Point *startpoint,
 
   custom = g_new0(Custom, 1);
   elem = &custom->element;
-  obj = (Object *) custom;
+  obj = &elem->object;
   
   obj->type = info->object_type;
 
@@ -1075,7 +1088,7 @@ custom_create(Point *startpoint,
   }
   element_init(elem, 8, info->nconnections);
 
-  custom->connections = g_new(ConnectionPoint, info->nconnections);
+  custom->connections = g_new0(ConnectionPoint, info->nconnections);
   for (i = 0; i < info->nconnections; i++) {
     obj->connections[i] = &custom->connections[i];
     custom->connections[i].object = obj;
@@ -1086,7 +1099,7 @@ custom_create(Point *startpoint,
 
   *handle1 = NULL;
   *handle2 = obj->handles[7];  
-  return (Object *)custom;
+  return &custom->element.object;
 }
 
 static void
@@ -1112,7 +1125,7 @@ custom_copy(Custom *custom)
   
   newcustom = g_new0(Custom, 1);
   newelem = &newcustom->element;
-  newobj = (Object *) newcustom;
+  newobj = &newcustom->element.object;
 
   element_copy(elem, newelem);
 
@@ -1132,7 +1145,7 @@ custom_copy(Custom *custom)
   if (custom->info->has_text)
     newcustom->text = text_copy(custom->text);
   
-  newcustom->connections = g_new(ConnectionPoint, custom->info->nconnections);
+  newcustom->connections = g_new0(ConnectionPoint, custom->info->nconnections);
 
   for (i = 0; i < custom->info->nconnections; i++) {
     newobj->connections[i] = &newcustom->connections[i];
@@ -1142,7 +1155,7 @@ custom_copy(Custom *custom)
     newcustom->connections[i].last_pos = custom->connections[i].last_pos;
   }
 
-  return (Object *)newcustom;
+  return &newcustom->element.object;
 }
 
 static void
@@ -1193,7 +1206,7 @@ custom_load(ObjectNode obj_node, int version, const char *filename)
 
   custom = g_new0(Custom, 1);
   elem = &custom->element;
-  obj = (Object *) custom;
+  obj = &elem->object;
   
   /* find out what type of object this is ... */
   custom->info = shape_info_get(obj_node);
@@ -1257,7 +1270,7 @@ custom_load(ObjectNode obj_node, int version, const char *filename)
 
   element_init(elem, 8, custom->info->nconnections);
 
-  custom->connections = g_new(ConnectionPoint, custom->info->nconnections);
+  custom->connections = g_new0(ConnectionPoint, custom->info->nconnections);
   for (i = 0; i < custom->info->nconnections; i++) {
     obj->connections[i] = &custom->connections[i];
     custom->connections[i].object = obj;
@@ -1266,7 +1279,7 @@ custom_load(ObjectNode obj_node, int version, const char *filename)
 
   custom_update_data(custom, ANCHOR_MIDDLE, ANCHOR_MIDDLE);
 
-  return (Object *)custom;
+  return &custom->element.object;
 }
 
 struct CustomObjectChange {
@@ -1305,7 +1318,7 @@ static ObjectChange *
 custom_flip_h_callback (Object *obj, Point *clicked, gpointer data)
 {
   Custom *custom = (Custom *)obj;
-  struct CustomObjectChange *change = g_new(struct CustomObjectChange, 1);
+  struct CustomObjectChange *change = g_new0(struct CustomObjectChange, 1);
 
   change->objchange.apply = (ObjectChangeApplyFunc)custom_change_apply;
   change->objchange.revert = (ObjectChangeRevertFunc)custom_change_revert;
@@ -1323,7 +1336,7 @@ static ObjectChange *
 custom_flip_v_callback (Object *obj, Point *clicked, gpointer data)
 {
   Custom *custom = (Custom *)obj;
-  struct CustomObjectChange *change = g_new(struct CustomObjectChange, 1);
+  struct CustomObjectChange *change = g_new0(struct CustomObjectChange, 1);
 
   change->objchange.apply = (ObjectChangeApplyFunc)custom_change_apply;
   change->objchange.revert = (ObjectChangeRevertFunc)custom_change_revert;
@@ -1359,7 +1372,7 @@ custom_get_object_menu(Custom *custom, Point *clickedpoint)
 void
 custom_object_new(ShapeInfo *info, ObjectType **otype)
 {
-  ObjectType *obj = g_new(ObjectType, 1);
+  ObjectType *obj = g_new0(ObjectType, 1);
 
   *obj = custom_type;
 
