@@ -18,12 +18,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+#include <config.h>
+
 #include <stdlib.h>
 #include <tree.h>
 #include <parser.h>
 #include <float.h>
 #include <ctype.h>
 #include <string.h>
+#include <locale.h>
 #include "shape_info.h"
 #include "custom_util.h"
 #include "intl.h"
@@ -77,6 +80,7 @@ static void
 parse_style(xmlNodePtr node, GraphicStyle *s)
 {
   char *str, *ptr;
+  char *old_locale;
 
   ptr = str = xmlGetProp(node, "style");
   if (!str)
@@ -88,7 +92,9 @@ parse_style(xmlNodePtr node, GraphicStyle *s)
 
     if (!strncmp("stroke-width:", ptr, 13)) {
       ptr += 13;
-      s->line_width = g_strtod(ptr, &ptr);
+      old_locale = setlocale(LC_NUMERIC, "C");
+      s->line_width = strtod(ptr, &ptr);
+      setlocale(LC_NUMERIC, old_locale);
     } else if (!strncmp("stroke:", ptr, 7)) {
       ptr += 7;
       while (ptr[0] != '\0' && isspace(ptr[0])) ptr++;
@@ -173,8 +179,11 @@ parse_style(xmlNodePtr node, GraphicStyle *s)
 
       if (!strncmp(ptr, "default", 7))
 	s->dashlength = 1.0;
-      else
-	s->dashlength = g_strtod(ptr, &ptr);
+      else {
+	old_locale = setlocale(LC_NUMERIC, "C");
+	s->dashlength = strtod(ptr, &ptr);
+	setlocale(LC_NUMERIC, old_locale);
+      }
     }
 
     /* skip up to the next attribute */
@@ -200,6 +209,7 @@ parse_path(ShapeInfo *info, const char *path_str, GraphicStyle *s)
   static GArray *points = NULL;
   BezPoint bez;
   gchar *path = (gchar *)path_str;
+  char *old_locale;
 
   if (!points)
     points = g_array_new(FALSE, FALSE, sizeof(BezPoint));
@@ -321,9 +331,11 @@ parse_path(ShapeInfo *info, const char *path_str, GraphicStyle *s)
     switch (last_type) {
     case PATH_MOVE:
       bez.type = BEZ_MOVE_TO;
-      bez.p1.x = g_strtod(path, &path);
+      old_locale = setlocale(LC_NUMERIC, "C");
+      bez.p1.x = strtod(path, &path);
       path_chomp(path);
-      bez.p1.y = g_strtod(path, &path);
+      bez.p1.y = strtod(path, &path);
+      setlocale(LC_NUMERIC, old_locale);
       path_chomp(path);
       if (last_relative) {
 	bez.p1.x += last_point.x;
@@ -333,9 +345,10 @@ parse_path(ShapeInfo *info, const char *path_str, GraphicStyle *s)
       last_control = bez.p1;
       last_open = bez.p1;
 
-      /* if there is some unclosed commands, add them as a GE_PATH */
+      /* if there is some unclosed commands, add them as a GE_PATH,
+       * except if the previous command was also a move */
       if (points->len > 0 &&
-	  g_array_index(points, BezPoint, 0).type != BEZ_MOVE_TO) {
+	  g_array_index(points, BezPoint, points->len).type != BEZ_MOVE_TO) {
 	GraphicElementPath *el = g_malloc(sizeof(GraphicElementPath) +
 					  points->len * sizeof(BezPoint));
 	el->type = GE_PATH;
@@ -349,9 +362,11 @@ parse_path(ShapeInfo *info, const char *path_str, GraphicStyle *s)
       break;
     case PATH_LINE:
       bez.type = BEZ_LINE_TO;
-      bez.p1.x = g_strtod(path, &path);
+      old_locale = setlocale(LC_NUMERIC, "C");
+      bez.p1.x = strtod(path, &path);
       path_chomp(path);
-      bez.p1.y = g_strtod(path, &path);
+      bez.p1.y = strtod(path, &path);
+      setlocale(LC_NUMERIC, old_locale);
       path_chomp(path);
       if (last_relative) {
 	bez.p1.x += last_point.x;
@@ -364,7 +379,9 @@ parse_path(ShapeInfo *info, const char *path_str, GraphicStyle *s)
       break;
     case PATH_HLINE:
       bez.type = BEZ_LINE_TO;
-      bez.p1.x = g_strtod(path, &path);
+      old_locale = setlocale(LC_NUMERIC, "C");
+      bez.p1.x = strtod(path, &path);
+      setlocale(LC_NUMERIC, old_locale);
       path_chomp(path);
       bez.p1.y = last_point.y;
       if (last_relative)
@@ -377,7 +394,9 @@ parse_path(ShapeInfo *info, const char *path_str, GraphicStyle *s)
     case PATH_VLINE:
       bez.type = BEZ_LINE_TO;
       bez.p1.x = last_point.x;
-      bez.p1.y = g_strtod(path, &path);
+      old_locale = setlocale(LC_NUMERIC, "C");
+      bez.p1.y = strtod(path, &path);
+      setlocale(LC_NUMERIC, old_locale);
       path_chomp(path);
       if (last_relative)
 	bez.p1.y += last_point.y;
@@ -388,17 +407,19 @@ parse_path(ShapeInfo *info, const char *path_str, GraphicStyle *s)
       break;
     case PATH_CURVE:
       bez.type = BEZ_CURVE_TO;
-      bez.p1.x = g_strtod(path, &path);
+      old_locale = setlocale(LC_NUMERIC, "C");
+      bez.p1.x = strtod(path, &path);
       path_chomp(path);
-      bez.p1.y = g_strtod(path, &path);
+      bez.p1.y = strtod(path, &path);
       path_chomp(path);
-      bez.p2.x = g_strtod(path, &path);
+      bez.p2.x = strtod(path, &path);
       path_chomp(path);
-      bez.p2.y = g_strtod(path, &path);
+      bez.p2.y = strtod(path, &path);
       path_chomp(path);
-      bez.p3.x = g_strtod(path, &path);
+      bez.p3.x = strtod(path, &path);
       path_chomp(path);
-      bez.p3.y = g_strtod(path, &path);
+      bez.p3.y = strtod(path, &path);
+      setlocale(LC_NUMERIC, old_locale);
       path_chomp(path);
       if (last_relative) {
 	bez.p1.x += last_point.x;
@@ -417,13 +438,15 @@ parse_path(ShapeInfo *info, const char *path_str, GraphicStyle *s)
       bez.type = BEZ_CURVE_TO;
       bez.p1.x = 2 * last_point.x - last_control.x;
       bez.p1.y = 2 * last_point.y - last_control.y;
-      bez.p2.x = g_strtod(path, &path);
+      old_locale = setlocale(LC_NUMERIC, "C");
+      bez.p2.x = strtod(path, &path);
       path_chomp(path);
-      bez.p2.y = g_strtod(path, &path);
+      bez.p2.y = strtod(path, &path);
       path_chomp(path);
-      bez.p3.x = g_strtod(path, &path);
+      bez.p3.x = strtod(path, &path);
       path_chomp(path);
-      bez.p3.y = g_strtod(path, &path);
+      bez.p3.y = strtod(path, &path);
+      setlocale(LC_NUMERIC, old_locale);
       path_chomp(path);
       if (last_relative) {
 	bez.p2.x += last_point.x;
@@ -479,6 +502,7 @@ parse_svg_node(ShapeInfo *info, xmlNodePtr node, xmlNsPtr svg_ns,
 	       GraphicStyle *style)
 {
   CHAR *str;
+  char *old_locale;
 
   /* walk SVG node ... */
   for (node = node->childs; node != NULL; node = node->next) {
@@ -490,28 +514,36 @@ parse_svg_node(ShapeInfo *info, xmlNodePtr node, xmlNsPtr svg_ns,
     s = *style;
     parse_style(node, &s);
     if (!strcmp(node->name, "line")) {
-      GraphicElementLine *line = g_new(GraphicElementLine, 1);
+      GraphicElementLine *line = g_new0(GraphicElementLine, 1);
 
       el = (GraphicElement *)line;
       line->type = GE_LINE;
       str = xmlGetProp(node, "x1");
       if (str) {
-	line->p1.x = g_strtod(str, NULL);
+	old_locale = setlocale(LC_NUMERIC, "C");
+	line->p1.x = strtod(str, NULL);
+	setlocale(LC_NUMERIC, old_locale);
 	free(str);
       }
       str = xmlGetProp(node, "y1");
       if (str) {
-	line->p1.y = g_strtod(str, NULL);
+	old_locale = setlocale(LC_NUMERIC, "C");
+	line->p1.y = strtod(str, NULL);
+	setlocale(LC_NUMERIC, old_locale);
 	free(str);
       }
       str = xmlGetProp(node, "x2");
       if (str) {
-	line->p2.x = g_strtod(str, NULL);
+	old_locale = setlocale(LC_NUMERIC, "C");
+	line->p2.x = strtod(str, NULL);
+	setlocale(LC_NUMERIC, old_locale);
 	free(str);
       }
       str = xmlGetProp(node, "y2");
       if (str) {
-	line->p2.y = g_strtod(str, NULL);
+	old_locale = setlocale(LC_NUMERIC, "C");
+	line->p2.y = strtod(str, NULL);
+	setlocale(LC_NUMERIC, old_locale);
 	free(str);
       }
     } else if (!strcmp(node->name, "polyline")) {
@@ -527,7 +559,9 @@ parse_svg_node(ShapeInfo *info, xmlNodePtr node, xmlNsPtr svg_ns,
 	while (tmp[0] != '\0' && !isdigit(tmp[0]) && tmp[0]!='.'&&tmp[0]!='-')
 	  tmp++;
 	if (tmp[0] == '\0') break;
-	val = g_strtod(tmp, &tmp);
+	old_locale = setlocale(LC_NUMERIC, "C");
+	val = strtod(tmp, &tmp);
+	setlocale(LC_NUMERIC, old_locale);
 	g_array_append_val(arr, val);
       }
       free(str);
@@ -557,7 +591,9 @@ parse_svg_node(ShapeInfo *info, xmlNodePtr node, xmlNsPtr svg_ns,
 	while (tmp[0] != '\0' && !isdigit(tmp[0]) && tmp[0]!='.'&&tmp[0]!='-')
 	  tmp++;
 	if (tmp[0] == '\0') break;
-	val = g_strtod(tmp, &tmp);
+	old_locale = setlocale(LC_NUMERIC, "C");
+	val = strtod(tmp, &tmp);
+	setlocale(LC_NUMERIC, old_locale);
 	g_array_append_val(arr, val);
       }
       free(str);
@@ -575,73 +611,95 @@ parse_svg_node(ShapeInfo *info, xmlNodePtr node, xmlNsPtr svg_ns,
       }
       g_array_free(arr, TRUE);
     } else if (!strcmp(node->name, "rect")) {
-      GraphicElementRect *rect = g_new(GraphicElementRect, 1);
+      GraphicElementRect *rect = g_new0(GraphicElementRect, 1);
 
       el = (GraphicElement *)rect;
       rect->type = GE_RECT;
       str = xmlGetProp(node, "x");
       if (str) {
-	rect->corner1.x = g_strtod(str, NULL);
+	old_locale = setlocale(LC_NUMERIC, "C");
+	rect->corner1.x = strtod(str, NULL);
+	setlocale(LC_NUMERIC, old_locale);
 	free(str);
       }
       str = xmlGetProp(node, "y");
       if (str) {
-	rect->corner1.y = g_strtod(str, NULL);
+	old_locale = setlocale(LC_NUMERIC, "C");
+	rect->corner1.y = strtod(str, NULL);
+	setlocale(LC_NUMERIC, old_locale);
 	free(str);
       }
       str = xmlGetProp(node, "width");
       if (str) {
-	rect->corner2.x = rect->corner1.x + g_strtod(str, NULL);
+	old_locale = setlocale(LC_NUMERIC, "C");
+	rect->corner2.x = rect->corner1.x + strtod(str, NULL);
+	setlocale(LC_NUMERIC, old_locale);
 	free(str);
       }
       str = xmlGetProp(node, "height");
       if (str) {
-	rect->corner2.y = rect->corner1.y + g_strtod(str, NULL);
+	old_locale = setlocale(LC_NUMERIC, "C");
+	rect->corner2.y = rect->corner1.y + strtod(str, NULL);
+	  setlocale(LC_NUMERIC, old_locale);
 	free(str);
       }
     } else if (!strcmp(node->name, "circle")) {
-      GraphicElementEllipse *ellipse = g_new(GraphicElementEllipse, 1);
+      GraphicElementEllipse *ellipse = g_new0(GraphicElementEllipse, 1);
 
       el = (GraphicElement *)ellipse;
       ellipse->type = GE_ELLIPSE;
       str = xmlGetProp(node, "cx");
       if (str) {
-	ellipse->center.x = g_strtod(str, NULL);
+	old_locale = setlocale(LC_NUMERIC, "C");
+	ellipse->center.x = strtod(str, NULL);
+	setlocale(LC_NUMERIC, old_locale);
 	free(str);
       }
       str = xmlGetProp(node, "cy");
       if (str) {
-	ellipse->center.y = g_strtod(str, NULL);
+	old_locale = setlocale(LC_NUMERIC, "C");
+	ellipse->center.y = strtod(str, NULL);
+	setlocale(LC_NUMERIC, old_locale);
 	free(str);
       }
       str = xmlGetProp(node, "r");
       if (str) {
-	ellipse->width = ellipse->height = 2 * g_strtod(str, NULL);
+	old_locale = setlocale(LC_NUMERIC, "C");
+	ellipse->width = ellipse->height = 2 * strtod(str, NULL);
+	  setlocale(LC_NUMERIC, old_locale);
 	free(str);
       }
     } else if (!strcmp(node->name, "ellipse")) {
-      GraphicElementEllipse *ellipse = g_new(GraphicElementEllipse, 1);
+      GraphicElementEllipse *ellipse = g_new0(GraphicElementEllipse, 1);
 
       el = (GraphicElement *)ellipse;
       ellipse->type = GE_ELLIPSE;
       str = xmlGetProp(node, "cx");
       if (str) {
-	ellipse->center.x = g_strtod(str, NULL);
+	old_locale = setlocale(LC_NUMERIC, "C");
+	ellipse->center.x = strtod(str, NULL);
+	setlocale(LC_NUMERIC, old_locale);
 	free(str);
       }
       str = xmlGetProp(node, "cy");
       if (str) {
-	ellipse->center.y = g_strtod(str, NULL);
+	old_locale = setlocale(LC_NUMERIC, "C");
+	ellipse->center.y = strtod(str, NULL);
+	setlocale(LC_NUMERIC, old_locale);
 	free(str);
       }
       str = xmlGetProp(node, "rx");
       if (str) {
-	ellipse->width = 2 * g_strtod(str, NULL);
+	old_locale = setlocale(LC_NUMERIC, "C");
+	ellipse->width = 2 * strtod(str, NULL);
+	  setlocale(LC_NUMERIC, old_locale);
 	free(str);
       }
       str = xmlGetProp(node, "ry");
       if (str) {
-	ellipse->height = 2 * g_strtod(str, NULL);
+	old_locale = setlocale(LC_NUMERIC, "C");
+	ellipse->height = 2 * strtod(str, NULL);
+	setlocale(LC_NUMERIC, old_locale);
 	free(str);
       }
     } else if (!strcmp(node->name, "path")) {
@@ -733,6 +791,7 @@ load_shape_info(const gchar *filename)
   xmlNodePtr node,root;
   ShapeInfo *info;
   char *tmp;
+  char *old_locale;
 #if 0
   int descr_score = -1;
 #endif
@@ -752,14 +811,10 @@ load_shape_info(const gchar *filename)
     g_warning("could not find shape namespace");
     return NULL;
   }
-  if (!(svg_ns = xmlSearchNsByHref(doc, root,
-		"http://www.w3.org/TR/2000/03/WD-SVG-20000303/DTD/svg-20000303-stylable.dtd"))) {
-    if (!(svg_ns = xmlSearchNsByHref(doc, root,
-		"http://www.w3.org/Graphics/SVG/svg-19990730.dtd"))) {
-      xmlFreeDoc(doc);
-      g_warning("could not find svg namespace");
-      return NULL;
-    }
+  if (!(svg_ns = xmlSearchNsByHref(doc, root, "http://www.w3.org/2000/svg"))) {
+    xmlFreeDoc(doc);
+    g_warning("could not find svg namespace");
+    return NULL;
   }
   if (root->ns != shape_ns || strcmp(root->name, "shape")) {
     g_warning("root element was %s -- expecting shape", root->name);
@@ -817,12 +872,16 @@ load_shape_info(const gchar *filename)
 
 	  str = xmlGetProp(pt_node, "x");
 	  if (str) {
-	    pt.x = g_strtod(str, NULL);
+	    old_locale = setlocale(LC_NUMERIC, "C");
+	    pt.x = strtod(str, NULL);
+	    setlocale(LC_NUMERIC, old_locale);
 	    free(str);
 	  }
 	  str = xmlGetProp(pt_node, "y");
 	  if (str) {
-	    pt.y = g_strtod(str, NULL);
+	    old_locale = setlocale(LC_NUMERIC, "C");
+	    pt.y = strtod(str, NULL);
+	    setlocale(LC_NUMERIC, old_locale);
 	    free(str);
 	  }
 	  g_array_append_val(arr, pt);
@@ -836,22 +895,45 @@ load_shape_info(const gchar *filename)
       
       str = xmlGetProp(node, "x1");
       if (str) {
-	info->text_bounds.left = g_strtod(str, NULL);
+	old_locale = setlocale(LC_NUMERIC, "C");
+	info->text_bounds.left = strtod(str, NULL);
+	setlocale(LC_NUMERIC, old_locale);
 	free(str);
       }
       str = xmlGetProp(node, "y1");
       if (str) {
-	info->text_bounds.top = g_strtod(str, NULL);
+	old_locale = setlocale(LC_NUMERIC, "C");
+	info->text_bounds.top = strtod(str, NULL);
+	setlocale(LC_NUMERIC, old_locale);
 	free(str);
       }
       str = xmlGetProp(node, "x2");
       if (str) {
-	info->text_bounds.right = g_strtod(str, NULL);
+	old_locale = setlocale(LC_NUMERIC, "C");
+	info->text_bounds.right = strtod(str, NULL);
+	setlocale(LC_NUMERIC, old_locale);
 	free(str);
       }
       str = xmlGetProp(node, "y2");
       if (str) {
-	info->text_bounds.bottom = g_strtod(str, NULL);
+	old_locale = setlocale(LC_NUMERIC, "C");
+	info->text_bounds.bottom = strtod(str, NULL);
+	setlocale(LC_NUMERIC, old_locale);
+	free(str);
+      }
+      info->resize_with_text = TRUE;
+      str = xmlGetProp(node, "resize");
+      if (str) {
+	info->resize_with_text = strcmp(str, "no");
+	free(str);
+      }
+      info->text_align = ALIGN_CENTER;
+      str = xmlGetProp(node, "align");
+      if (str) {
+	if (!strcmp(str, "left")) 
+	  info->text_align = ALIGN_LEFT;
+	else if (!strcmp(str, "right"))
+	  info->text_align = ALIGN_RIGHT;
 	free(str);
       }
       info->has_text = TRUE;
@@ -870,12 +952,16 @@ load_shape_info(const gchar *filename)
 	  info->aspect_max = G_MAXFLOAT;
 	  str = xmlGetProp(node, "min");
 	  if (str) {
-	    info->aspect_min = g_strtod(str, NULL);
+	    old_locale = setlocale(LC_NUMERIC, "C");
+	    info->aspect_min = strtod(str, NULL);
+	    setlocale(LC_NUMERIC, old_locale);
 	    free(str);
 	  }
 	  str = xmlGetProp(node, "max");
 	  if (str) {
-	    info->aspect_max = g_strtod(str, NULL);
+	    old_locale = setlocale(LC_NUMERIC, "C");
+	    info->aspect_max = strtod(str, NULL);
+	    setlocale(LC_NUMERIC, old_locale);
 	    free(str);
 	  }
 	  if (info->aspect_max < info->aspect_min) {
