@@ -18,11 +18,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
 #include <gtk/gtk.h>
 #include <math.h>
 
-#include "config.h"
 #include "intl.h"
 #include "object.h"
 #include "element.h"
@@ -204,7 +208,8 @@ box_get_props(Box *box, Property *props, guint nprops)
 {
   guint i;
 
-  if (object_get_props_from_offsets((Object *)box, box_offsets, props, nprops))
+  if (object_get_props_from_offsets(&box->element.object, 
+                                    box_offsets, props, nprops))
     return;
   /* these props can't be handled as easily */
   if (quarks[0].q == 0)
@@ -233,8 +238,8 @@ box_get_props(Box *box, Property *props, guint nprops)
 static void
 box_set_props(Box *box, Property *props, guint nprops)
 {
-  if (!object_set_props_from_offsets((Object *)box, box_offsets,
-				     props, nprops)) {
+  if (!object_set_props_from_offsets(&box->element.object, 
+                                     box_offsets, props, nprops)) {
     guint i;
 
     if (quarks[0].q == 0)
@@ -320,7 +325,7 @@ box_get_defaults()
     label = gtk_label_new(_("Corner rounding:"));
     gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
     gtk_widget_show (label);
-    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.0, 10.0, 0.1, 0.0, 0.0);
+    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.0, 10.0, 0.1, 1.0, 1.0);
     corner_radius = gtk_spin_button_new(adj, 1.0, 2);
     gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(corner_radius), TRUE);
     gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(corner_radius), TRUE);
@@ -334,7 +339,7 @@ box_get_defaults()
     label = gtk_label_new(_("Text padding:"));
     gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
     gtk_widget_show (label);
-    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.0, 10.0, 0.1, 0.0, 0.0);
+    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.0, 10.0, 0.1, 1.0, 1.0);
     padding = gtk_spin_button_new(adj, 1.0, 2);
     gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(padding), TRUE);
     gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(padding), TRUE);
@@ -359,7 +364,7 @@ box_get_defaults()
     label = gtk_label_new(_("Font size:"));
     gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
     gtk_widget_show (label);
-    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.1, 10.0, 0.1, 0.0, 0.0);
+    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.1, 10.0, 0.1, 1.0, 1.0);
     font_size = gtk_spin_button_new(adj, 1.0, 2);
     gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(font_size), TRUE);
     gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(font_size), TRUE);
@@ -407,7 +412,7 @@ box_select(Box *box, Point *clicked_point,
   real radius;
 
   text_set_cursor(box->text, clicked_point, interactive_renderer);
-  text_grab_focus(box->text, (Object *)box);
+  text_grab_focus(box->text, &box->element.object);
 
   element_update_handles(&box->element);
 
@@ -591,7 +596,8 @@ static void
 box_update_data(Box *box, AnchorShape horiz, AnchorShape vert)
 {
   Element *elem = &box->element;
-  Object *obj = (Object *) box;
+  ElementBBExtras *extra = &elem->extra_spacing;
+  Object *obj = &elem->object;
   Point center, bottom_right;
   Point p;
   real radius;
@@ -674,12 +680,8 @@ box_update_data(Box *box, AnchorShape horiz, AnchorShape vert)
   box->connections[15].pos.x = elem->corner.x + elem->width - radius;
   box->connections[15].pos.y = elem->corner.y + elem->height - radius;
 
+  extra->border_trans = box->border_width / 2.0;
   element_update_boundingbox(elem);
-  /* fix boundingbox for line_width: */
-  obj->bounding_box.top -= box->border_width/2;
-  obj->bounding_box.left -= box->border_width/2;
-  obj->bounding_box.bottom += box->border_width/2;
-  obj->bounding_box.right += box->border_width/2;
   
   obj->position = elem->corner;
   
@@ -714,9 +716,9 @@ box_create(Point *startpoint,
 
   init_default_values();
 
-  box = g_malloc(sizeof(Box));
+  box = g_malloc0(sizeof(Box));
   elem = &box->element;
-  obj = (Object *) box;
+  obj = &elem->object;
   
   obj->type = &fc_box_type;
 
@@ -754,7 +756,7 @@ box_create(Point *startpoint,
 
   *handle1 = NULL;
   *handle2 = obj->handles[7];  
-  return (Object *)box;
+  return &box->element.object;
 }
 
 static void
@@ -775,9 +777,9 @@ box_copy(Box *box)
   
   elem = &box->element;
   
-  newbox = g_malloc(sizeof(Box));
+  newbox = g_malloc0(sizeof(Box));
   newelem = &newbox->element;
-  newobj = (Object *) newbox;
+  newobj = &newelem->object;
 
   element_copy(elem, newelem);
 
@@ -800,7 +802,7 @@ box_copy(Box *box)
     newbox->connections[i].last_pos = box->connections[i].last_pos;
   }
 
-  return (Object *)newbox;
+  return &newbox->element.object;
 }
 
 static void
@@ -850,7 +852,7 @@ box_load(ObjectNode obj_node, int version, const char *filename)
 
   box = g_new(Box, 1);
   elem = &box->element;
-  obj = (Object *) box;
+  obj = &elem->object;
   
   obj->type = &fc_box_type;
   obj->ops = &box_ops;
@@ -912,5 +914,5 @@ box_load(ObjectNode obj_node, int version, const char *filename)
 
   box_update_data(box, ANCHOR_MIDDLE, ANCHOR_MIDDLE);
 
-  return (Object *)box;
+  return &box->element.object;
 }
