@@ -18,11 +18,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
 #include <gtk/gtk.h>
 #include <math.h>
 
-#include "config.h"
 #include "intl.h"
 #include "object.h"
 #include "element.h"
@@ -199,8 +203,8 @@ ellipse_get_props(Ellipse *ellipse, Property *props, guint nprops)
 {
   guint i;
 
-  if (object_get_props_from_offsets((Object *)ellipse, ellipse_offsets,
-                                    props, nprops))
+  if (object_get_props_from_offsets(&ellipse->element.object,
+                                    ellipse_offsets, props, nprops))
     return;
   /* these props can't be handled as easily */
   if (quarks[0].q == 0)
@@ -229,8 +233,8 @@ ellipse_get_props(Ellipse *ellipse, Property *props, guint nprops)
 static void
 ellipse_set_props(Ellipse *ellipse, Property *props, guint nprops)
 {
-  if (!object_set_props_from_offsets((Object *)ellipse, ellipse_offsets,
-                                     props, nprops)) {
+  if (!object_set_props_from_offsets(&ellipse->element.object, 
+                                     ellipse_offsets, props, nprops)) {
     guint i;
 
     if (quarks[0].q == 0)
@@ -314,7 +318,7 @@ ellipse_get_defaults()
     label = gtk_label_new(_("Text padding:"));
     gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
     gtk_widget_show (label);
-    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.0, 10.0, 0.1, 0.0, 0.0);
+    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.0, 10.0, 0.1, 1.0, 1.0);
     padding = gtk_spin_button_new(adj, 1.0, 2);
     gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(padding), TRUE);
     gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(padding), TRUE);
@@ -339,7 +343,7 @@ ellipse_get_defaults()
     label = gtk_label_new(_("Font size:"));
     gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
     gtk_widget_show (label);
-    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.1, 10.0, 0.1, 0.0, 0.0);
+    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.1, 10.0, 0.1, 1.0, 1.0);
     font_size = gtk_spin_button_new(adj, 1.0, 2);
     gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(font_size), TRUE);
     gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(font_size), TRUE);
@@ -412,7 +416,7 @@ ellipse_select(Ellipse *ellipse, Point *clicked_point,
 	   Renderer *interactive_renderer)
 {
   text_set_cursor(ellipse->text, clicked_point, interactive_renderer);
-  text_grab_focus(ellipse->text, (Object *)ellipse);
+  text_grab_focus(ellipse->text, &ellipse->element.object);
 
   element_update_handles(&ellipse->element);
 }
@@ -499,7 +503,8 @@ static void
 ellipse_update_data(Ellipse *ellipse, AnchorShape horiz, AnchorShape vert)
 {
   Element *elem = &ellipse->element;
-  Object *obj = (Object *) ellipse;
+  ElementBBExtras *extra = &elem->extra_spacing;
+  Object *obj = &elem->object;
   Point center, bottom_right;
   Point p, c;
   real dw, dh;
@@ -572,13 +577,9 @@ ellipse_update_data(Ellipse *ellipse, AnchorShape horiz, AnchorShape vert)
     ellipse->connections[i].pos.y = c.y - dh * sin(theta);
   }
 
+  extra->border_trans = ellipse->border_width / 2.0;
   element_update_boundingbox(elem);
-  /* fix boundingbox for line_width: */
-  obj->bounding_box.top -= ellipse->border_width/2;
-  obj->bounding_box.left -= ellipse->border_width/2;
-  obj->bounding_box.bottom += ellipse->border_width/2;
-  obj->bounding_box.right += ellipse->border_width/2;
-  
+
   obj->position = elem->corner;
   
   element_update_handles(elem);
@@ -600,9 +601,9 @@ ellipse_create(Point *startpoint,
 
   init_default_values();
 
-  ellipse = g_malloc(sizeof(Ellipse));
+  ellipse = g_malloc0(sizeof(Ellipse));
   elem = &ellipse->element;
-  obj = (Object *) ellipse;
+  obj = &elem->object;
   
   obj->type = &fc_ellipse_type;
 
@@ -639,7 +640,7 @@ ellipse_create(Point *startpoint,
 
   *handle1 = NULL;
   *handle2 = obj->handles[7];  
-  return (Object *)ellipse;
+  return &ellipse->element.object;
 }
 
 static void
@@ -660,9 +661,9 @@ ellipse_copy(Ellipse *ellipse)
   
   elem = &ellipse->element;
   
-  newellipse = g_malloc(sizeof(Ellipse));
+  newellipse = g_malloc0(sizeof(Ellipse));
   newelem = &newellipse->element;
-  newobj = (Object *) newellipse;
+  newobj = &newelem->object;
 
   element_copy(elem, newelem);
 
@@ -684,7 +685,7 @@ ellipse_copy(Ellipse *ellipse)
     newellipse->connections[i].last_pos = ellipse->connections[i].last_pos;
   }
 
-  return (Object *)newellipse;
+  return &newellipse->element.object;
 }
 
 static void
@@ -729,9 +730,9 @@ ellipse_load(ObjectNode obj_node, int version, const char *filename)
   int i;
   AttributeNode attr;
 
-  ellipse = g_malloc(sizeof(Ellipse));
+  ellipse = g_malloc0(sizeof(Ellipse));
   elem = &ellipse->element;
-  obj = (Object *) ellipse;
+  obj = &elem->object;
   
   obj->type = &fc_ellipse_type;
   obj->ops = &ellipse_ops;
@@ -788,5 +789,5 @@ ellipse_load(ObjectNode obj_node, int version, const char *filename)
 
   ellipse_update_data(ellipse, ANCHOR_MIDDLE, ANCHOR_MIDDLE);
 
-  return (Object *)ellipse;
+  return &ellipse->element.object;
 }
