@@ -18,11 +18,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
 #include <gtk/gtk.h>
 #include <math.h>
 
-#include "config.h"
 #include "intl.h"
 #include "object.h"
 #include "element.h"
@@ -206,8 +210,8 @@ pgram_get_props(Pgram *pgram, Property *props, guint nprops)
 {
   guint i;
 
-  if (object_get_props_from_offsets((Object *)pgram, pgram_offsets,
-                                    props, nprops))
+  if (object_get_props_from_offsets(&pgram->element.object, 
+                                    pgram_offsets, props, nprops))
     return;
   /* these props can't be handled as easily */
   if (quarks[0].q == 0)
@@ -236,8 +240,8 @@ pgram_get_props(Pgram *pgram, Property *props, guint nprops)
 static void
 pgram_set_props(Pgram *pgram, Property *props, guint nprops)
 {
-  if (!object_set_props_from_offsets((Object *)pgram, pgram_offsets,
-                                     props, nprops)) {
+  if (!object_set_props_from_offsets(&pgram->element.object, 
+                                     pgram_offsets, props, nprops)) {
     guint i;
 
     if (quarks[0].q == 0)
@@ -325,7 +329,7 @@ pgram_get_defaults()
     label = gtk_label_new(_("Shear angle:"));
     gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
     gtk_widget_show (label);
-    adj = (GtkAdjustment *) gtk_adjustment_new(60.0, 45.0, 135.0, 1.0,0.0,0.0);
+    adj = (GtkAdjustment *) gtk_adjustment_new(60.0, 45.0, 135.0, 1.0,10.0,10.0);
     shear_angle = gtk_spin_button_new(adj, 1.0, 2);
     gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(shear_angle), TRUE);
     gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(shear_angle), TRUE);
@@ -339,7 +343,7 @@ pgram_get_defaults()
     label = gtk_label_new(_("Text padding:"));
     gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
     gtk_widget_show (label);
-    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.0, 10.0, 0.1, 0.0, 0.0);
+    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.0, 10.0, 0.1, 1.0, 1.0);
     padding = gtk_spin_button_new(adj, 1.0, 2);
     gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(padding), TRUE);
     gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(padding), TRUE);
@@ -364,7 +368,7 @@ pgram_get_defaults()
     label = gtk_label_new(_("Font size:"));
     gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
     gtk_widget_show (label);
-    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.1, 10.0, 0.1, 0.0, 0.0);
+    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.1, 10.0, 0.1, 1.0, 1.0);
     font_size = gtk_spin_button_new(adj, 1.0, 2);
     gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(font_size), TRUE);
     gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(font_size), TRUE);
@@ -438,7 +442,7 @@ pgram_select(Pgram *pgram, Point *clicked_point,
 	   Renderer *interactive_renderer)
 {
   text_set_cursor(pgram->text, clicked_point, interactive_renderer);
-  text_grab_focus(pgram->text, (Object *)pgram);
+  text_grab_focus(pgram->text, &pgram->element.object);
 
   element_update_handles(&pgram->element);
 }
@@ -538,7 +542,8 @@ static void
 pgram_update_data(Pgram *pgram, AnchorShape horiz, AnchorShape vert)
 {
   Element *elem = &pgram->element;
-  Object *obj = (Object *) pgram;
+  ElementBBExtras *extra = &elem->extra_spacing;
+  Object *obj = &elem->object;
   Point center, bottom_right;
   Point p;
   real offs;
@@ -653,12 +658,8 @@ pgram_update_data(Pgram *pgram, AnchorShape horiz, AnchorShape vert)
     pgram->connections[14].pos.x -= offs;
   }
 
+  extra->border_trans = pgram->border_width/2.0;
   element_update_boundingbox(elem);
-  /* fix boundingbox for line_width: */
-  obj->bounding_box.top -= pgram->border_width/2;
-  obj->bounding_box.left -= pgram->border_width/2;
-  obj->bounding_box.bottom += pgram->border_width/2;
-  obj->bounding_box.right += pgram->border_width/2;
   
   obj->position = elem->corner;
   
@@ -681,9 +682,9 @@ pgram_create(Point *startpoint,
 
   init_default_values();
 
-  pgram = g_malloc(sizeof(Pgram));
+  pgram = g_malloc0(sizeof(Pgram));
   elem = &pgram->element;
-  obj = (Object *) pgram;
+  obj = &elem->object;
   
   obj->type = &pgram_type;
 
@@ -722,7 +723,7 @@ pgram_create(Point *startpoint,
 
   *handle1 = NULL;
   *handle2 = obj->handles[7];  
-  return (Object *)pgram;
+  return &pgram->element.object;
 }
 
 static void
@@ -743,9 +744,9 @@ pgram_copy(Pgram *pgram)
   
   elem = &pgram->element;
   
-  newpgram = g_malloc(sizeof(Pgram));
+  newpgram = g_malloc0(sizeof(Pgram));
   newelem = &newpgram->element;
-  newobj = (Object *) newpgram;
+  newobj = &newelem->object;
 
   element_copy(elem, newelem);
 
@@ -769,7 +770,7 @@ pgram_copy(Pgram *pgram)
     newpgram->connections[i].last_pos = pgram->connections[i].last_pos;
   }
 
-  return (Object *)newpgram;
+  return &newpgram->element.object;
 }
 
 static void
@@ -817,9 +818,9 @@ pgram_load(ObjectNode obj_node, int version, const char *filename)
   int i;
   AttributeNode attr;
 
-  pgram = g_malloc(sizeof(Pgram));
+  pgram = g_malloc0(sizeof(Pgram));
   elem = &pgram->element;
-  obj = (Object *) pgram;
+  obj = &elem->object;
   
   obj->type = &pgram_type;
   obj->ops = &pgram_ops;
@@ -882,5 +883,5 @@ pgram_load(ObjectNode obj_node, int version, const char *filename)
 
   pgram_update_data(pgram, ANCHOR_MIDDLE, ANCHOR_MIDDLE);
 
-  return (Object *)pgram;
+  return &pgram->element.object;
 }
