@@ -15,11 +15,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
 #include <gtk/gtk.h>
 #include <math.h>
 
-#include "config.h"
 #include "intl.h"
 #include "object.h"
 #include "connection.h"
@@ -246,7 +250,7 @@ bus_move(Bus *bus, Point *to)
 {
   Point delta;
   Point *endpoints = &bus->connection.endpoints[0]; 
-  Object *obj = (Object *) bus;
+  Object *obj = &bus->connection.object;
   int i;
   
   delta = *to;
@@ -301,19 +305,21 @@ bus_create(Point *startpoint,
 {
   Bus *bus;
   Connection *conn;
+  ConnectionBBExtras *extra;
   Object *obj;
   Point defaultlen = { 5.0, 0.0 };
   int i;
 
-  bus = g_malloc(sizeof(Bus));
+  bus = g_malloc0(sizeof(Bus));
 
   conn = &bus->connection;
   conn->endpoints[0] = *startpoint;
   conn->endpoints[1] = *startpoint;
   point_add(&conn->endpoints[1], &defaultlen);
  
-  obj = (Object *) bus;
-  
+  obj = &conn->object;
+  extra = &conn->extra_spacing;
+
   obj->type = &bus_type;
   obj->ops = &bus_ops;
 
@@ -324,7 +330,7 @@ bus_create(Point *startpoint,
   bus->handles = g_malloc(sizeof(Handle *)*bus->num_handles);
   bus->parallel_points = g_malloc(sizeof(Point)*bus->num_handles);
   for (i=0;i<bus->num_handles;i++) {
-    bus->handles[i] = g_new(Handle,1);
+    bus->handles[i] = g_new0(Handle,1);
     bus->handles[i]->id = HANDLE_BUS;
     bus->handles[i]->type = HANDLE_MINOR_CONTROL;
     bus->handles[i]->connect_type = HANDLE_CONNECTABLE_NOBREAK;
@@ -336,12 +342,16 @@ bus_create(Point *startpoint,
   }
 
   bus->properties_dialog = NULL;
-  
+
+  extra->start_trans = 
+    extra->end_trans = 
+    extra->start_long =
+    extra->end_long = LINE_WIDTH/2.0;  
   bus_update_data(bus);
 
   *handle1 = obj->handles[0];
   *handle2 = obj->handles[1];
-  return (Object *)bus;
+  return &bus->connection.object;
 }
 
 static void
@@ -369,9 +379,9 @@ bus_copy(Bus *bus)
   
   conn = &bus->connection;
   
-  newbus = g_malloc(sizeof(Bus));
+  newbus = g_malloc0(sizeof(Bus));
   newconn = &newbus->connection;
-  newobj = (Object *) newbus;
+  newobj = &newconn->object;
   
   connection_copy(conn, newconn);
 
@@ -381,7 +391,7 @@ bus_copy(Bus *bus)
   newbus->parallel_points = g_malloc(sizeof(Point)*newbus->num_handles);
   
   for (i=0;i<newbus->num_handles;i++) {
-    newbus->handles[i] = g_new(Handle,1);
+    newbus->handles[i] = g_new0(Handle,1);
     *newbus->handles[i] = *bus->handles[i];
     newbus->handles[i]->connected_to = NULL;
     newobj->handles[2+i] = newbus->handles[i];
@@ -393,7 +403,7 @@ bus_copy(Bus *bus)
 
   newbus->properties_dialog = NULL;
 
-  return (Object *)newbus;
+  return &newbus->connection.object;
 }
 
 
@@ -401,7 +411,7 @@ static void
 bus_update_data(Bus *bus)
 {
   Connection *conn = &bus->connection;
-  Object *obj = (Object *) bus;
+  Object *obj = &conn->object;
   int i;
   Point u, v, vhat;
   Point *endpoints;
@@ -447,11 +457,6 @@ bus_update_data(Bus *bus)
   for (i=0;i<bus->num_handles;i++) {
     rectangle_add_point(&obj->bounding_box, &bus->handles[i]->pos);
   }
-  /* fix boundingbox for bus_width: */
-  obj->bounding_box.top -= LINE_WIDTH/2;
-  obj->bounding_box.left -= LINE_WIDTH/2;
-  obj->bounding_box.bottom += LINE_WIDTH/2;
-  obj->bounding_box.right += LINE_WIDTH/2;
 
   connection_update_handles(conn);
 }
@@ -477,7 +482,7 @@ bus_add_handle(Bus *bus, Point *p, Handle *handle)
   bus->handles[i]->connect_type = HANDLE_CONNECTABLE_NOBREAK;
   bus->handles[i]->connected_to = NULL;
   bus->handles[i]->pos = *p;
-  object_add_handle((Object *) bus, bus->handles[i]);
+  object_add_handle(&bus->connection.object, bus->handles[i]);
 }
 
 static void
@@ -487,7 +492,7 @@ bus_remove_handle(Bus *bus, Handle *handle)
   
   for (i=0;i<bus->num_handles;i++) {
     if (bus->handles[i] == handle) {
-      object_remove_handle((Object *) bus, handle);
+      object_remove_handle(&bus->connection.object, handle);
 
       for (j=i;j<bus->num_handles-1;j++) {
 	bus->handles[j] = bus->handles[j+1];
@@ -511,7 +516,7 @@ bus_add_handle_callback (Object *obj, Point *clicked, gpointer data)
   Bus *bus = (Bus *) obj;
   Handle *handle;
 
-  handle = g_new(Handle,1);
+  handle = g_new0(Handle,1);
   bus_add_handle(bus, clicked, handle);
   bus_update_data(bus);
 
@@ -606,15 +611,17 @@ bus_load(ObjectNode obj_node, int version, const char *filename)
 {
   Bus *bus;
   Connection *conn;
+  ConnectionBBExtras *extra;
   Object *obj;
   AttributeNode attr;
   DataNode data;
   int i;
 
-  bus = g_malloc(sizeof(Bus));
+  bus = g_malloc0(sizeof(Bus));
 
   conn = &bus->connection;
-  obj = (Object *) bus;
+  obj = &conn->object;
+  extra = &conn->extra_spacing;
 
   obj->type = &bus_type;
   obj->ops = &bus_ops;
@@ -635,7 +642,7 @@ bus_load(ObjectNode obj_node, int version, const char *filename)
   bus->handles = g_malloc(sizeof(Handle *)*bus->num_handles);
   bus->parallel_points = g_malloc(sizeof(Point)*bus->num_handles);
   for (i=0;i<bus->num_handles;i++) {
-    bus->handles[i] = g_new(Handle,1);
+    bus->handles[i] = g_new0(Handle,1);
     bus->handles[i]->id = HANDLE_BUS;
     bus->handles[i]->type = HANDLE_MINOR_CONTROL;
     bus->handles[i]->connect_type = HANDLE_CONNECTABLE_NOBREAK;
@@ -646,9 +653,13 @@ bus_load(ObjectNode obj_node, int version, const char *filename)
     data = data_next(data);
   }
 
+  extra->start_trans = 
+    extra->end_trans = 
+    extra->start_long =
+    extra->end_long = LINE_WIDTH/2.0;  
   bus_update_data(bus);
 
-  return (Object *)bus;
+  return &bus->connection.object;
 }
 
 static void
