@@ -15,11 +15,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
 #include <gtk/gtk.h>
 #include <math.h>
 
-#include "config.h"
 #include "intl.h"
 #include "object.h"
 #include "element.h"
@@ -97,8 +101,8 @@ static void box_set_props(Box *box, Property *props, guint nprops);
 
 static void box_save(Box *box, ObjectNode obj_node, const char *filename);
 static Object *box_load(ObjectNode obj_node, int version, const char *filename);
-static GtkWidget *box_get_defaults();
-static void box_apply_defaults();
+static GtkWidget *box_get_defaults(void);
+static void box_apply_defaults(void);
 
 static ObjectTypeOps box_type_ops =
 {
@@ -173,13 +177,15 @@ static PropOffset box_offsets[] = {
 static void
 box_get_props(Box *box, Property *props, guint nprops)
 {
-  object_get_props_from_offsets((Object *)box, box_offsets, props, nprops);
+  object_get_props_from_offsets(&box->element.object, 
+                                box_offsets, props, nprops);
 }
 
 static void
 box_set_props(Box *box, Property *props, guint nprops)
 {
-  object_set_props_from_offsets((Object *)box, box_offsets, props, nprops);
+  object_set_props_from_offsets(&box->element.object, 
+                                box_offsets, props, nprops);
   box_update_data(box);
 }
 
@@ -234,7 +240,7 @@ box_get_defaults()
     label = gtk_label_new(_("Corner rounding:"));
     gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
     gtk_widget_show (label);
-    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.0, 10.0, 0.1, 0.0, 0.0);
+    adj = (GtkAdjustment *) gtk_adjustment_new(0.1, 0.0, 10.0, 0.1, 1.0, 1.0);
     corner_radius = gtk_spin_button_new(adj, 1.0, 2);
     gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(corner_radius), TRUE);
     gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(corner_radius), TRUE);
@@ -434,7 +440,8 @@ static void
 box_update_data(Box *box)
 {
   Element *elem = &box->element;
-  Object *obj = (Object *) box;
+  ElementBBExtras *extra = &elem->extra_spacing;
+  Object *obj = &elem->object;
   real radius;
   
   radius = box->corner_radius;
@@ -460,12 +467,8 @@ box_update_data(Box *box)
   box->connections[7].pos.x = elem->corner.x + elem->width - radius;
   box->connections[7].pos.y = elem->corner.y + elem->height - radius;
 
+  extra->border_trans = box->border_width / 2.0;
   element_update_boundingbox(elem);
-  /* fix boundingbox for line_width: */
-  obj->bounding_box.top -= box->border_width/2;
-  obj->bounding_box.left -= box->border_width/2;
-  obj->bounding_box.bottom += box->border_width/2;
-  obj->bounding_box.right += box->border_width/2;
   
   obj->position = elem->corner;
   
@@ -497,9 +500,9 @@ box_create(Point *startpoint,
 
   init_default_values();
 
-  box = g_malloc(sizeof(Box));
+  box = g_malloc0(sizeof(Box));
   elem = &box->element;
-  obj = (Object *) box;
+  obj = &elem->object;
   
   obj->type = &box_type;
 
@@ -528,7 +531,7 @@ box_create(Point *startpoint,
 
   *handle1 = NULL;
   *handle2 = obj->handles[7];  
-  return (Object *)box;
+  return &box->element.object;
 }
 
 static void
@@ -547,9 +550,9 @@ box_copy(Box *box)
   
   elem = &box->element;
   
-  newbox = g_malloc(sizeof(Box));
+  newbox = g_malloc0(sizeof(Box));
   newelem = &newbox->element;
-  newobj = (Object *) newbox;
+  newobj = &newelem->object;
 
   element_copy(elem, newelem);
 
@@ -569,7 +572,7 @@ box_copy(Box *box)
     newbox->connections[i].last_pos = box->connections[i].last_pos;
   }
 
-  return (Object *)newbox;
+  return &newbox->element.object;
 }
 
 static void
@@ -614,9 +617,9 @@ box_load(ObjectNode obj_node, int version, const char *filename)
   int i;
   AttributeNode attr;
 
-  box = g_malloc(sizeof(Box));
+  box = g_malloc0(sizeof(Box));
   elem = &box->element;
-  obj = (Object *) box;
+  obj = &elem->object;
   
   obj->type = &box_type;
   obj->ops = &box_ops;
@@ -668,5 +671,5 @@ box_load(ObjectNode obj_node, int version, const char *filename)
 
   box_update_data(box);
 
-  return (Object *)box;
+  return &box->element.object;
 }
