@@ -15,13 +15,16 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+#include <config.h>
+
 #include <assert.h>
 #include <string.h>
 #include <gtk/gtk.h>
 #include <math.h>
+#ifdef HAVE_UNIST_H
 #include <unistd.h>
+#endif
 
-#include "config.h"
 #include "intl.h"
 #include "message.h"
 #include "object.h"
@@ -196,7 +199,8 @@ static PropOffset image_offsets[] = {
 static void
 image_get_props(Image *image, Property *props, guint nprops)
 {
-  object_get_props_from_offsets((Object *)image, image_offsets, props, nprops);
+  object_get_props_from_offsets(&image->element.object, 
+                                image_offsets, props, nprops);
 }
 
 static void
@@ -204,12 +208,18 @@ image_set_props(Image *image, Property *props, guint nprops)
 {
   char *old_file = image->file ? g_strdup(image->file) : NULL;
 
-  object_set_props_from_offsets((Object *)image, image_offsets, props, nprops);
+  object_set_props_from_offsets(&image->element.object, 
+                                image_offsets, props, nprops);
 
   /* handle changing the image. */
   if (strcmp(image->file, old_file) != 0) {
-    Element *elem = (Element *)image;
-    image->image = dia_image_load(image->file);
+    Element *elem = &image->element;
+    DiaImage img = dia_image_load(image->file);
+
+    if (img)
+      image->image = img;
+    else
+      image->image = dia_image_get_broken();
     elem->height = (elem->width*(float)dia_image_height(image->image))/
       (float)dia_image_width(image->image);
   }
@@ -301,7 +311,7 @@ image_get_defaults(void)
 static real
 image_distance_from(Image *image, Point *point)
 {
-  Element *elem = (Element *)image;
+  Element *elem = &image->element;
   Rectangle rect;
 
   rect.left = elem->corner.x - image->border_width;
@@ -322,6 +332,7 @@ static void
 image_move_handle(Image *image, Handle *handle,
 		  Point *to, HandleMoveReason reason, ModifierKeys modifiers)
 {
+  Element *elem = &image->element;
   assert(image!=NULL);
   assert(handle!=NULL);
   assert(to!=NULL);
@@ -343,12 +354,12 @@ image_move_handle(Image *image, Handle *handle,
       }
       to->x = image->element.corner.x+(image->element.width-new_width);
       to->y = image->element.corner.y+(image->element.height-new_height);
-      element_move_handle((Element *)image, HANDLE_RESIZE_NW, to, reason);
+      element_move_handle(elem, HANDLE_RESIZE_NW, to, reason);
       break;
     case HANDLE_RESIZE_N:
       new_width = (-(to->y-image->element.corner.y)+height)*width/height;
       to->x = image->element.corner.x+new_width;
-      element_move_handle((Element *)image, HANDLE_RESIZE_NE, to, reason);
+      element_move_handle(elem, HANDLE_RESIZE_NE, to, reason);
       break;
     case HANDLE_RESIZE_NE:
       new_width = to->x-image->element.corner.x;
@@ -360,12 +371,12 @@ image_move_handle(Image *image, Handle *handle,
       }
       to->x = image->element.corner.x+new_width;
       to->y = image->element.corner.y+(image->element.height-new_height);
-      element_move_handle((Element *)image, HANDLE_RESIZE_NE, to, reason);
+      element_move_handle(elem, HANDLE_RESIZE_NE, to, reason);
       break;
     case HANDLE_RESIZE_E:
       new_height = (to->x-image->element.corner.x)*height/width;
       to->y = image->element.corner.y+new_height;
-      element_move_handle((Element *)image, HANDLE_RESIZE_SE, to, reason);
+      element_move_handle(elem, HANDLE_RESIZE_SE, to, reason);
       break;
     case HANDLE_RESIZE_SE:
       new_width = to->x-image->element.corner.x;
@@ -377,12 +388,12 @@ image_move_handle(Image *image, Handle *handle,
       }
       to->x = image->element.corner.x+new_width;
       to->y = image->element.corner.y+new_height;
-      element_move_handle((Element *)image, HANDLE_RESIZE_SE, to, reason);
+      element_move_handle(elem, HANDLE_RESIZE_SE, to, reason);
       break;
     case HANDLE_RESIZE_S:
       new_width = (to->y-image->element.corner.y)*width/height;
       to->x = image->element.corner.x+new_width;
-      element_move_handle((Element *)image, HANDLE_RESIZE_SE, to, reason);
+      element_move_handle(elem, HANDLE_RESIZE_SE, to, reason);
       break;
     case HANDLE_RESIZE_SW:
       new_width = -(to->x-image->element.corner.x)+width;
@@ -394,12 +405,12 @@ image_move_handle(Image *image, Handle *handle,
       }
       to->x = image->element.corner.x+(image->element.width-new_width);
       to->y = image->element.corner.y+new_height;
-      element_move_handle((Element *)image, HANDLE_RESIZE_SW, to, reason);
+      element_move_handle(elem, HANDLE_RESIZE_SW, to, reason);
       break;
     case HANDLE_RESIZE_W:
       new_height = (-(to->x-image->element.corner.x)+width)*height/width;
       to->y = image->element.corner.y+new_height;
-      element_move_handle((Element *)image, HANDLE_RESIZE_SW, to, reason);
+      element_move_handle(elem, HANDLE_RESIZE_SW, to, reason);
       break;
     default:
       message_warning("Unforeseen handle in image_move_handle: %d\n",
@@ -407,7 +418,7 @@ image_move_handle(Image *image, Handle *handle,
       
     }
   } else {
-    element_move_handle((Element *)image, handle->id, to, reason);
+    element_move_handle(elem, handle->id, to, reason);
   }
   image_update_data(image);
 }
@@ -462,8 +473,9 @@ image_draw(Image *image, Renderer *renderer)
 static void
 image_update_data(Image *image)
 {
-  Element *elem = (Element *)image;
-  Object *obj = (Object *)image;
+  Element *elem = &image->element;
+  ElementBBExtras *extra = &elem->extra_spacing;
+  Object *obj = &elem->object;
 
   /* Update connections: */
   image->connections[0].pos = elem->corner;
@@ -481,13 +493,9 @@ image_update_data(Image *image)
   image->connections[6].pos.y = elem->corner.y + elem->height;
   image->connections[7].pos.x = elem->corner.x + elem->width;
   image->connections[7].pos.y = elem->corner.y + elem->height;
-
+  
+  extra->border_trans = image->border_width / 2.0;
   element_update_boundingbox(elem);
-  /* fix boundingbox for line_width: */
-  obj->bounding_box.top -= image->border_width;
-  obj->bounding_box.left -= image->border_width;
-  obj->bounding_box.bottom += image->border_width;
-  obj->bounding_box.right += image->border_width;
   
   obj->position = elem->corner;
   
@@ -505,14 +513,12 @@ static GtkFileSelection *dialog;
 static void
 image_browse_ok(GtkWidget *widget, gpointer data) {
   Image *image = (Image *)data;
-  Element *elem;
+  Element *elem = &image->element;
 
   image->file = gtk_file_selection_get_filename(dialog);
   image->image = dia_image_load(image->file);
 
   if (image->image) {
-    elem = (Element *) image;
-
     elem->width = (elem->width*(float)dia_image_width(image->image))/
       (float)dia_image_height(image->image);
   }
@@ -563,12 +569,11 @@ image_create(Point *startpoint,
   Object *obj;
   int i;
 
-  image = g_malloc(sizeof(Image));
-  elem = (Element *) image;
-  obj = (Object *) image;
+  image = g_malloc0(sizeof(Image));
+  elem = &image->element;
+  obj = &elem->object;
   
   obj->type = &image_type;
-
   obj->ops = &image_ops;
 
   elem->corner = *startpoint;
@@ -613,7 +618,7 @@ image_create(Point *startpoint,
   
   *handle1 = NULL;
   *handle2 = obj->handles[7];  
-  return (Object *)image;
+  return &image->element.object;
 }
 
 static void 
@@ -635,11 +640,11 @@ image_copy(Image *image)
   Element *elem, *newelem;
   Object *newobj;
   
-  elem = (Element *)image;
+  elem = &image->element;
   
-  newimage = g_malloc(sizeof(Image));
-  newelem = (Element *)newimage;
-  newobj = (Object *) newimage;
+  newimage = g_malloc0(sizeof(Image));
+  newelem = &newimage->element;
+  newobj = &newelem->object;
 
   element_copy(elem, newelem);
 
@@ -663,7 +668,7 @@ image_copy(Image *image)
   newimage->draw_border = image->draw_border;
   newimage->keep_aspect = image->keep_aspect;
 
-  return (Object *)newimage;
+  return &newimage->element.object;
 }
 
 /* Gets the directory path of a filename.
@@ -678,30 +683,28 @@ get_directory(const char *filename)
 {
   char *cwd;
   char *directory;
-  char *end;
+  char *dirname;
   int len;
   
   if (filename==NULL)
     return NULL;
 
-  if (g_path_is_absolute(filename)) {
-    end = strrchr(filename, G_DIR_SEPARATOR);
-    len = end - filename + 1;
-    directory = g_malloc((len+1)*sizeof(char));
-    memcpy(directory, filename, len);
-    directory[len] = 0; /* zero terminate string */
+  dirname = g_dirname(filename);
+  if (g_path_is_absolute(dirname)) {
+      len = strlen(dirname)+strlen(G_DIR_SEPARATOR_S)+1;
+      directory = g_malloc(len*sizeof(char));
+      strcpy(directory, dirname);
+      strcat(directory, G_DIR_SEPARATOR_S);
   } else {
-    cwd = g_get_current_dir();
-    len = strlen(cwd)+strlen(filename)+1;
-    directory = g_malloc(len*sizeof(char));
-    strncpy(directory, cwd, len);
-    strncat(directory, "/", len);
-    strncat(directory, filename, len);
-    g_free(cwd);
-    end = strrchr(directory, G_DIR_SEPARATOR);
-    if (end!=NULL)
-      *(end+1) = 0;
+      cwd = g_get_current_dir();
+      len = strlen(cwd)+strlen(dirname)+strlen(G_DIR_SEPARATOR_S)+1;
+      directory = g_malloc(len*sizeof(char));
+      strcpy(directory, cwd);
+      strcat(directory, G_DIR_SEPARATOR_S);
+      strcat(directory, dirname);
+      g_free(cwd);
   }
+  g_free(dirname);
 
   return directory;
 }
@@ -767,9 +770,9 @@ image_load(ObjectNode obj_node, int version, const char *filename)
   AttributeNode attr;
   char *diafile_dir;
   
-  image = g_malloc(sizeof(Image));
-  elem = (Element *)image;
-  obj = (Object *)image;
+  image = g_malloc0(sizeof(Image));
+  elem = &image->element;
+  obj = &elem->object;
   
   obj->type = &image_type;
   obj->ops = &image_ops;
@@ -898,6 +901,5 @@ image_load(ObjectNode obj_node, int version, const char *filename)
 
   image_update_data(image);
 
-  return (Object *)image;
-
+  return &image->element.object;
 }
