@@ -15,11 +15,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
 #include <gtk/gtk.h>
 #include <math.h>
 
-#include "config.h"
 #include "intl.h"
 #include "object.h"
 #include "polyshape.h"
@@ -171,15 +175,15 @@ static PropOffset polygon_offsets[] = {
 static void
 polygon_get_props(Polygon *polygon, Property *props, guint nprops)
 {
-  object_get_props_from_offsets((Object *)polygon, polygon_offsets,
-				props, nprops);
+  object_get_props_from_offsets(&polygon->poly.object, 
+                                polygon_offsets, props, nprops);
 }
 
 static void
 polygon_set_props(Polygon *polygon, Property *props, guint nprops)
 {
-  object_set_props_from_offsets((Object *)polygon, polygon_offsets,
-				props, nprops);
+  object_set_props_from_offsets(&polygon->poly.object, 
+                                polygon_offsets, props, nprops);
   polygon_update_data(polygon);
 }
 
@@ -314,9 +318,9 @@ polygon_create(Point *startpoint,
   init_default_values();
 
   /*polygon_init_defaults();*/
-  polygon = g_new(Polygon, 1);
+  polygon = g_new0(Polygon, 1);
   poly = &polygon->poly;
-  obj = (Object *) polygon;
+  obj = &poly->object;
 
   obj->type = &polygon_type;
   obj->ops = &polygon_ops;
@@ -340,7 +344,7 @@ polygon_create(Point *startpoint,
 
   *handle1 = poly->object.handles[0];
   *handle2 = poly->object.handles[2];
-  return (Object *)polygon;
+  return &polygon->poly.object;
 }
 
 static void
@@ -358,9 +362,9 @@ polygon_copy(Polygon *polygon)
 
   poly = &polygon->poly;
  
-  newpolygon = g_malloc(sizeof(Polygon));
+  newpolygon = g_malloc0(sizeof(Polygon));
   newpoly = &newpolygon->poly;
-  newobj = (Object *) newpolygon;
+  newobj = &newpoly->object;
 
   polyshape_copy(poly, newpoly);
 
@@ -371,7 +375,7 @@ polygon_copy(Polygon *polygon)
   newpolygon->inner_color = polygon->inner_color;
   newpolygon->show_background = polygon->show_background;
 
-  return (Object *)newpolygon;
+  return &newpolygon->poly.object;
 }
 
 
@@ -379,16 +383,13 @@ static void
 polygon_update_data(Polygon *polygon)
 {
   PolyShape *poly = &polygon->poly;
-  Object *obj = (Object *) polygon;
+  Object *obj = &poly->object;
+  PolyShapeBBExtras *extra = &poly->extra_spacing;
 
   polyshape_update_data(poly);
   
+  extra->border_trans = polygon->line_width / 2.0;
   polyshape_update_boundingbox(poly);
-  /* fix boundingbox for line_width: */
-  obj->bounding_box.top -= polygon->line_width/2;
-  obj->bounding_box.left -= polygon->line_width/2;
-  obj->bounding_box.bottom += polygon->line_width/2;
-  obj->bounding_box.right += polygon->line_width/2;
 
   obj->position = poly->points[0];
 }
@@ -433,10 +434,10 @@ polygon_load(ObjectNode obj_node, int version, const char *filename)
   Object *obj;
   AttributeNode attr;
 
-  polygon = g_malloc(sizeof(Polygon));
+  polygon = g_malloc0(sizeof(Polygon));
 
   poly = &polygon->poly;
-  obj = (Object *) polygon;
+  obj = &poly->object;
   
   obj->type = &polygon_type;
   obj->ops = &polygon_ops;
@@ -475,7 +476,7 @@ polygon_load(ObjectNode obj_node, int version, const char *filename)
 
   polygon_update_data(polygon);
 
-  return (Object *)polygon;
+  return &polygon->poly.object;
 }
 
 static ObjectChange *
