@@ -40,12 +40,15 @@ NOT WORKING (exporting macros):
 */
 
 
+#include <config.h>
+
 #include <string.h>
 #include <time.h>
 #include <math.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 
-#include "config.h"
 #include "intl.h"
 #include "render_pstricks.h"
 #include "message.h"
@@ -208,21 +211,29 @@ new_pstricks_renderer(DiagramData *data, const char *filename,
 	name = "a user";
   
     fprintf(file,
-	    "%% PSTricks TeX macro\n"
-	    "%% Title: %s\n"
-	    "%% Creator: Dia v%s\n"
-	    "%% CreationDate: %s"
-	    "%% For: %s\n"
-	    "%% \\usepackage{pstricks}\n"
-	    "%% Uncomment these if there is still no such functionality in PSTricks\n"
-	    "%% \\newcommand{\\setlinejoinmode}[1]{}\n"
-	    "%% \\newcommand{\\setlinecaps}[1]{}\n"
-	    "%% This way define your own fonts mapping (for example with ifthen)\n"
-	    "%% \\newcommand{\\setfont}[2]{}\n",
-	    diafilename,
-	    VERSION,
-	    ctime(&time_now),
-	    name);
+	"%% PSTricks TeX macro\n"
+	"%% Title: %s\n"
+	"%% Creator: Dia v%s\n"
+	"%% CreationDate: %s"
+	"%% For: %s\n"
+	"%% \\usepackage{pstricks}\n"
+	"%% The following commands are not supported in PSTricks at present\n"
+	"%% We define them conditionally, so when they are implemented,\n"
+	"%% this pstricks file will use them.\n"
+	"\\ifx\\setlinejoinmode\\undefined\n"
+	"  \\newcommand{\\setlinejoinmode}[1]{}\n"
+	"\\fi\n"
+	"\\ifx\\setlinecaps\\undefined\n"
+	"  \\newcommand{\\setlinecaps}[1]{}\n"
+	"\\fi\n"
+	"%% This way define your own fonts mapping (for example with ifthen)\n"
+	"\\ifx\\setfont\\undefined\n"
+	"  \\newcommand{\\setfont}[2]{}\n"
+        "\\fi\n",
+	diafilename,
+	VERSION,
+	ctime(&time_now),
+	name);
 
     fprintf(renderer->file,"\\pspicture(%f,%f)(%f,%f)\n",
 	    extent->left * data->paper.scaling,
@@ -731,11 +742,14 @@ draw_image(RendererPSTRICKS *renderer,
     /*  fprintf(renderer->file, "%f %f scale\n", 1.0, 1.0/ratio);*/
     fprintf(renderer->file, "grestore\n");
     fprintf(renderer->file, "}}\n");
+
+    g_free (rgb_data);
 }
 
 /* --- export filter interface --- */
 static void
-export_pstricks(DiagramData *data, const gchar *filename, const gchar *diafilename)
+export_pstricks(DiagramData *data, const gchar *filename, 
+                const gchar *diafilename, void* user_data)
 {
     RendererPSTRICKS *renderer;
 
