@@ -18,11 +18,15 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
+#include <config.h>
+
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
 #include <math.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #include <locale.h>
 
 #include <entities.h>
@@ -31,7 +35,6 @@
 #define childs children
 #endif
 
-#include "config.h"
 #include <tree.h>
 #include "geometry.h"
 #include "render.h"
@@ -194,8 +197,8 @@ new_svg_renderer(DiagramData *data, const char *filename)
   renderer->doc = xmlNewDoc("1.0");
   renderer->doc->standalone = FALSE;
   xmlCreateIntSubset(renderer->doc, "svg",
-		     "-//W3C//DTD SVG 20000303 Stylable//EN",
-		     "http://www.w3.org/TR/2000/03/WD-SVG-20000303/DTD/svg-20000303-stylable.dtd");
+		     "-//W3C//DTD SVG 20000802//EN",
+		     "http://www.w3.org/TR/2000/CR-SVG-20000802/DTD/svg-20000802.dtd");
   renderer->root = xmlNewDocNode(renderer->doc, NULL, "svg", NULL);
   renderer->doc->root = renderer->root;
 
@@ -209,7 +212,8 @@ new_svg_renderer(DiagramData *data, const char *filename)
   xmlSetProp(renderer->root, "height", buf);
   g_snprintf(buf, sizeof(buf), "%d %d %d %d",
 	     (int)floor(extent->left), (int)floor(extent->top),
-	     (int)ceil(extent->right), (int)ceil(extent->bottom));
+	     (int)ceil(extent->right - extent->left),
+	     (int)ceil(extent->bottom - extent->top));
   xmlSetProp(renderer->root, "viewBox", buf);
   
   time_now = time(NULL);
@@ -622,7 +626,7 @@ draw_ellipse(RendererSVG *renderer,
   g_snprintf(buf, sizeof(buf), "%g", width / 2);
   xmlSetProp(node, "rx", buf);
   g_snprintf(buf, sizeof(buf), "%g", height / 2);
-  xmlSetProp(node, "rx", buf);
+  xmlSetProp(node, "ry", buf);
 }
 
 static void
@@ -645,7 +649,7 @@ fill_ellipse(RendererSVG *renderer,
   g_snprintf(buf, sizeof(buf), "%g", width / 2);
   xmlSetProp(node, "rx", buf);
   g_snprintf(buf, sizeof(buf), "%g", height / 2);
-  xmlSetProp(node, "rx", buf);
+  xmlSetProp(node, "ry", buf);
 }
 
 static void
@@ -802,7 +806,8 @@ draw_image(RendererSVG *renderer,
 }
 
 static void
-export_svg(DiagramData *data, const gchar *filename, const gchar *diafilename)
+export_svg(DiagramData *data, const gchar *filename, 
+           const gchar *diafilename, void* user_data)
 {
   RendererSVG *renderer;
   char *old_locale;
